/******************************************************************************

         Copyright (c) 2020 MaxLinear, Inc.
         Copyright (c) 2020 Intel Corporation

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

*******************************************************************************/
#define PATH_SEPARATOR "."
#define DSL_CSP_SECTION_PATH "dsl"

/** \file dsl_cpe_interface.h
   \brief File contains the structure and function definitions for DSL functionality.
   The objects definitions are based on TR-181 definitions.
*/

/** \defgroup DSL_STATUS_PARSER DSL Status Parser
 * Lists the entire modules from the DSL CPE Status Parser */

/** \defgroup DSL_STATUS DSL Status
 * Lists the entire modules from the DSL CPE Status group
 * \ingroup DSL_STATUS_PARSER */
/** @{  */

/**
   Common DSL object indicating the number of available line entries.

   - JSON status path: dsl.line_number_of_entries
   - Parameter type: Integer

   This value indicates the number of available lines which are operated by the
   running instance of the DSL daemon respective dsl_cpe_control.

   It also defines how many line objects are available within JSON status file
   (/tmp/status/dsl) for the following parent objects
   - dsl.status.line.<#line_idx>
   - dsl.stats.line.<#line_idx>

   \note As the line index starts at zero, the max. number of <#line_idx> will
         be  line_number_of_entries - 1.*/
#define DSL_CSP_STAT_LINE_NUMBER_OF_ENTRIES "line_number_of_entries"

/**
   Common DSL object indicating the number of available channel entries.

   - JSON status path: dsl.channel_number_of_entries
   - Parameter type: Integer

   This value indicates the number of available channels which are operated by
   the running instance of the DSL daemon respective dsl_cpe_control.
   The channel object is stacked on top of each line object.
   It also defines how many channel objects are available within JSON status
   file (/tmp/status/dsl) for the following parent objects
   - dsl.status.line.<#line_idx>.channel.<#channel_idx>
   - dsl.stats.line.<#line_idx>.channel.<#channel_idx>

   \note As the channel index starts at zero, the max. number of <#channel_idx>
         will be  channel_number_of_entries - 1.

   \note There is no support for dual (or higher) latency path. Therefore the
         currently returned value will be always 1. */
#define DSL_CSP_STAT_CHANNEL_NUMBER_OF_ENTRIES "channel_number_of_entries"

/**
   Common DSL object indicating the number of available bonding group entries.

   - JSON status path: dsl.bonding_group_number_of_entries
   - Parameter type: Integer

   This parameter is currently not supported and will therefore always
   return 0. */
#define DSL_CSP_STAT_BONDING_GROUP_NUMBER_OF_ENTRIES "bonding_group_number_of_entries"

/**
   Common DSL object indicating the version number of the DSL API.

   - JSON status path: dsl.api_version
   - Parameter type: String

   This parameter indicates the version of the running DSL API instance (DSL
   daemon respective dsl_cpe_control). The version number is formated as
   follows.

       major.minor.build[.maint[.maint[...]]][-pa#][-pd#]

   - major: Major version, incremented for new versions with significant
            updates, for example supporting a new chip type, etc.
   - minor: Minor version, incremented for feature steps
   - build: Build step, incremented regularly for new SW versions within same
            feature step.
   - maint: Maintenance version. The availability of more than three digits
            indicates that a maintenance SW version is operated. Such versions
            typically do not get any feature updates but only bug fixes. The
            version is therefore generated from a dedicated maintenance branch
            where the first three digits of the version number defines the
            branch-off point.
            There might be additional maintenance version digits available which
            indicates that an additional branch was used, where the branch-off
            point is again defined by the "higher priority" version digits.

   \note There might be patch level indications available which are added
         tailing the version number. The format of the patch level is defined
         as follows
         Application patches: -pa<#patch_level_number>
         Driver patches: -pd<#patch_level_number>
         An example of a resulting API version could be: V4.21.6-pa1pd2 */
#define DSL_CSP_STAT_API_VERSION "api_version"

/**
   Common DSL object indicating the version number of the low level Driver.

   - JSON status path: dsl.mei_driver_version
   - Parameter type: String

   This parameter indicates the version of the running low level driver, which
   is the MEI Driver for VRX500 (and before) and the PMI Driver for VRX600.
   The version number is formated in the same way and the same rules apply
   accordingly as for the API version (\ref DSL_CSP_STAT_API_VERSION). */
#define DSL_CSP_STAT_LL_MEI_DRIVER_VERSION "mei_driver_version"
#define DSL_CSP_STAT_LL_PMI_DRIVER_VERSION "pmi_driver_version"

/**
   Common DSL object indicating the hardware version number.

   - JSON status path: dsl.hardware_version
   - Parameter type: String

   This parameter returns a string which indicates the the hardware chip
   version, for example "VRX-DFE_V2.1". */
#define DSL_CSP_STAT_HARDWARE_VERSION "hardware_version"

/**
   Common DSL object indicating the chipset type.

   - JSON status path: dsl.chipset_type
   - Parameter type: String

   This parameter returns a string which indicates the the chipset type, for
   example "Lantiq-VRX500". */
#define DSL_CSP_STAT_CHIPSET_TYPE "chipset_type"

/**
   The current operational state of the DSL line (see [Section 4.2.2/TR-181i2]).

   - JSON status path: dsl.status.line.<#line_idx>.status
   - Parameter type: String

   This parameter is based on ifOperStatus from [RFC2863].

   Defined valid strings
   - "Up"
   - "Down"
   - "Unknown" (not used)
   - "Dormant" (not used)
   - "NotPresent" (not used)
   - "LowerLayerDown" (not used)
   - "Error" */
#define DSL_CSP_STAT_LINE_STATUS "status"

/**
   Indicates whether the interface points towards the Internet or towards End Devices.

   - JSON status path: dsl.status.line.<#line_idx>.upstream
   - Parameter type: Boolean

   Indicates whether the interface points towards the Internet (true) or towards
   End Devices (false).

   \note This parameter is always returned as true */
#define DSL_CSP_STAT_UPSTREAM "upstream"

/**
   A string identifying the version of the modem firmware currently operated.

   - JSON status path: dsl.status.line.<#line_idx>.firmware_version
   - Parameter type: String

   A string identifying the version of the modem firmware currently operated
   for this interface. This is applicable only when the modem firmware is
   separable from the overall CPE software */
#define DSL_CSP_STAT_FIRMWARE_VERSION "firmware_version"

/** Status of the DSL physical link.

   - JSON status path: dsl.status.line.<#line_idx>.link_status
   - Parameter type: String

   Defined valid strings
   - "Up"
   - "Initializing"
   - "EstablishingLink"
   - "NoSignal" (not used, upper layer handling)
   - "Disabled"
   - "Error" */
#define DSL_CSP_STAT_LINK_STATUS "link_status"

/**
   Indicates which DSL standards and recommendations are supported.

   - JSON status path: dsl.status.line.<#line_idx>.standards_supported
   - Parameter type: String

   Comma-separated list of strings indicating which DSL standards and
   recommendations are supported by the Line instance.

   Defined valid enumeration strings
   - "T1.413"
   - "ETSI_101_388"
   - "G.992.1_Annex_A"
   - "G.992.1_Annex_B"
   - "G.992.1_Annex_C" (not supported)
   - "G.992.2_Annex_A"
   - "G.992.2_Annex_B" (not supported)
   - "G.992.2_Annex_C" (not supported)
   - "G.992.3_Annex_A"
   - "G.992.3_Annex_B"
   - "G.992.4_Annex_A"
   - "G.992.3_Annex_I"
   - "G.992.3_Annex_J"
   - "G.992.4_Annex_I" (not supported)
   - "G.992.3_Annex_L"
   - "G.992.3_Annex_M"
   - "G.992.5_Annex_A"
   - "G.992.5_Annex_B"
   - "G.992.5_Annex_I"
   - "G.992.5_Annex_J"
   - "G.992.5_Annex_M"
   - "G.993.2_Annex_A"
   - "G.993.2_Annex_B"
   - "G.993.2_Annex_C"

   \note In G.997.1, this parameter is called "xDSL Transmission system
         capabilities". See ITU-T Recommendation [G.997.1]. */
#define DSL_CSP_STAT_STANDARDS_SUPPORTED "standards_supported"

/**
   This status parameter provides the allowed transmission system types.

   - JSON status path: dsl.status.line.<#line_idx>.xtse
   - Parameter type: Array of strings

   This status parameter provides the allowed transmission system types
   by the xTU on this Line instance.
   The array of comma separated strings which are provided, includes a bitmask
   in hexadecimal representation.
   Each bit is 0 if not allowed or 1 if allowed, as defined for the xTU
   transmission system enabling (XTSE) in ITU-T G.997.1.

   \anchor XTSE
   <table border="1">
   <caption>Definition of XTSE Octet Bits</caption>
   <tr><th></th>           <th>Bit 8</th>
                                       <th>Bit 7</th>
                                                   <th>Bit 6</th>
                                                               <th>Bit 5</th>
                                                                           <th>Bit 4</th>
                                                                                       <th>Bit 3</th>
                                                                                                   <th>Bit 2</th>
                                                                                                               <th>Bit 1</th></tr>
   <tr><th>Octet 1</th>    <td></td>   <td></td>   <td></td>   <td>G992.1<br>B<br>ADSL1</td>
                                                                           <td></td>   <td>G992.1<br>A<br>ADSL1</td>
                                                                                                   <td></td>   <td>ANSI T1.413</td></tr>
   <tr><th>Octet 2</th>    <td></td>   <td></td>   <td></td>   <td></td>   <td></td>   <td></td>   <td></td>   <td></td></tr>
   <tr><th>Octet 3</th>    <td></td>   <td></td>   <td></td>   <td>G992.3<br>B<br>ADSL2</td>
                                                                           <td></td>   <td>G992.3<br>A<br>ADSL2</td>
                                                                                                   <td></td>   <td></td></tr>
   <tr><th>Octet 4</th>    <td></td>   <td>G992.3<br>J<br>ADSL2</td>
                                                   <td></td>   <td></td>   <td></td>   <td></td>   <td></td>   <td></td></tr>
   <tr><th>Octet 5</th>    <td></td>   <td></td>   <td></td>   <td></td>   <td></td>   <td></td>   <td></td>   <td></td></tr>
   <tr><th>Octet 6</th>    <td></td>   <td></td>   <td></td>   <td></td>   <td></td>   <td>G992.5<br>B<br>ADSL2+</td>
                                                                                                   <td></td>   <td>G992.5<br>A<br>ADSL2+</td></tr>
   <tr><th>Octet 7</th>    <td></td>   <td></td>   <td></td>   <td></td>   <td></td>   <td></td>   <td></td>   <td>G992.5<br>J<br>ADSL2+</td></tr>
   <tr><th>Octet 8</th>    <td></td>   <td></td>   <td></td>   <td></td>   <td></td>   <td>G993.2<br>C<br>VDSL</td>
                                                                                                   <td>G993.2<br>B<br>VDSL</td>
                                                                                                               <td>G993.2<br>A<br>VDSL</td></tr>
   <tr><th>Octet 9</th>    <td>G.9701<br>FAST</td>
                                       <td></td>   <td>G993.5<br>C<br>Vect. friendly</td>
                                                               <td>G993.5<br>B<br>Vect. friendly</td>
                                                                           <td>G993.5<br>A<br>Vect. friendly</td>
                                                                                       <td>G993.5<br>C<br>Vectoring</td>
                                                                                                   <td>G993.5<br>B<br>Vectoring</td>
                                                                                                               <td>G993.5<br>A<br>Vectoring</td></tr>
   </table>

   \note This parameter is defined as xDSL Transmission system enabling (XTSE)
         in ITU-T G.997.1. For a full definition, see Clause 7.3.1.1.1 of ITU-T
         Recommendation [G.997.1].

   \note You might also refer to the chapter "G.997.1 xTU System Enabling Octets"
         within (UM)PR of the VRX518 to get an easy human readable overview
         about all defined bits within octets that are defined by the G.997.1
         */
#define DSL_CSP_STAT_XTSE "xtse"

/**
   Indicates the standard that the Line instance is using for the connection.

   - JSON status path: dsl.status.line.<#line_idx>.standard_used
   - Parameter type: String

   The value will be a member of the list reported by the
   \ref standards_supported parameter.

   \note In G.997.1, this parameter is called "xDSL Transmission system".
         See ITU-T Recommendation [G.997.1]. */
#define DSL_CSP_STAT_STANDARD_USED "standard_used"

/**
   Indicates the transmission system type that is currently used by the Line instance.

   - JSON status path: dsl.status.line.<#line_idx>.xtse_used
   - Parameter type: Array of strings

   The array of comma separated strings which are provided, includes a bitmask
   in hexadecimal representation. It indicates the transmission system type that
   is currently in use by the Line instance.
   Only one bit is set, as defined for the xTU transmission system enabling
   (XTSE) in ITU-T G.997.1.

   See \ref  XTSE table for bit definitions.

   \note You might refer to the chapter "G.997.1 xTU System Enabling Octets"
         within (UM)PR of the VRX518 to get an easy human readable overview
         about all defined bits within octets that are defined by the G.997.1 */

#define DSL_CSP_STAT_XTSE_USED "xtse_used"

/**
   Indicates the line encoding method used to establish the Layer 1 DSL connection.

   - JSON status path: dsl.status.line.<#line_idx>.line_encoding
   - Parameter type: String

   The line encoding method used in establishing the Layer 1 DSL connection
   between the CPE and the DSLAM.

   \note This parameter is always returned as "DMT" */
#define DSL_CSP_STAT_LINE_ENCODING "line_encoding"

/**
   Indicates which VDSL2 profiles are allowed on the line.

   - JSON status path: dsl.status.line.<#line_idx>.allowed_profiles
   - Parameter type: String

    Comma-separated list of strings indicating which VDSL2 profiles are allowed
    on the line.

    Defined valid enumeration strings
   - "8a"
   - "8b"
   - "8c"
   - "8d"
   - "12a"
   - "12b"
   - "17a"
   - "30a"
   - "35b"

   \note In G.997.1, this parameter is called PROFILES. See ITU-T Recommendation
         [G.997.1]. */
#define DSL_CSP_STAT_ALLOWED_PROFILES "allowed_profiles"

/**
   Indicates the VDSL2 profile that the Line instance is using for the connection.

   - JSON status path: dsl.status.line.<#line_idx>.current_profile
   - Parameter type: String

   The value will be a either a member of the list reported by the parameter
   \ref allowed_profiles or an empty string. */
#define DSL_CSP_STAT_CURRENT_PROFILE "current_profile"

/** Indicates the power management state of the line.

   - JSON status path: dsl.status.line.<#line_idx>.power_management_state
   - Parameter type: String

   Defined valid strings
   - "L0"
   - "L1"
   - "L2"
   - "L3"
   - "L4" (not supported)

   \note See ITU-T Recommendation [G.997.1, chapter 7.5.1.2]. */
#define DSL_CSP_STAT_POWER_MANAGEMENT_STATE "power_management_state"

/**
   Indicates the success/failure cause of the initialization.

   - JSON status path: dsl.status.line.<#line_idx>.success_failure_cause
   - Parameter type: Integer

   Defined valid strings
   - 0: Successful
   - 1: Configuration error
        This error occurs with inconsistencies in configuration parameters,
        e.g. when the line is initialized in an xDSL Transmission system where
        an xTU does not support the configured Maximum Delay or the configured
        Minimum or Maximum Data Rate for one or more bearer channels
   - 2: Configuration not feasible on the line
        This error occurs if the Minimum Data Rate cannot be reached on the
        line with the Minimum Noise Margin, Maximum PSD level, Maximum Delay
        and Maximum Bit Error Ratio for one or more bearer channels
   - 3: Communication problem
        This error occurs, for example, due to corrupted messages or bad syntax
        messages or if no common mode can be selected in the G.994.1
        handshaking procedure or due to a timeout
   - 4: No peer xTU detected
        This error occurs if the peer xTU is not powered or not connected or if
        the line is too long to allow detection of a peer xTU
   - 5: Any other or unknown Initialization Failure cause

   \note In G.997.1, this parameter is called "Initialization success/failure
         cause". See ITU-T Recommendation [G.997.1, (see chapter 7.5.1.3]. */
#define DSL_CSP_STAT_SUCCESS_FAILURE_CAUSE "success_failure_cause"

/**
   This parameter represents the last successful transmitted initialization state.

   - JSON status path: dsl.status.line.<#line_idx>.last_state_transmitted
   - Parameter type: Integer

   This parameter represents the last successful transmitted initialization
   state in the last full initialization performed on the line.
   Initialization states are defined in the individual xDSL Recommendations and
   are counted from 0 (if G.994.1 is used) or 1 (if G.994.1 is not used) up to
   Showtime.
   This parameter needs to be interpreted along with the xDSL Transmission System.
   This parameter is available only when, after a failed full initialization,
   the line diagnostics procedures are activated on the line.

   This parameter is provided both for upstream (us) as well as downstream (ds)
   - last_state_transmitted.us
   - last_state_transmitted.ds

   \note The Line diagnostics procedures can be activated by the operator of the
         system (through the Line State Forced line configuration parameter) or
         autonomously by the xTU-C or xTU-R.

   \note See ITU-T Recommendation [G.997.1]. */
#define DSL_CSP_STAT_LAST_STATE_TRANSMITTED "last_state_transmitted"

/**
   Indicates the allowed VDSL2 US0 PSD masks for Annex A operation.

   - JSON status path: dsl.status.line.<#line_idx>.us0_mask
   - Parameter type: String

   The string which is provided, includes a bitmask in hexadecimal representation.
   as specified in ITU-T Recommendation G.997.1.

   \note For a VDSL2-capable multimode device operating in a mode other
         than VDSL2, the value of this parameter SHOULD be set to 0.

   \note See ITU-T Recommendation [G.997.1]. */
#define DSL_CSP_STAT_US0_MASK "us0_mask"

/**
   Reports whether trellis coding is enabled or not.

   - JSON status path: dsl.status.line.<#line_idx>.trellis
   - Parameter type: Integer

   Indicates whether the trellis coding is used (1) or not used (0).

   This parameter is provided both for upstream (us) as well as downstream (ds)
   - trellis.us for upstream direction
   - trellis.ds for downstream direction

   \note For a multimode device operating in a mode in which this parameter
         does not apply, the value of this parameter SHOULD be set to -1.

   \note See ITU-T Recommendation [G.997.1] */
#define DSL_CSP_STAT_TRELLIS "trellis"

/**
   Reports whether the OPTIONAL virtual noise mechanism is in use.

   - JSON status path: dsl.status.line.<#line_idx>.act_snr_mode
   - Parameter type: String

   Indicates whether the virtual noise mechanism is in use (2) or not is use (1).

   This parameter is provided both for upstream (us) as well as downstream (ds)
   - act_snr_mode.us
   - act_snr_mode.ds

   \note For a multimode device operating in a mode in which this parameter
         does not apply, the value of this parameter SHOULD be set to 0.

   \note See ITU-T Recommendation [G.997.1]. */
#define DSL_CSP_STAT_ACT_SNR_MODE "act_snr_mode"

/**
   Signifies the line pair that the modem is using to be connected.

   - JSON status path: dsl.status.line.<#line_idx>.line_number
   - Parameter type: Integer

    If this value equals 1 the innermost pair is used to connect the line
    instance. */
#define DSL_CSP_STAT_LINE_NUMBER "line_number"

/**
   The current maximum attainable data rate (expressed in Kbps).

   - JSON status path: dsl.status.line.<#line_idx>.max_bit_rate
   - Parameter type: Integer

   This parameter is provided both for upstream (us) as well as downstream (ds)
   - max_bit_rate.us
   - max_bit_rate.ds

   \note This parameter is related to the G.997.1 parameter ATTNDRus, which is
         measured in bits/s.

   \note See ITU-T Recommendation [G.997.1]. */
#define DSL_CSP_STAT_MAX_BIT_RATE "max_bit_rate"

/**
   Indicates the current signal-to-noise ratio margin (expressed in 0.1dB).

   - JSON status path: dsl.status.line.<#line_idx>.noise_margin
   - Parameter type: Integer

   This parameter is provided both for upstream (us) as well as downstream (ds)
   - noise_margin.us
   - noise_margin.ds

   \note In G.997.1, this parameter is called SNRMus.

   \note See ITU-T Recommendation [G.997.1]. */
#define DSL_CSP_STAT_NOISE_MARGIN "noise_margin"

/**
   Indicates the current signal-to-noise ratio margin per band.

   - JSON status path: dsl.status.line.<#line_idx>.snr_mpb
   - Parameter type: String

   This parameter returns a string which includes a comma-separated list of
   integer values (maximum list length is 24 integer values).

   This parameter is provided both for the upstream bands (us) as well as for
   the downstream bands (ds)
   - snr_mpb.us
   - snr_mpb.ds

   \note Interpretation of the values is as defined in ITU-T Rec. G.997.1.

   \note See ITU-T Recommendation [G.997.1]. */
#define DSL_CSP_STAT_SNR_MPB "snr_mpb"

/**
   Indicates the current output or received power at the CPE's DSL line.

   - JSON status path: dsl.status.line.<#line_idx>.power
   - Parameter type: Integer

   This parameter is provided both for upstream (us) which is related to the
   output power as well as for the downstream (ds) which related to the received
   power, both expressed in 0.1dBmV
   - power.us
   - power.ds */
#define DSL_CSP_STAT_POWER "power"

/**
   Indicates the xTU-R vendor identifier as defined in G.994.1 and T1.413.

   - JSON status path: dsl.status.line.<#line_idx>.xtur_vendor
   - Parameter type: Array of strings

   An array of four comma separated strings including hexadecimal values is
   returned.
   In the case of G.994.1 this corresponds to the four-octets provider code,
   which will be represented as eight hexadecimal digits.

   \note This parameter is optional at the G and S/T interfaces in G.997.1
         Amendment 1.

   \note If the parameter is implemented but no value is available,
         it will have the value "0x00,0x00,0x00,0x00".

   \note In G.997.1, this parameter is called "xTU-R G.994.1 Vendor ID".

   \note See ITU-T Recommendation [G.997.1]. */
#define DSL_CSP_STAT_XTUR_VENDOR "xtur_vendor"

/**
   Indicates the T.35 country code of the xTU-R vendor as defined in G.994.1.

   - JSON status path: dsl.status.line.<#line_idx>.xtur_country
   - Parameter type: Array of strings

   An array of two comma separated strings including hexadecimal values is
   returned.
   The two-octet values defined in G.994.1 will be represented as four
   hexadecimal digits.

   \note This parameter is OPTIONAL at the G and S/T interfaces in G.997.1
         Amendment 1.

   \note If the parameter is implemented but no value is available,
         it MUST have the value "0x00,0x00".

   \note In G.997.1, this parameter is called "xTU-R G.994.1 Vendor ID".

   \note See ITU-T Recommendation [G.997.1]. */
#define DSL_CSP_STAT_XTUR_COUNTRY "xtur_country"

/**
   Indicates the xTU-R T1.413 Revision Number as defined in T1.413 Issue 2.

   - JSON status path: dsl.status.line.<#line_idx>.xtur_ansi_std
   - Parameter type: Integer

   \note When T1.413 modulation is not in use, the parameter value will be
         set to 0. */
#define DSL_CSP_STAT_XTUR_ANSI_STD "xtur_ansi_std"

/**
   Indicates the xTU-R Vendor Revision Number as defined in T1.413 Issue 2.

   - JSON status path: dsl.status.line.<#line_idx>.xtur_ansi_rev
   - Parameter type: Integer

   \note When T1.413 modulation is not in use, the parameter value will be
         set to 0. */
#define DSL_CSP_STAT_XTUR_ANSI_REV "xtur_ansi_rev"

/**
   Indicates the xTU-C vendor identifier as defined in G.994.1 and T1.413.

   - JSON status path: dsl.status.line.<#line_idx>.xtuc_vendor
   - Parameter type: Integer

   An array of four comma separated strings including hexadecimal values is
   returned.
   In the case of G.994.1 this corresponds to the four-octet provider code,
   which will be represented as eight hexadecimal digits.

   \note This parameter is OPTIONAL at the G and S/T interfaces in G.997.1
         Amendment 1.
   \note If the parameter is implemented but no value is available,
         it MUST have the value "0x00,0x00,0x00,0x00". */
#define DSL_CSP_STAT_XTUC_VENDOR "xtuc_vendor"

/**
   Indicates the T.35 country code of the xTU-C vendor as defined in G.994.1,

   - JSON status path: dsl.status.line.<#line_idx>.xtuc_country
   - Parameter type: Array of strings

   An array of two comma separated strings including hexadecimal values is
   returned.
   According to the G.994.1 these value will be represented by four hexadecimal
   digits.

   \note This parameter is optional at the G and S/T interfaces in G.997.1
         Amendment 1.

   \note If the parameter is implemented but no value is available, it will have
         the value "0x00,0x00". */
#define DSL_CSP_STAT_XTUC_COUNTRY "xtuc_country"

/**
   Indicates the xTU-C T1.413 Revision Number as defined in T1.413 Issue 2.

   - JSON status path: dsl.status.line.<#line_idx>.xtuc_ansi_std
   - Parameter type: Integer

   \note When T1.413 modulation is not in use, the parameter value will be
         set to 0. */
#define DSL_CSP_STAT_XTUC_ANSI_STD "xtuc_ansi_std"

/**
   Indicates the xTU-C Vendor Revision Number as defined in T1.413 Issue 2.

   - JSON status path: dsl.status.line.<#line_idx>.xtuc_ansi_rev
   - Parameter type: Integer

   \note When T1.413 modulation is not in use, the parameter value will be
         set to 0. */
#define DSL_CSP_STAT_XTUC_ANSI_REV "xtuc_ansi_rev"

/**
   Indicates the VTU-R estimated upstream power back-off electrical length.

   - JSON status path: dsl.status.line.<#line_idx>.upbokler
   - Parameter type: Integer

   This parameter contains the estimated electrical loop length estimated by the
   VTU-R expressed in 0.1 dB at 1MHz (see O-UPDATE in section 12.2.4.2.1.2/G.993.2).
   The value is coded as an unsigned Intrger in the range 0 (coded as 0) to
   128 dB (coded as 1280) in steps of 0.1 dB.

   \note This parameter is defined as UPBOKLE-R in Clause 7.5.1.23.2 of
         ITU-T Recommendation [G.997.1]. */
#define DSL_CSP_STAT_UPBOKLER "upbokler"

/**
   Indicates the VTU-R estimated upstream power back-off electrical length per band.

   - JSON status path: dsl.status.line.<#line_idx>.upbokler_pb
   - Parameter type: Integer

   An array of comma-separated unsigned integers values (0 to 1280, or 2047) is
   returned.
   This parameter is a vector of UPBO electrical length per-band estimates for
   each supported downstream band, expressed in 0.1 dB at 1 MHz (kl0) calculated
   by the VTU-R, based on separate measurements in the supported downstream bands.
   The value of each list element ranges from 0 to 128 dB in steps of 0.1 dB,
   with special value 204.7 which indicates that the estimate is greater than
   128 dB.
   This parameter is required for the alternative electrical length estimation
   method (ELE-M1). The value of each list element is coded as an unsigned
   16 bit number in the range 0 (coded as 0) to 128 dB (coded as 1280)
   in steps of 0.1 dB.

   \note This parameter is defined as UPBOKLE-R-pb in Clause 7.5.1.23.4 of
         ITU-T Recommendation [G.997.1]. */
#define DSL_CSP_STAT_UPBOKLER_PB "upbokler_pb"

/**
   Indicates the UPBO downstream receiver signal level threshold.

   - JSON status path: dsl.status.line.<#line_idx>.rxthrsh_ds
   - Parameter type: Integer

   An array of comma-separated unsigned integers values (0 to 1280, or 2047) is
   returned.
   This parameter reports the downstream received signal level threshold value
   used in the alternative electrical length estimation method (ELE-M1).
   This parameter represents an offset from -100 dBm/Hz, and ranges from -64 dB
   to 0 dB in steps of 0.1 dB.
   The value is coded as a 16 bit number in the range -64 (coded as -640) to
   0 dB (coded as 0) in steps of 0.1 dB.

   \note See ITU-T Recommendation [G.997.1]. */
#define DSL_CSP_STAT_RXTHRSH_DS "rxthrsh_ds"

/**
   Indicates the actual active rate adaptation mode.

   - JSON status path: dsl.status.line.<#line_idx>.act_ra_mode
   - Parameter type: Integer

   Defined valid values
   - 2: the link is operating in RA-MODE 2 (AT_INIT).
   - 3: the link is operating in RA-MODE 3 (DYNAMIC).
   - 4: the link is operating in RA-MODE 4 (DYNAMIC with SOS).

   This parameter is provided both for upstream (us) as well as downstream (ds)
   - act_ra_mode.us
   - act_ra_mode.ds

   \note This parameter is defined as ACT-RA-MODEds in Clause 7.5.1.33.1 of
         ITU-T Recommendation [G.997.1]. */
#define DSL_CSP_STAT_ACT_RA_MODE "act_ra_mode"

/**
   Indicates the actual signal-to-noise margin of the robust overhead channel (ROC)

   - JSON status path: dsl.status.line.<#line_idx>.snr_mroc
   - Parameter type: Integer

   This value is expressed in 0.1 dB. The format is identical to the format of
   the line status parameter SNRM margin (noise_margin).

   This parameter is provided both for upstream (us) as well as downstream (ds)
   - snr_mroc.us
   - snr_mroc.ds

   \note This parameter is defined as SNRM-ROC-us in Clause 7.5.1.35.2 ITU-T
         Recommendation [G.997.1]. */
#define DSL_CSP_STAT_SNR_MROC "snr_mroc"

/**
   The current upstream or downstream signal loss.

   - JSON status path: dsl.status.line.<#line_idx>.attenuation
   - Parameter type: Integer

   This value is expressed in 0.1 dB.

   This parameter is provided both for upstream (us) as well as downstream (ds)
   - attenuation.us
   - attenuation.ds */
#define DSL_CSP_STAT_ATTENUATION "attenuation"

/**
   Micro filter detection result from last initialization measurement.

   - JSON status path: dsl.status.line.<#line_idx>.mfd_basic_data.filter_detection
   - Parameter type: Integer
   - Platform restriction: VRX518 (and before)

   Reports the results of the micro filter detection (MFD) from last
   initialization measurement.

   Defined valid values
   - 0: Microfilter detection not possible
   - 1: No microfilter detected
   - 2: Microfilter detected
   - 3: Measurement not done yet or not available in current operation state

   \note This value is only available in ADSL showtime, otherwise it will be
         set to 3. */
#define DSL_CSP_STAT_FILTER_DETECTION "filter_detection"

/**
   Metric3 is used to conclude on microfilter existence.

   - JSON status path: dsl.status.line.<#line_idx>.mfd_basic_data.metric3
   - Parameter type: Signed Integer
   - Platform restriction: VRX518 (and before)

   This parameter is used (amongst further parameters) to conclude on the
   existence of microfilters.

   Supported range: -256 to 255
   Special value 0x7FFF (32767 decimal) indicates that measurement has not be done

   Metric3 > 0 : microfilter missing
   Metric3 < 0 : microfilter present

   A higher value indicates higher detection accuracy

   \note This value is only applicable if measurement has been done
         in current operation state ("filter_detection" < 3). */
#define DSL_CSP_STAT_METRIC3 "metric3"

/**
   The current operational state of the DSL channel.

   - JSON status path: dsl.status.line.<#line_idx>.channel.<#channel_idx>.status
   - Parameter type: String

   This parameter is based on ifOperStatus from [RFC2863].

   Defined valid strings
   - "Up"
   - "Down"
   - "Unknown" (not used)
   - "Dormant" (not used)
   - "NotPresent" (not used)
   - "LowerLayerDown" (not used)
   - "Error"

   \note see [Section 4.2.2/TR-181i2]. */
#define DSL_CSP_STAT_CHANNEL_STATUS "status"

/**
   Indicates which link encapsulation standards and recommendations are supported
   by the channel instance.

   - JSON status path:
     dsl.status.line.<#line_idx>.channel.<#channel_idx>.link_encapsulation_supported
   - Parameter type: String

   Comma-separated list of strings indicating which standards and recommendations
   are supported by the channel instance.

   Defined valid enumeration strings
   - G.992.3_Annex_K_ATM
   - G.992.3_Annex_K_PTM
   - G.993.2_Annex_K_ATM
   - G.993.2_Annex_K_PTM
   - G.994.1 (Auto) */
#define DSL_CSP_STAT_LINK_ENCAPSULATION_SUPPORTED "link_encapsulation_supported"

/**
   Indicates the link encapsulation standard that the Channel instance is using
   for the connection.

   - JSON status path:
     dsl.status.line.<#line_idx>.channel.<#channel_idx>.link_encapsulation_used
   - Parameter type: String

   The value will be a member of the list reported by the
   \ref link_encapsulation_supported parameter. */
#define DSL_CSP_STAT_LINK_ENCAPSULATION_USED "link_encapsulation_used"

/**
   Indicates the index of the latency path supporting the bearer channel.

   - JSON status path: dsl.status.line.<#line_idx>.channel.<#channel_idx>.lpath
   - Parameter type: Integer

   \note See ITU-T Recommendation [Section 7.5.2.7/G.997.1]. */
#define DSL_CSP_STAT_LPATH "lpath"

/**
   Indicates the interleaver depth D for the latency path indicated in \ref lpath.

   - JSON status path: dsl.status.line.<#line_idx>.channel.<#channel_idx>.intlvdepth
   - Parameter type: Integer

   \note See ITU-T Recommendation [G.997.1].

   \note For a multimode device operating in a mode in which this parameter
         does not apply, the value of this parameter will be set to 0. */
#define DSL_CSP_STAT_INTLVDEPTH "intlvdepth"

/**
   Indicates the interleaver block length in use on the latency path indicated
   in \ref lpath.

   - JSON status path: dsl.status.line.<#line_idx>.channel.<#channel_idx>.intlvblock
   - Parameter type: Integer

   \note See ITU-T Recommendation [G.997.1].

   \note For a multimode device operating in a mode in which this parameter
         does not apply, the value of this parameter SHOULD be set to -1. */
#define DSL_CSP_STAT_INTLVBLOCK "intlvblock"

/**
   Indicates the actual delay, in milliseconds, of the latency path due to
   interleaving.

   - JSON status path:
     dsl.status.line.<#line_idx>.channel.<#channel_idx>.actual_interleaving_delay
   - Parameter type: Integer

   \note See ITU-T Recommendation [G.997.1]. */
#define DSL_CSP_STAT_ACTUAL_INTERLEAVING_DELAY "actual_interleaving_delay"

/**
   Indicates the actual impulse noise protection (INP) provided by the latency
   path indicated in \ref lpath.

   - JSON status path: dsl.status.line.<#line_idx>.channel.<#channel_idx>.actinp
   - Parameter type: Integer

   The value is the actual INP in the L0 (i.e., Showtime) state.

   \note See ITU-T Recommendation [G.997.1].

   \note For a multimode device operating in a mode in which this parameter does
         not apply, the value of this parameter SHOULD be set to -1. */
#define DSL_CSP_STAT_ACTINP "actinp"

/**
   Indicates if the value reported in actinp was computed.

   - JSON status path: dsl.status.line.<#line_idx>.channel.<#channel_idx>.inpreport
   - Parameter type: Integer

   Indicates whether the value reported in \ref actinp was computed assuming the
   receiver does not use erasure decoding.

   Defined valid values:
   - 0: computed per the formula assuming no erasure decoding
   - 1: computed by taking into account erasure decoding capabilities of receiver

   \note See ITU-T Recommendation [G.997.1].

   \note For a multimode device operating in a mode in which this parameter does
         not apply, the value of this parameter SHOULD be set to false. */
#define DSL_CSP_STAT_INPREPORT "inpreport"

/**
   Indicates the size of the Reed-Solomon codeword.

   - JSON status path: dsl.status.line.<#line_idx>.channel.<#channel_idx>.nfec
   - Parameter type: Integer

   Indicates the size, in octets, of the Reed-Solomon codeword in use on the
   latency path indicated in \ref lpath.

   \note See ITU-T Recommendation [G.997.1]

   \note For a multimode device operating in a mode in which this parameter does
         not apply, the value of this parameter will be set to -1. */
#define DSL_CSP_STAT_NFEC "nfec"

/**
   Indicates the number of redundancy bytes per Reed-Solomon codeword.

   - JSON status path: dsl.status.line.<#line_idx>.channel.<#channel_idx>.rfec
   - Parameter type: Integer

   Indicates the number of redundancy bytes per Reed-Solomon codeword on the
   latency path indicated in \ref lpath.

   \note See ITU-T Recommendation [G.997.1]

   \note For a multimode device operating in a mode in which this parameter
         does not apply, the value of this parameter SHOULD be set to -1. */
#define DSL_CSP_STAT_RFEC "rfec"

/**
   Indicates the number of bits per symbol assigned to the latency path.

   - JSON status path: dsl.status.line.<#line_idx>.channel.<#channel_idx>.lsymb
   - Parameter type: Integer

   Indicates the number of bits per symbol assigned to the latency path
   indicated in \ref lpath. This value does not include overhead due to trellis
   coding.

   \note See ITU-T Recommendation [G.997.1]

   \note For a multimode device operating in a mode in which this parameter
         does not apply, the value of this parameter SHOULD be set to -1. */
#define DSL_CSP_STAT_LSYMB "lsymb"

/**
   The current physical layer aggregate data rate (expressed in Kbps).

   - JSON status path: dsl.status.line.<#line_idx>.channel.<#channel_idx>.curr_rate
   - Parameter type: Integer

   This parameter is provided both for upstream (us) as well as downstream (ds)
   current physical layer aggregate data rate of the DSL connection.
   - curr_rate.us
   - curr_rate.ds */
#define DSL_CSP_STAT_CURR_RATE "curr_rate"

/**
   Obsolete actual net data rate

   - JSON status path: dsl.status.line.<#line_idx>.channel.<#channel_idx>.actndr
   - Parameter type: Integer

   Actual net data rate expressed in Kbps. In L0 state, this parameter reports
   the net data rate at which the bearer channel is operating.
   In L2 state, the parameter contains the net data rate in the previous L0 state.

   This parameter is provided both for upstream (us) as well as downstream (ds)
   - actndr.us
   - actndr.ds

   \note See [ITU T G.992.3], [ITU T G.992.5], [ITU T G.993.2].

   \note See ITU-T Recommendation [G.997.1]." */
#define DSL_CSP_STAT_ACTNDR "actndr"

/**
   Obsolete actual impulse noise protection

   - JSON status path: dsl.status.line.<#line_idx>.channel.<#channel_idx>.actinprein
   - Parameter type: Integer

   Actual impulse noise protection against REIN, expressed in 0.1 DMT symbols.
   If retransmission is used in a given transmit direction,
   this parameter reports the actual impulse noise protection (INP) against REIN
   (under specific conditions detailed in [ITU T G.998.4]) on the bearer
   channel in the L0 state. In the L2 state, the parameter contains the INP in
   the previous L0 state. The value is coded in fractions of DMT symbols with a
   granularity of 0.1 symbols. The range is from 0 to 25.4. A special value of
   25.5 indicates an ACTINP_REIN of 25.5 or higher.

   This parameter is provided both for upstream (us) as well as downstream (ds)
   - actinprein.us
   - actinprein.ds

   \note This parameter is defined as ACTINP_REIN in Clause 7.5.2.9 of ITU-T
         Recommendation [G.997.1]. */
#define DSL_CSP_STAT_ACTINPREIN "actinprein"

/**
   Indicates which DSL bonding schemes are supported.

   - JSON status path: dsl.status.bonding_group.<#grp_idx>.bond_schemes_supported
   - Parameter type: String

   Comma-separated list of strings indicating which DSL bonding schemes are
   supported.

   Defined valid enumeration strings
   - "ATM"       ([G.998.1] ATM-based bonding)
   - "Ethernet"  ([G.998.2] Ethernet-based bonding)
   - "TDIM"      ([G.998.3] TDIM-based bonding; not supported)

   \note Corresponds to [TR-159] oBondingGroup.aGroupBondSchemesSupported. */
#define DSL_CSP_STAT_BOND_SCHEMES_SUPPORTED "bond_schemes_supported"

/**
   Indicates the bonding scheme on which the bonding group is currently operated.

   - JSON status path: dsl.status.bonding_group.<#grp_idx>.bond_scheme
   - Parameter type: String

   The value will be a member of the list reported by the
   \ref bond_schemes_supported parameter.

   \note Corresponds to [TR-159] aGroupOperBondScheme. */
#define DSL_CSP_STAT_BOND_SCHEME "bond_scheme"

/**
   The achieved group data rate (expressed in Kbps).

   - JSON status path: dsl.status.bonding_group.<#grp_idx>.group_rate
   - Parameter type: Integer

   This parameter is provided both for upstream (us) as well as downstream (ds)
   achieved group data rate of the DSL bonding group.
   - group_rate.us
   - group_rate.ds

   \note This parameter corresponds to Clause 11.4.2 of ITU-T Recommendation
         [G.998.1] Achieved Aggregate Data Rate.

   \note This parameter might change subject to dynamic link usage conditions. */
#define DSL_CSP_STAT_BOND_GROUP_RATE "group_rate"

/**
   Indicate which FAST profiles are allowed on the line.

   - JSON status path: dsl.status.fast.line.<#line_idx>.allowed_profiles
   - Parameter type: String
   - Platform restriction: VRX618

   Comma-separated list of strings.
   List items indicate which FAST profiles are allowed on the line.
   Each list item is an enumeration of:
   - 106a
   - 212a

   \note In G.997.2, this parameter is called PROFILES.
   \note See ITU-T Recommendation [G.997.2]. */
#define DSL_CSP_STAT_FAST_ALLOWED_PROFILES "allowed_profiles"

/**
   Indicates which FAST profile is currently in use on the line.

   - JSON status path: dsl.status.fast.line.<#line_idx>.current_profile
   - Parameter type: String
   - Platform restriction: VRX618

   The value MUST be a member of the list reported by the
   \ref AllowedProfiles parameter. Or else be an empty string. */
#define DSL_CSP_STAT_FAST_CURRENT_PROFILE "current_profile"

/**
   Reports the signal-to-noise margin for the robust management channel (RMC).

   - JSON status path: dsl.status.fast.line.<#line_idx>.snrmrmc
   - Parameter type: Integer
   - Platform restriction: VRX618

   Express in 0.1dB.
   A special value (-512) indicates that the signal-to-noise ratio
   margin for the RMC is out of the range to be represented.
   This parameter is defined in clauses 7.10.12.1 of
   ITU-T Recommendation [G.997.2].

   This parameter is provided both for upstream (us) as well as downstream (ds)
   - snrmrmc.us
   - snrmrmc.ds */
#define DSL_CSP_STAT_FAST_SNRMRMC "snrmrmc"

/**
   Report the bit allocation values on RMC sub-carriers in RMC symbols.

   - JSON status path: dsl.status.fast.line.<#line_idx>.bitsrmcps
   - Parameter type: Integer
   - Platform restriction: VRX618

   Comma - separated list(maximum list length 4610) of integers. Each pair
   composes of a sub - carrier index from 0..4095 and an 8 bit allocation
   value.There are maximum 512 pairs. This parameter is defined in clause
   7.10.12.3 of ITU - T Recommendation[G.997.2].

   This parameter is provided both for upstream (us) as well as downstream (ds)
   - bitsrmcps.us
   - bitsrmcps.ds */
#define DSL_CSP_STAT_FAST_BITSRMCPS "bitsrmcps"

/**
   Reports the minimum error free throughput value expressed in Kbps.

   - JSON status path: dsl.status.fast.line.<#line_idx>.mineftr
   - Parameter type: Integer
   - Platform restriction: VRX618

   This parameter is computed from power up as defined in clause 11.4.1.1.3 of
   ITU-T Recommendation [G.9701] */
#define DSL_CSP_STAT_FAST_MINEFTR "mineftr"

/**
   The current status of the configuration apply.

   - JSON status path: dsl.configuration.status
   - Parameter type: Array of strings */
#define DSL_CSP_STAT_CONFIG_STATUS "status"

/** @} */

/** \defgroup DSL_STATS DSL Statistics Counter
 * Lists the entire modules from the DSL CPE Stats group.
 * \ingroup DSL_STATUS_PARSER */
/** @{ */

/**
   Seconds since the start of the period used for collection of Total statistics.

   - JSON status path: dsl.stats.line.<#line_idx>.total_start
   - Parameter type: Integer

   Statistics will continue to be accumulated across CPE reboots, though this
   might not always be possible.

   \note total_start will not be reset when the interface statistics are reset
         via an interface disable / enable cycle. */
#define DSL_CSP_STAT_TOTAL_START "total_start"

/**
   The Number of seconds since the most recent DSL Showtime.

   - JSON status path: dsl.stats.line.<#line_idx>.showtime_start
   - Parameter type: Integer

   The beginning of the period used for collection of Showtime statistics.
   Showtime is defined as successful completion of the DSL link establishment
   process. The Showtime statistics are those collected since the most recent
   establishment of the DSL link.

   \note Showtime_start will be not reset when the interface statistics are
         reset via an interface disable/enable cycle. */
#define DSL_CSP_STAT_SHOWTIME_START "showtime_start"

/**
   The Number of seconds since the second most recent DSL Showtime.

   - JSON status path: dsl.stats.line.<#line_idx>.last_showtime_start
   - Parameter type: Integer

   The beginning of the period used for collection of LastShowtime statistics.
   If the CPE has not retained information about the second most recent Showtime
   (e.g., on reboot), the start of LastShowtime statistics may temporarily
   coincide with the start of Showtime statistics.

   \note Last_showtime_start will be not reset when the interface statistics
         are reset via an interface disable/enable cycle. */
#define DSL_CSP_STAT_LAST_SHOWTIME_START "last_showtime_start"

/**
   Seconds since the start of the period used for collection of CurrentDay statistics.

   - JSON status path: dsl.stats.line.<#line_idx>.current_day_start
   - Parameter type: Integer

   Statistics will continue to be accumulated across CPE reboots, though this
   might not always be possible.

   \note Current_day_start will be not reset when the interface statistics are
         reset via an interface disable/enable cycle. */
#define DSL_CSP_STAT_CURRENT_DAY_START "current_day_start"

/**
   Seconds since the start of the period used for collection of QuarterHour statistics.

   - JSON status path: dsl.stats.line.<#line_idx>.current_day_start
   - Parameter type: Integer

   Statistics will continue to be accumulated across CPE reboots, though this
   might not always be possible.

   \note Quarter_hour_start wil be not reset when the interface statistics
         are reset via an interface disable/enable cycle. */
#define DSL_CSP_STAT_QUARTER_HOUR_START "quarter_hour_start"

/**
   DSL Line object total statistics declaration based on TR-181 definitions.

   - JSON status path: dsl.stats.line.<#line_idx>.total
   - Parameter type: Integer */
#define DSL_CSP_STAT_TOTAL "total"

/**
   DSL Line object showtime statistics declaration based on TR-181 definitions.

   - JSON status path: dsl.stats.line.<#line_idx>.showtime
   - Parameter type: Integer */
#define DSL_CSP_STAT_SHOWTIME "showtime"

/**
   DSL Line object last showtime statistics declaration based on TR-181 definitions.

   - JSON status path: dsl.stats.line.<#line_idx>.last_showtime
   - Parameter type: Integer */
#define DSL_CSP_STAT_LAST_SHOWTIME "last_showtime"

/**
   DSL Line object current day statistics declaration based on TR-181 definitions.

   - JSON status path: dsl.stats.line.<#line_idx>.current_day
   - Parameter type: Integer */
#define DSL_CSP_STAT_CURRENT_DAY "current_day"

/**
   DSL Line object quarter hour statistics declaration based on TR-181 definitions.

   - JSON status path: dsl.stats.line.<#line_idx>.quarter_hour
   - Parameter type: Integer */
#define DSL_CSP_STAT_QUARTER_HOUR "quarter_hour"

/**
   Common data structure for several line based errored second objects.

   - JSON status path: dsl.stats.line.<#line_idx>.<#obj_type>.errored_secs
   - Parameter type: Integer

   Defined valid object types (<#obj_type)
   - Total: Total number of errored seconds.
   - Showtime: Number of errored seconds since the most recent DSL Showtime.
   - LastShowtime: Number of errored seconds since the second most recent
     DSL Showtime.
   - CurrentDay: Number of errored seconds accumulated during the current day.
   - QuarterHour: Number of errored seconds accumulated during the current
     quarter hour.

  \note ES-L as defined in ITU-T Rec. [G.997.1]

  \note This parameter is optional at the G and S/T interfaces in G.997.1
        Amendment 1. */
#define DSL_CSP_STAT_ERRORED_SECS "errored_secs"

/**
   Common data structure for several line based severely errored second objects.

   - JSON status path: dsl.stats.line.<#line_idx>.<#obj_type>.severely_errored_secs
   - Parameter type: Integer

   Defined valid object types (<#obj_type)
    - Total: Total number of severely errored seconds.
    - Showtime: Number of severely errored seconds since the most recent
      DSL Showtime.
    - LastShowtime: Number of severely errored seconds since the second most
      recent DSL Showtime.
    - CurrentDay: Number of severely errored seconds accumulated during the
      current day.
    - QuarterHour: Number of severely errored seconds accumulated during the
      current quarter hour.

   \note SES-L as defined in ITU-T Rec. [G.997.1].

   \note This parameter is optional at the G and S/T interfaces in G.997.1
         Amendment 1. */
#define DSL_CSP_STAT_SEVERELY_ERRORED_SECS "severely_errored_secs"


/**
   Common data structure for several channel based error objects.

   - JSON status path:
     dsl.stats.line.<#line_idx>.channel.<#channel_idx>.<#obj_type>.xtur_fec_errors
   - Parameter type: Integer

   Defined valid object types (<#obj_type)
    - Total: Total number of FEC errors detected by the xTU-R.
    - Showtime: Number of FEC errors detected by the xTU-R since the most recent
      DSL Showtime.
    - LastShowtime: Number of FEC errors detected xTU-R since the second most
      recent DSL Showtime.
    - CurrentDay: Number of FEC errors detected by the xTU-R during the current
      day.
    - QuarterHour: Number of FEC errors detected by the xTU-R during the current
      quarter hour.

   \note FEC-C as defined in ITU-T Rec. [G.997.1]. */
#define DSL_CSP_STAT_XTUR_FEC_ERRORS "xtur_fec_errors"

/**
   Common data structure for several channel based error objects.

   - JSON status path:
     dsl.stats.line.<#line_idx>.channel.<#channel_idx>.<#obj_type>.xtuc_fec_errors
   - Parameter type: Integer

   Defined valid object types (<#obj_type)
    - Total: Total number of FEC errors detected by the xTU-C.
    - Showtime: Number of FEC errors detected by the xTU-C since the most recent
      DSL Showtime.
    - LastShowtime: Number of FEC errors detected by the xTU-C since the second
      most recent DSL Showtime.
    - CurrentDay: Number of FEC errors detected by the xTU-C during the current
      day.
    - QuarterHour: Number of FEC errors detected by the xTU-C during the current
      quarter hour.

   \note FEC-CFE as defined in ITU-T Rec. [G.997.1]. */
#define DSL_CSP_STAT_XTUC_FEC_ERRORS "xtuc_fec_errors"

/**
   Common data structure for several channel based error objects.

   - JSON status path:
     dsl.stats.line.<#line_idx>.channel.<#channel_idx>.<#obj_type>.xtur_hec_errors
   - Parameter type: Integer

   Defined valid object types (<#obj_type)
    - Total: Total number of HEC errors detected.
    - Showtime: Number of HEC errors detected by the xTU-R since the most recent
      DSL Showtime.
    - LastShowtime: Number of HEC errors detected by the xTU-R since the second
      most recent DSL Showtime.
    - CurrentDay: Number of HEC errors detected by the xTU-R during the current
      day.
    - QuarterHour: Number of HEC errors detected by the xTU-R during the current
      quarter hour.

   \note HEC-P as defined in ITU-T Rec. [G.997.1]. */
#define DSL_CSP_STAT_XTUR_HEC_ERRORS "xtur_hec_errors"

/**
   Common data structure for several channel based error objects.

   - JSON status path:
     dsl.stats.line.<#line_idx>.channel.<#channel_idx>.<#obj_type>.xtuc_hec_errors
   - Parameter type: Integer

   Defined valid object types (<#obj_type)
    - Total: Total number of HEC errors detected by the xTU-C.
    - Showtime: Number of HEC errors detected by the xTU-C since the most recent
      DSL Showtime.
    - LastShowtime: Number of HEC errors detected by the xTU-C since the second
      most recent DSL Showtime.
    - CurrentDay: Number of HEC errors detected by the xTU-C during the current
      day.
    - QuarterHour: Number of HEC errors detected by the xTU-C during the current
      quarter hour.

   \note HEC-PFE as defined in ITU-T Rec. [G.997.1]. */
#define DSL_CSP_STAT_XTUC_HEC_ERRORS "xtuc_hec_errors"

   /**
      Common data structure for several channel based error objects.

   - JSON status path:
     dsl.stats.line.<#line_idx>.channel.<#channel_idx>.<#obj_type>.xtur_crc_errors
   - Parameter type: Integer

   Defined valid object types (<#obj_type)
    - Total: Total number of CRC errors detected by the xTU-R.
    - Showtime: Number of CRC errors detected by the xTU-R since the most recent
      DSL Showtime.
    - LastShowtime: Number of CRC errors detected by the xTU-R since the second
      most recent DSL Showtime.
    - CurrentDay: Number of CRC errors detected by the xTU-R during the current
      day.
    - QuarterHour: Number of CRC errors detected by the xTU-R during the current
      quarter hour.

   \note CV-C as defined in ITU-T Rec. [G.997.1]. */
#define DSL_CSP_STAT_XTUR_CRC_ERRORS "xtur_crc_errors"

/**
   Common data structure for several channel based error objects.

   - JSON status path:
     dsl.stats.line.<#line_idx>.channel.<#channel_idx>.<#obj_type>.xtuc_crc_errors
   - Parameter type: Integer

   Defined valid object types (<#obj_type)
    - Total: Total number of CRC errors detected by the xTU-C.
    - Showtime: Number of CRC errors detected by the xTU-C since the most recent
      DSL Showtime.
    - LastShowtime: Number of CRC errors detected by the xTU-C since the second
      most recent DSL Showtime.
    - CurrentDay: Number of CRC errors detected by the xTU-C during the current
      day.
    - QuarterHour: Number of CRC errors detected by the xTU-C during the current
      quarter hour.

   \note CV-CFE as defined in ITU-T Rec. [G.997.1]. */
#define DSL_CSP_STAT_XTUC_CRC_ERRORS "xtuc_crc_errors"

/**
   Reports a count of the successful bit swap (BSW) primitives.

   - JSON status path: dsl.stats.fast.line.<#line_idx>.<total/showtime/last_showtime>.success_bsw
   - Parameter type: Integer
   - Platform restriction: VRX618

   This parameter is defined as success_BSW in clause 7.7.20 of ITU-T Rec. [G.997.2].
*/
#define DSL_CSP_STAT_FAST_SUCCESS_BSW "success_bsw"

/**
   Reports a count of the successful autonomous SRA (seamless rate adaptation) primitives.

   - JSON status path: dsl.stats.fast.line.<#line_idx>.<total/showtime/last_showtime>.success_sra
   - Parameter type: Integer
   - Platform restriction: VRX618

   This parameter is defined as success_SRA in
   clause 7.7.21 of ITU-T Rec. [G.997.2].
*/
#define DSL_CSP_STAT_FAST_SUCCESS_SRA "success_sra"

/**
   Reports a count of the successful FRA (fast rate adaptation) primitives.

   - JSON status path: dsl.stats.fast.line.<#line_idx>.<total/showtime/last_showtime>.success_fra
   - Parameter type: Integer
   - Platform restriction: VRX618

   This parameter is defined as success_FRA in clause 7.7.22 of
   ITU-T Rec. [G.997.2].
*/
#define DSL_CSP_STAT_FAST_SUCCESS_FRA "success_fra"

/**
   Reports a count of the successful RPA (RMC parameter adjustment) primitives.

   - JSON status path: dsl.stats.fast.line.<#line_idx>.<total/showtime/last_showtime>.success_rpa
   - Parameter type: Integer
   - Platform restriction: VRX618

   This parameter is defined as success_RPA in clause 7.7.23 of
   ITU-T Rec. [G.997.2].
*/
#define DSL_CSP_STAT_FAST_SUCCESS_RPA "success_rpa"

/**
   Reports a count of the successful TIGA (transmitter initiated gain adjustment) primitives.

   - JSON status path: dsl.stats.fast.line.<#line_idx>.<total/showtime/last_showtime>.success_tiga
   - Parameter type: Integer
   - Platform restriction: VRX618

   This parameter is defined as success_TIGA in
   clause 7.7.24 of ITU-T Rec. [G.997.2].
*/
#define DSL_CSP_STAT_FAST_SUCCESS_TIGA "success_tiga"

/** @} */

#define US "us"
#define DS "ds"

/** \defgroup DSL_CONFIG DSL Configuration Parser
 * Lists the entire modules from the DSL CPE Configuration Parser */
/** @{  */

/**
   This configuration parameter defines the allowed transmission system types.

   Parameter definitions:
   - JSON cfg path: dsl.xtse
   - Parameter type: Array of strings

   This configuration parameter defines the allowed transmission system types
   by the xTU on this Line instance.
   The array of strings which are provided, includes a bitmask in hex
   representation.
   Each bit is 0 if not allowed or 1 if allowed, as defined for the xTU
   transmission system enabling (XTSE) in ITU-T G.997.1.

   See \ref XTSE for bit definitions.

   \note This parameter is defined as xDSL Transmission system enabling (XTSE)
         in ITU-T G.997.1. For a full definition, see Clause 7.3.1.1.1 of ITU-T
         Recommendation [G.997.1]

   \note You might refer to the chapter "G.997.1 xTU System Enabling Octets"
         within (UM)PR of the VRX518 to get an easy human readable overview
         about all defined bits within octets that are defined by the G.997.1 */

#define DSL_CSP_CFG_XTSE "xtse"

/** \defgroup Debugging Debugging configuration
   Configuration parameters which are only relevant in case of operating a debug
   image.
   \ingroup DSL_CONFIG */

/**  @{  */

/**
   Debug configuration which controls several platform specific functions as follows
   - if Daemon activates the link (1) or not (0) after bootup
     \note This functionality is valid for VRX618 only
   - if a changed configuration (which is applied via SIGHUP) will be also
     applied automatically to the line (1) by performing a link re-start or
     not (0).
     \note This functionality is valid for both VRX518 and VRX618
     \note Link re-start will be always performed automatically on SIGHUP in
           case of using a productive image

   Parameter definitions:
   - JSON cfg path: dsl.debugging.link_start
   - Parameter type: Integer

   Defined valid values:
   - 0: Do not activate link after bootup.
   - 1: Activate link after bootup. */
#define DSL_CSP_CFG_LINK_START "link_start"

/**
   Configuration that controls if Daemon activates Debug Streams dump or not.

   Parameter definitions:
   - JSON cfg path: dsl.debugging.debug_stream_dump
   - Parameter type: Integer
   - Platform restriction: VRX618

   Defined valid values:
   - 0: Disabled.
   - 1: Enabled. */
#define DSL_CSP_CFG_DEBUG_STREAM_DUMP "debug_stream_dump"

/**
   Configuration of test and debug interface usage.

   Parameter definitions:
   - JSON cfg path: dsl.debugging.debug_and_test_interfaces
   - Parameter type: Integer
   - Platform restriction: VRX518 (and before)

   Defined valid values:
   - 0: Do not use debug and test interfaces.
        This configuration value is independent from any feature availability.
   - 1: Use debug and test interfaces on LAN interfaces only.
        This configuration value is the DEFAULT, dependent on feature
        availability.
   - 2: Use debug and test interfaces on ALL available interfaces.
        Dependent on feature availability.

   \note This configuration value will only have an influence if the DSL
         subsystem has been compiled to include TCP-Messages and/or DTI
         interface functionality.*/
#define DSL_CSP_CFG_DEBUG_AND_TEST_INTERFACES "debug_and_test_interfaces"

/** @} */

/** \defgroup Misc Miscellaneous configuration
   Configuration parameters of miscellaneous features.
   \ingroup DSL_CONFIG */

/**  @{  */

/**
    Configuration to enable or disable the overhead channel.

    Parameter definitions:
   - JSON cfg path: dsl.misc.ohc
   - Parameter type: Integer
   - Platform restriction: VRX618

   This configuration parameter enables or disables the background
   communication on the overhead channel. This feature is required
   for updates of the FE test parameters (QLN, HLog, SNR).

   Defined valid values:
   - 1: Enable,
   - 0: Disable,
   - -1: Automatically */
#define DSL_CSP_CFG_OHC "ohc"

/** @} */

/** \defgroup HybridConfig Low Level Line configuration
   Configuration parameters of LowLevelLine settings.
   \ingroup DSL_CONFIG */

/**  @{  */

/**
    Configuration parameter to select the Hybrid (Line Interface Module).

    Parameter definitions:
   - JSON cfg path: dsl.hybrid_config.relay_bi
   - Parameter type: Integer
   - Platform restriction: VRX618

   This configuration specifies if the module is bi-stable relay or not.

   Defined valid values:
   - 1: module is bi-stable relay,
   - 0: module is not bi-stable relay */
#define DSL_CSP_CFG_RELAY_BI "relay_bi"

/**
    Configuration parameter to select the oscilator clock (CO only).

    Parameter definitions:
   - JSON cfg path: dsl.hybrid_config.co_clk_35m_usage
   - Parameter type: Integer
   - Platform restriction: VRX618

   This configuration specifies if the module has 35.328M clock
   for CO application.

   Defined valid values:
   - 0: XoClock 36.00 MHz,
   - 1: XoClock 35.328 MHz */
#define DSL_CSP_CFG_CO_CLK_35M "co_clk_35m_usage"

/**
    Configuration parameter to select the Hybrid type (SINGLE, DUAL).

    Parameter definitions:
   - JSON cfg path: dsl.hybrid_config.type
   - Parameter type: Integer
   - Platform restriction: VRX618

   Defined valid values:
   - 0: Hybrid type DUAL, two hybrids and two line-drivers assembled,
   one for G.FAST and one for VDSL/ADSL
   - 1: Hybrid type SINGLE optimized for G.FAST; VDSL2 supported,
   - 2: Hybrid type SINGLE MULTI, for G.FAST and VDSL2 */
#define DSL_CSP_CFG_HYBRID_TYPE "type"

/**
    Configuration parameter to select the line driver.

    Parameter definitions:
   - JSON cfg path: dsl.hybrid_config.ld_type
   - Parameter type: Integer
   - Platform restriction: VRX618

   Defined valid values:
   - 0: LE87285 Microsemi Single Channel G.FAST Linedriver (+VDSL/ADSL)
   - 1: LE87281 Microsemi Single Channel G.FAST Linedriver (+VDSL/ADSL),
   - 2: LE87286 Microsemi Single Channel G.FAST Linedriver (+VDSL/ADSL)
   - 3: THS6301 Texas Instruments Single Channel G.FAST Linedriver
   - 4: HNX106 Lantiq Linedriver */
#define DSL_CSP_CFG_LD_TYPE "ld_type"

/** @} */

/** \defgroup StartupInit Startup Initialization
   Configuration parameters which are applied on initial startup of the
   DSL daemon respective dsl_cpe_control.
   \ingroup DSL_CONFIG */

/**  @{  */
/**
   Next DSL mode to be used on API startup of the dsl_cpe_control.

   Parameter definitions:
   - JSON cfg path: dsl.startup_init.next_mode
   - Parameter type: Integer
   - Platform restriction: VRX518 (and before)

   Only of relevance if xDSL multimode is enabled by enabling VDSL and ADSL
   G997 XTU octet bits.

   Defined valid values:
   - 0: use API-default value (if A+V modes are selected VDSL will be used)
   - 1: use ADSL mode as initial one for multimode handling
   - 2: use VDSL mode as initial one for multimode handling */
#define DSL_CSP_CFG_NEXT_MODE "next_mode"

/**
   Defines the number of devices/lines which are used on operating the DSL subsystem.

   Parameter definitions:
   - JSON cfg path: dsl.startup_init.max_device_number
   - Parameter type: Integer
   - Platform restriction: VRX518 (and before)

   This value must be equal or smaller than the number of devices which are
   detected by the MEI Driver.

   MEI Driver propagates this value via read-only procfs entry as follows.
   "MaxDeviceNumber" from
       "/proc/driver/mei_cpe/devinfo"

   The typical configuration will be that the procfs value is used here
   directly which takes care that all available devices will be operated. */
#define DSL_CSP_CFG_MAX_DEVICE_NUMBER "max_device_number"

/**
   Defines the number of lines which are used on operating the DSL subsystem.

   Parameter definitions:
   - JSON cfg path: dsl.startup_init.lines_per_device
   - Parameter type: Integer
   - Platform restriction: VRX518 (and before)

   It is strictly recommended to use the number which is propagated by the
   MEI Driver at boot time here directly.

   MEI Driver propagates this value via read-only procfs entry as follows.
   "LinesPerDevice" from
       "/proc/driver/mei_cpe/devinfo"

   The default value for VRX518 will be always "1"

   \note Values which are bigger than 1 are only allowed for on-chip bonding
         scenarios which are not supported by VRX518. */
#define DSL_CSP_CFG_LINES_PER_DEVICE "lines_per_device"

/**
   Defines the number of (bearer) channels which are used on operating the DSL subsystem.

   Parameter definitions:
   - JSON cfg path: dsl.startup_init.channels_per_line
   - Parameter type: Integer
   - Platform restriction: VRX518 (and before)

   It is strictly recommended to use the number which is propagated by the
   MEI Driver at boot time here directly.

   MEI Driver propagates this value via read-only procfs entry as follows
   "ChannelsPerLine" from
       "/proc/driver/mei_cpe/devinfo"

   The default value for VRX518 will be always "1"

   \note Values which are bigger than 1 would be only valid for non-single
         bearer channel scenarios which are not supported by VRX518. */
#define DSL_CSP_CFG_CHANNELS_PER_LINE "channels_per_line"

/**
   Defines if bonding is supported.

   Parameter definitions:
   - JSON cfg path: dsl.common.bonding_support
   - Parameter type: Integer
   - Platform restriction: VRX618

   Defined valid values:
   - 0: bonding is not supported
   - 1: bonding is supported */
#define DSL_CSP_CFG_BONDING_SUPPORT "bonding_support"

/** @} */

/** \defgroup Bitswap Bitswap configuration
   Configuration parameters which are related to Bitswap.
   \ingroup DSL_CONFIG */

/** @{ */

/**
   Configuration of bitswap setting for VDSL mode in Upstream direction.

   Parameter definitions:
   - JSON cfg path: dsl.bitswap.vdsl_us
   - Parameter type: Integer

   Defined valid values:
   - 0: Disable bitswap
   - 1: Enable bitswap */
#define DSL_CSP_CFG_BITSWAP_VDSL_US "vdsl_us"

/**
   Configuration of bitswap setting for VDSL mode in Downstream direction.

   Parameter definitions:
   - JSON cfg path: dsl.bitswap.vdsl_ds
   - Parameter type: Integer

   Defined valid values:
   - 0: Disable bitswap
   - 1: Enable bitswap */
#define DSL_CSP_CFG_BITSWAP_VDSL_DS "vdsl_ds"

/**
   Configuration of bitswap setting for ADSL mode in Upstream direction.

   Parameter definitions:
   - JSON cfg path: dsl.bitswap.adsl_us
   - Parameter type: Integer

   Defined valid values:
   - 0: Disable bitswap
   - 1: Enable bitswap */
#define DSL_CSP_CFG_BITSWAP_ADSL_US "adsl_us"

/**
   Configuration of bitswap setting for ADSL mode in Downstream direction.

   Parameter definitions:
   - JSON cfg path: dsl.bitswap.adsl_ds
   - Parameter type: Integer

   Defined valid values:
   - 0: Disable bitswap
   - 1: Enable bitswap */
#define DSL_CSP_CFG_BITSWAP_ADSL_DS "adsl_ds"

/** @} */

/** \defgroup ReTX Retransmission configuration
   Configuration parameters which are related to Retransmission (ReTX).
   \ingroup DSL_CONFIG */

/** @{ */

/**
   Configuration of retransmission setting for VDSL mode in Upstream direction.

   Parameter definitions:
   - JSON cfg path: dsl.retransmission.vdsl_us
   - Parameter type: Integer

   Defined valid values:
   - 0: Disable retransmission
   - 1: Enable retransmission
   - 2: Use API internal default configuration */
#define DSL_CSP_CFG_RETX_VDSL_US "vdsl_us"

/**
   Configuration of retransmission setting for VDSL mode in Downstream direction.

   Parameter definitions:
   - JSON cfg path: dsl.retransmission.vdsl_ds
   - Parameter type: Integer

   Defined valid values:
   - 0: Disable retransmission
   - 1: Enable retransmission
   - 2: Use API internal default configuration */
#define DSL_CSP_CFG_RETX_VDSL_DS "vdsl_ds"

/**
   Configuration of retransmission setting for ADSL mode in Upstream direction.

   Parameter definitions:
   - JSON cfg path: dsl.retransmission.adsl_us
   - Parameter type: Integer

   \note As retransmission is not defined for US in ADSL mode this value will
         be don't care. */
#define DSL_CSP_CFG_RETX_ADSL_US "adsl_us"

/**
   Configuration of retransmission setting for ADSL mode in Downstream direction

   Parameter definitions:
   - JSON cfg path: dsl.retransmission.adsl_ds
   - Parameter type: Integer

   Defined valid values:
   - 0: Disable retransmission
   - 1: Enable retransmission
   - 2: Use API internal default configuration */
#define DSL_CSP_CFG_RETX_ADSL_DS "adsl_ds"

/** @} */

/** \defgroup SRA Seamless Rate Adaptation configuration
   Configuration parameters which are related to Seamless Rate Adaptation (SRA).
   \ingroup DSL_CONFIG */

/** @{ */
/**
   Configuration of seamless rate adaptation (SRA) for VDSL in Upstream direction.

   Parameter definitions:
   - JSON cfg path: dsl.sra.vdsl_us
   - Parameter type: Integer

   Defined valid values:
   - 0: Disable SRA
   - 1: Enable SRA
   - 2: Use API internal default configuration (enabled) */
#define DSL_CSP_CFG_SRA_VDSL_US "vdsl_us"

/**
   Configuration of seamless rate adaptation (SRA) for VDSL in Downstream direction.

   Parameter definitions:
   - JSON cfg path: dsl.sra.vdsl_ds
   - Parameter type: Integer

   Defined valid values:
   - 0: Disable SRA
   - 1: Enable SRA
   - 2: Use API internal default configuration (enabled) */
#define DSL_CSP_CFG_SRA_VDSL_DS "vdsl_ds"

/**
   Configuration of seamless rate adaptation (SRA) for ADSL in Upstream direction.

   Parameter definitions:
   - JSON cfg path: dsl.sra.adsl_us
   - Parameter type: Integer

   Defined valid values:
   0: Disable SRA
   1: Enable SRA
   2: Use API internal default configuration (disabled) */
#define DSL_CSP_CFG_SRA_ADSL_US "adsl_us"

/**
   Configuration of seamless rate adaptation (SRA) for ADSL in Downstream direction.

   Parameter definitions:
   - JSON cfg path: dsl.sra.adsl_ds
   - Parameter type: Integer

   Defined valid values:
   - 0: Disable SRA
   - 1: Enable SRA
   - 2: Use API internal default configuration (disabled) */
#define DSL_CSP_CFG_SRA_ADSL_DS "adsl_ds"

/** @} */

/** \defgroup VN Virtual Noise configuration
   Configuration parameters which are related to Virtual Noise (VN).
   \ingroup DSL_CONFIG */

/** @{ */

/**
   Configuration of virtual noise setting for Upstream direction

   Parameter definitions:
   - JSON cfg path: dsl.virtual_noise.us
   - Parameter type: Integer

   Defined valid values:
   - 0: Disable virtual noise
   - 1: Enable virtual noise */
#define DSL_CSP_CFG_VN_US "us"

/**
   Configuration of virtual noise setting for Downstream direction

   Parameter definitions:
   - JSON cfg path: dsl.virtual_noise.ds
   - Parameter type: Integer

   Defined valid values:
   - 0: Disable virtual noise
    1: Enable virtual noise */
#define DSL_CSP_CFG_VN_DS "ds"

/** @} */

/** \defgroup Bonding Bonding scheme configuration
   Configuration parameters which are related to the bonding schemes.
   \ingroup DSL_CONFIG */

/** @{ */

/**
   Configuration of ADSL PTM bonding scheme.

   Parameter definitions:
   - JSON cfg path: dsl.bonding.adsl_ptm
   - Parameter type: Integer
   _ Platform restriction: VRX518

   Defined valid values to configure ADSL PTM bonding scheme:
   - 0: Disable
   - 1: Enable */
#define DSL_CSP_CFG_BONDING_ADSL_PTM "adsl_ptm"

/**
   Configuration of ADSL ATM bonding scheme.

   Parameter definitions:
   - JSON cfg path: dsl.bonding.adsl_atm
    _ Platform restriction: VRX518
  - Parameter type: Integer

   Defined valid values to configure ADSL ATM bonding scheme:
   - 0: Disable
   - 1: Enable */
#define DSL_CSP_CFG_BONDING_ADSL_ATM "adsl_atm"

/**
   Configuration of VDSL PTM bonding scheme.

   Parameter definitions:
   - JSON cfg path: dsl.bonding.vdsl_ptm
   - Parameter type: Integer
   _ Platform restriction: VRX518

   Defined valid values to configure VDSL PTM bonding scheme:
   - 0: Disable
   - 1: Enable */
#define DSL_CSP_CFG_BONDING_VDSL_PTM "vdsl_ptm"

/**
   Configuration of VDSL ATM bonding scheme.

   Parameter definitions:
   - JSON cfg path: dsl.bonding.vdsl_atm
   - Parameter type: Integer
   _ Platform restriction: VRX518

   Defined valid values to configure VDSL ATM bonding scheme:
   - 0: Disable
   - 1: Enable */
#define DSL_CSP_CFG_BONDING_VDSL_ATM "vdsl_atm"

/** @} */

/** \defgroup TcLayer TC-Layer configuration
   Configuration parameters which are related to the TC-Layer.
   \ingroup DSL_CONFIG */

/** @{ */

/**
   Configuration of TC-Layer for ADSL mode.

   Parameter definitions:
   - JSON cfg path: dsl.tc_layer.adsl
   - Parameter type: Integer

   Defined valid values for ADSL TC-Layer:
   - 0: Use API internal defaults
   - 1: ATM-TC
   - 2: EFM/PTM-TC
   - 4: Auto-TC */
#define DSL_CSP_CFG_TC_ADSL "adsl"

/**
   Configuration of TC-Layer for VDSL mode.

   Parameter definitions:
   - JSON cfg path: dsl.tc_layer.vdsl
   - Parameter type: Integer

   Defined valid values for VDSL TC-Layer:
   - 0: Use API internal defaults
   - 1: ATM-TC
   - 2: EFM/PTM-TC
   - 4: Auto-TC */
#define DSL_CSP_CFG_TC_VDSL "vdsl"

/** @} */

/** \defgroup RC Reboot Criteria configuration
   Configuration parameters which are related to the link restart criteria's.
   \ingroup DSL_CONFIG */

/** @{ */

/**
   Link restart (reboot) criteria for ADSL.

   Parameter definitions:
   - JSON cfg path: dsl.reboot_criteria.adsl
   - Parameter type: String
   - Platform restriction: VRX518 (and before)

   The configuration value enables the link restrat (reboot) criteria's for ADSL.
   The string which is provided, includes a bitmask in hexadecimal representation.

   Defined valid values for bitmask:
   - "0": CLEANED (no reboot criteria active)
   - "1": LOM, Loss of margin, API default: ON
   - "2": LOF, Loss of frame, API default: ON
   - "4": LOS, Loss of signal, API default ON
   - "8": ESE, Excessive Severe Errors, API default: ON
   - "10": ES90, 90 consecutive errored seconds, API default: OFF
   - "20": SES30, 30 consecutive severely errored seconds, API default: OFF
   - "40": NEGATIVE_MARGIN, API default: ON
           Note that this option is only supported for ADSL only platforms
           (ARX and before). For VRX it is don't care!
   - "80": OOS_BC0, Out of sync failure, API default: OFF
   - "100": OOS_BC1, Out of sync failure, API default: OFF
   - "200": NCD_BC0, No-Cell-Delineation failure, API default: OFF
   - "400": NCD_BC1, No-Cell-Delineation failure, API default: OFF
   - "800": LCD_BC0, Loss-of-Cell-Delineation failure, API default: OFF
   - "1000": LCD_BC1, Loss-of-Cell-Delineation failure, API default: OFF

   \note The default configuration value for ADSL (as defined above) is 0x4F */
#define DSL_CSP_CFG_REBOOT_ADSL "adsl"

/**
   Link restart (reboot) criteria for VDSL.

   Parameter definitions:
   - JSON cfg path: dsl.reboot_criteria.vdsl
   - Parameter type: String
   - Platform restriction: VRX518 (and before)

   The configuration value enables the link restrat (reboot) criteria's for VDSL.
   The string which is provided, includes a bitmask in hex representation.

   Defined valid values for bitmask:
   - "0": CLEANED (no reboot criteria active)
   - "1": LOM, Loss of margin, API default: ON
   - "2": LOF, Loss of frame, API default: ON
   - "4": LOS, Loss of signal, API default ON
   - "8": ESE, Excessive Severe Errors, API default: ON
   - "10": ES90, 90 consecutive errored seconds, API default: OFF
   - "20": SES30, 30 consecutive severely errored seconds, API default: OFF
   - "40": NEGATIVE_MARGIN, API default: OFF
           Note that this option is only supported for ADSL only platforms
           (ARX and before). For VRX it is don't care!
   - "80": OOS_BC0, Out of sync failure, API default: OFF
   - "100": OOS_BC1, Out of sync failure, API default: OFF
   - "200": NCD_BC0, No-Cell-Delineation failure, API default: OFF
   - "400": NCD_BC1, No-Cell-Delineation failure, API default: OFF
   - "800": LCD_BC0, Loss-of-Cell-Delineation failure, API default: OFF
   - "1000": LCD_BC1, Loss-of-Cell-Delineation failure, API default: OFF

   \note The default configuration value for VDSL (as defined above) is 0xF */
#define DSL_CSP_CFG_REBOOT_VDSL "vdsl"

/** @} */

/** \defgroup DSL_COMMON Common configuration
   Common configuration parameters.
   \ingroup DSL_CONFIG */

/** @{ */

/**
   The message selects a DSL operator.

   The information is used to configure operator specific settings inside the
   DSL firmware.

   Parameter definitions:
   - JSON cfg path: dsl.common.operator_select
   - Parameter type: Integer

   \note Please note the following comments
   - For more detailed information please refer to the according description
     (e.g. Operator IOP settings) which is provided with the release or contact
     the support team.
   - This configuration setting was applied via the extended autoboot script
     file handling within earlier releases of the DSL Subsystem. */
#define DSL_CSP_CFG_OPERATOR_SELECT "operator_select"

/**
   Defines the handling on termination of the DSL Subsystem.

   Parameter definitions:
   - JSON cfg path: dsl.common.ld_afe_shutdown
   - Parameter type: Integer
   - Platform restriction: VRX518 (and before)

   \note This configuration option is currently not supported. */
#define DSL_CSP_CFG_LD_AFE_SHUTDOWN "ld_afe_shutdown"

/**
   Configuration of test and debug specific vectoring setting.

   Parameter definitions:
   - JSON cfg path: dsl.common.vectoring_enable
   - Parameter type: Integer
   - Platform restriction: VRX518 (and before)

   Defined valid values:
   - 0: Disable any vectoring operation
   - 1: Enables the G.993.5 full G.Vector for US and DS)
   - 2: Enables the G.993.2 Annex Y (G.Vector friendly)
   - 3: Enables automatic detection for the best fitting configuration

   \note Vectoring is only supported for VDSL

   \note There are some additional conditions that needs to be fulfilled for
         successful enabling of the full G.Vector functionality (Annex X) as
         follows
   - DSL Firmware needs to support vectoring, means that the last digit of the
     firmware version has to be "7"
   - The DSL Subsystem has to be compiled to support vectoring */
#define DSL_CSP_CFG_VECTORING_ENABLE "vectoring_enable"

/**
   Configuration of non-standard activation sequence.

   Parameter definitions:
   - JSON cfg path: dsl.common.activation_sequence
   - Parameter type: Integer
   - Platform restriction: VRX518 (and before)

   Defined valid values:
   - 0: Automatic detection from XTSE configuration (enables Non-standard
        activation sequence in case of T1.413 is enabled as well)
   - 1: Standard G.HS activation sequence (using G.994.1 AnnexA)
   - 2: Non-standard activation sequence ("Telefonica switching mode") */
#define DSL_CSP_CFG_ACTIVATION_SEQUENCE "activation_sequence"

/**
   Configuration of initial activation mode.

   Parameter definitions:
   - JSON cfg path: dsl.common.activation_mode
   - Parameter type: Integer
   - Platform restriction: VRX518 (and before)

   This configuration is only used if \ref DSL_CSP_CFG_ACTIVATION_SEQUENCE is
   set to one of the following configurations
   - 2, unconditionally
   - 0, in case of T1.413 mode is included within XTSE octets

   Defined valid values:
   - 1: Start with G.HS activation mode, this is the default configuration
   - 2: Start with ANSI-T1.413 activation mode */
#define DSL_CSP_CFG_ACTIVATION_MODE "activation_mode"

/**
   Configuration of remember functionality used in case of activated xDSL multimode

   Parameter definitions:
   - JSON cfg path: dsl.common.remember
   - Parameter type: Integer
   - Platform restriction: VRX518 (and before)

   Defined valid values:
   - 0: disable remember functionality. On link drop it will always re-train
        with default DSL mode, which is VDSL if enabled.
   - 1: enable remember functionality. On link drop it will re-train with
        last successful DSL showtime mode. */
#define DSL_CSP_CFG_REMEMBER "remember"

/**
   Test and debug configuration to enable event based FW message polling.

   Parameter definitions:
   - JSON cfg path: dsl.common.fw_msg_polling_only
   - Parameter type: Integer
   - Platform restriction: VRX518 (and before)

   This configuration can be used for test and debug puposes to enable the event
   based (autonomous FW message handling). By default the API is performing FW
   message polling which is also the strictly recommended mode for productive
   images!

   Defined valid values:
   - 0: Use autonomous FW message handling (instead of polling)
   - 1: Use polling mode only, this is the default configuration */
#define DSL_CSP_CFG_FW_MSG_POLLING_ONLY "fw_msg_polling_only"

/**
    Configuration to specify an additional service on low frequencies.

    Parameter definitions:
   - JSON cfg path: dsl.common.underlying_service
   - Parameter type: Integer
   - Platform restriction: VRX618

    Configuration to specify an additional service on low frequencies
    to be present and considered.

    Defined valid values:
    - 0: POTS (Annex A)
    - 1: ISDN (Annex B) */
#define DSL_CSP_CFG_UNDERLYING_SERVICE "underlying_service"

/**
   Defines the allowed VDSL profiles.

   Parameter definitions:
   - JSON cfg path: dsl.common.vdsl_profile
   - Parameter type: String

   The configuration value selects the allowed/activated VDSL profiles to be
   used for the handshake phase. (See G.994.1 Amendment 4 [10], NPAR(3) coding).
   The string which is provided includes a bitmask in hex representation.

   Defined valid values for bitmask:
   - "0": No specific configuration selected will result in usage of API
          internal default configuration to enable all possible profiles
          (strictly recommended!)
   - "1": Profile 8A
   - "2": Profile 8B
   - "4": Profile 8C
   - "8": Profile 8D
   - "10": Profile 12A
   - "20": Profile 12B
   - "40": Profile 17A
   - "80": Profile 30A
   - "100": Profile 35B */
#define DSL_CSP_CFG_VDSL_PROFILE "vdsl_profile"

/** \defgroup lowlevelhs Low Level HS Tones
   \ingroup DSL_COMMON */

/** @{ */

/**
   Configuration of low level handshake tones for ADSL.

   Parameter definitions:
   - JSON cfg path: dsl.common.low_level_hs_tones.adsl
   - Parameter type: String

   The string which is provided includes a bitmask in hex representation which
   defines the activated handshake tones according to the following bitmask
   definitions.

   Defined valid values for bitmask:
   - "0": No specific configuration selected will result in usage of API
   -      internal default handshake tones (strictly recommended!)
          VRX518: Please also refer to the following chapter of the (UM)PR for
          more details: "Selection of Handshake Tone Groups"
   - "1": B43
   - "2": A43
   - "4": V43
   - "8": VDSL1 V43POTS
   - "10": VDSL1 V43ISDN
   - "20": C43
   - "40": J43
   - "80": B43C
   - "100": A43C */
#define DSL_CSP_CFG_LOW_LEVEL_HS_TONES_ADSL "adsl"

/**
   Configuration of low level handshake tones for VDSL.

   Parameter definitions:
   - JSON cfg path: dsl.common.low_level_hs_tones.vdsl
   - Parameter type: String

   The string which is provided includes a bitmask in hex representation which
   defines the activated handshake tones according to the following bitmask
   definitions.

   Defined valid values for bitmask:
   - "0": No specific configuration selected will result in usage of API
          internal default handshake tones (strictly recommended!)
          VRX518: Please also refer to the following chapter of the (UM)PR for
          more details: "Selection of Handshake Tone Groups"
   - "1": B43
   - "2": A43
   - "4": V43
   - "8": VDSL1 V43POTS
   - "10": VDSL1 V43ISDN
   - "20": C43
   - "40": J43
   - "80": B43C
   - "100": A43C */
#define DSL_CSP_CFG_LOW_LEVEL_HS_TONES_VDSL "vdsl"

/**
   Configuration of low level handshake tones for VDSL, ADSL and G.FAST.

   Parameter definitions:
   - JSON cfg path: dsl.common.low_level_hs_tones.xdsl
   - Parameter type: String
   - Platform restriction: VRX618

   The string which is provided includes a bitmask in hex representation which
   defines the activated handshake tones according to the following bitmask
   definitions.

   Defined valid values for bitmask:
   - "0": No specific configuration selected will result in usage of API
          internal default handshake tones (strictly recommended!)
   - "1": B43
   - "2": A43
   - "4": V43
   - "8": VDSL1 V43POTS
   - "10": VDSL1 V43ISDN
   - "20": C43
   - "40": J43
   - "80": B43C
   - "100": A43C */
#define DSL_CSP_CFG_LOW_LEVEL_HS_TONES_XDSL "xdsl"

/** @} */

/** \defgroup fast_annex_s G.Fast Annex-S
   \ingroup DSL_FAST_ANNEX_S */

/** @{ */

/**
   Enabling G.Fast Annex-S feature.

   Parameter definitions:
   - JSON cfg path: dsl.fast_annex_s.enable
   - Parameter type: Integer
   - Platform restriction: VRX618

   Defined valid values:
   - 0: disable functionality,
   - 1: enable functionality. */
#define DSL_CSP_CFG_FAST_ANNEX_S_ENABLE "enable"

/**
   Configuration of G.FAST Annex-S accumulation timeout (ms).

   Parameter definitions:
   - JSON cfg path: dsl.fast_annex_s.accumulation_timeout
   - Parameter type: Integer
   - Platform restriction: VRX618

   Defined valid range:
   - 0: min,
   - 18: default,
   - 300: max. */
#define DSL_CSP_CFG_FAST_ANNEX_S_ACC_TIMEOUT "accumulation_timeout"

/** @} */

/** @} */
