/******************************************************************************

         Copyright (c) 2020 Intel Corporation

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

*******************************************************************************/

#include "dsl_mgmt_manager.h"
#include "dsl_mgmt_manager_objects_methods.h"
#include "dsl_mgmt_manager_interface.h"
#include "dsl_mgmt_ubus_interface.h"

#if defined(DSL_MNGR_UBUS_SUPPORT) && (DSL_MNGR_UBUS_SUPPORT == 1)

/**********************************************/
/* defines                                    */
/**********************************************/

#define DSL_MNGR_SEPARATOR "."

#define DSL_MNGR_OBJECT_DSL "dsl"
#define DSL_MNGR_OBJECT_LINE DSL_MNGR_OBJECT_DSL DSL_MNGR_SEPARATOR "line"
#define DSL_MNGR_OBJECT_CHANNEL DSL_MNGR_OBJECT_DSL DSL_MNGR_SEPARATOR "channel"
#define DSL_MNGR_OBJECT_BONDING_GROUP DSL_MNGR_OBJECT_DSL DSL_MNGR_SEPARATOR "bonding_group"

#define DSL_MNGR_NS_DSL DSL_MNGR_OBJECT_DSL DSL_MNGR_SEPARATOR
#define DSL_MNGR_NS_LINE DSL_MNGR_OBJECT_LINE DSL_MNGR_SEPARATOR
#define DSL_MNGR_NS_CHANNEL DSL_MNGR_OBJECT_CHANNEL DSL_MNGR_SEPARATOR
#define DSL_MNGR_NS_BONDING_GROUP DSL_MNGR_OBJECT_BONDING_GROUP DSL_MNGR_SEPARATOR

#define DSL_MNGR_METHOD_INFO "info"
#define DSL_MNGR_METHOD_STATS "stats"
#define DSL_MNGR_METHOD_STATS_INTERVAL "stats_interval"

#define DSL_MNGR_UBUS_OBJECT_NAME_LEN_MAX 256
#define DSL_MNGR_UBUS_METHOD_NAME_LEN_MAX 256

/* Might be adjusted after traffic load integration tests */
#define DSL_MNGR_TIMER_NOTIFY_POLL_TIME_MS 200

/**********************************************/
/* external declarations                      */
/**********************************************/

extern void DSL_MNGR_Log(
   const DSL_MNGR_LogLevel_t level,
   const char* msg,
   va_list args
);

/**********************************************/
/* typedefs                                   */
/**********************************************/

typedef enum {
   DSL_MNGR_OBJECT_FIRST_IDX = 0,
   DSL_MNGR_OBJECT_LINE_IDX = 0,
   DSL_MNGR_OBJECT_CHANNEL_IDX,
   DSL_MNGR_OBJECT_BONDING_GROUP_IDX,
   DSL_MNGR_OBJECT_LAST_IDX
} DSL_MNGR_UbusObjectIdx_t;

typedef enum {
   DSL_MNGR_METHOD_FIRST_IDX = 0,
   DSL_MNGR_METHOD_INFO_IDX = 0,
   DSL_MNGR_METHOD_STATS_IDX,
   DSL_MNGR_METHOD_STATS_INTERVAL_IDX,
   DSL_MNGR_METHOD_LAST_IDX
} DSL_MNGR_UbusMethodIdx_t;

typedef struct {
   DSL_MNGR_UbusObjectIdx_t idx;
   char name[DSL_MNGR_UBUS_OBJECT_NAME_LEN_MAX];
} DSL_MNGR_UbusObjectName_t;

typedef struct {
   DSL_MNGR_UbusMethodIdx_t idx;
   char name[DSL_MNGR_UBUS_METHOD_NAME_LEN_MAX];
} DSL_MNGR_UbusMethodName_t;

typedef struct {
   DSL_MNGR_UbusObjectIdx_t objectIdx;
   DSL_MNGR_UbusMethodIdx_t methodIdx;
   ubus_handler_t handler;
   bool bSupported;
} DSL_MNGR_UbusObjectMethod_t;

typedef struct {
   ubus_handler_t handler;
   struct blobmsg_policy *pPolicy;
   int nPolicy;
} DSL_MNGR_UbusMethodPolicy_t;

typedef struct {
   DSL_MNGR_UbusObjectIdx_t objectIdx;
   unsigned int nObjectsNum;
   unsigned int nMethodsNum;
} DSL_MNGR_UbusObjectMethodNums_t;

typedef struct {
    DSL_MNGR_UbusObjectIdx_t idx;
    struct ubus_object **pObjects;
} DSL_MNGR_UbusObjectFinal_t;

/**********************************************/
/* global variable declarations               */
/**********************************************/

/** Dictionary of ubus objects supported by DSL Manager */
static const DSL_MNGR_UbusObjectName_t g_ubusObjectsNames[] = {
   { DSL_MNGR_OBJECT_LINE_IDX, DSL_MNGR_OBJECT_LINE },
   { DSL_MNGR_OBJECT_CHANNEL_IDX, DSL_MNGR_OBJECT_CHANNEL },
   { DSL_MNGR_OBJECT_BONDING_GROUP_IDX, DSL_MNGR_OBJECT_BONDING_GROUP }
};

/** Dictionary of ubus methods supported by DSL Manager */
static const DSL_MNGR_UbusMethodName_t g_ubusMethodNames[] = {
   { DSL_MNGR_METHOD_INFO_IDX, DSL_MNGR_METHOD_INFO },
   { DSL_MNGR_METHOD_STATS_IDX , DSL_MNGR_METHOD_STATS },
   { DSL_MNGR_METHOD_STATS_INTERVAL_IDX, DSL_MNGR_METHOD_STATS_INTERVAL }
};

struct blobmsg_policy
   DSL_MNGR_ChannelMethodPolicy[__DSL_MNGR_CHANNEL_METHOD_ARG_MAX] = {
      [DSL_MNGR_CHANNEL_METHOD_ARG_LINE] =
         { .name = "line", .type = BLOBMSG_TYPE_INT32 },
};

static const DSL_MNGR_UbusMethodPolicy_t g_ubusMethodPolicy[] = {
   { DSL_MNGR_ChannelInfoHandle, DSL_MNGR_ChannelMethodPolicy, ARRAY_SIZE(DSL_MNGR_ChannelMethodPolicy) },
   { DSL_MNGR_ChannelStatsHandle, DSL_MNGR_ChannelMethodPolicy, ARRAY_SIZE(DSL_MNGR_ChannelMethodPolicy) },
   { DSL_MNGR_ChannelStatsIntervalHandle, DSL_MNGR_ChannelMethodPolicy, ARRAY_SIZE(DSL_MNGR_ChannelMethodPolicy) }
};
static const unsigned int g_ubusMethodPolicyNum =
   sizeof(g_ubusMethodPolicy)/sizeof(g_ubusMethodPolicy[0]);

/**
 * All object and method pairs supported by DSL Manager.
 *
 * Helps sync functionality between ubus framework and DSL API
 * (member \ref bSupported).
*/
static DSL_MNGR_UbusObjectMethod_t g_ubusObjectsMethods[] = {
   { DSL_MNGR_OBJECT_LINE_IDX, DSL_MNGR_METHOD_INFO_IDX,
      DSL_MNGR_LineInfoHandle, false },
   { DSL_MNGR_OBJECT_LINE_IDX, DSL_MNGR_METHOD_STATS_IDX,
      DSL_MNGR_LineStatsHandle, false },
   { DSL_MNGR_OBJECT_LINE_IDX, DSL_MNGR_METHOD_STATS_INTERVAL_IDX,
      DSL_MNGR_LineStatsIntervalHandle, false },
   { DSL_MNGR_OBJECT_CHANNEL_IDX, DSL_MNGR_METHOD_INFO_IDX,
      DSL_MNGR_ChannelInfoHandle, false },
   { DSL_MNGR_OBJECT_CHANNEL_IDX, DSL_MNGR_METHOD_STATS_IDX,
      DSL_MNGR_ChannelStatsHandle, false },
   { DSL_MNGR_OBJECT_CHANNEL_IDX, DSL_MNGR_METHOD_STATS_INTERVAL_IDX,
      DSL_MNGR_ChannelStatsIntervalHandle, false },
   { DSL_MNGR_OBJECT_BONDING_GROUP_IDX, DSL_MNGR_METHOD_INFO_IDX,
      DSL_MNGR_BondingGroupInfoHandle, false }
};
static const unsigned int g_ubusObjectsMethodsNum =
   sizeof(g_ubusObjectsMethods)/sizeof(g_ubusObjectsMethods[0]);

/**
 * Number of object instances and methods supported by Ubus Module.
 * Used to create \ref DSL_MNGR_UbusObjectFinal_t instances.
*/
static DSL_MNGR_UbusObjectMethodNums_t g_ubusObjectsMethodsQuantity[] = {
   { DSL_MNGR_OBJECT_LINE_IDX, 0, 0 },
   { DSL_MNGR_OBJECT_CHANNEL_IDX, 0, 0 },
   { DSL_MNGR_OBJECT_BONDING_GROUP_IDX, 0, 0 }
};
static const unsigned int g_ubusObjectsMethodsQuantityNum =
   sizeof(g_ubusObjectsMethodsQuantity)/sizeof(g_ubusObjectsMethodsQuantity[0]);

DSL_MNGR_LogLevel_t nLogLevel = DSL_MNGR_LOG_LEVEL_INFO;
static struct ubus_context *gpUbusCtx = NULL;
DSL_MNGR_UbusObjectFinal_t **gpUbusObjectsFinal = NULL;
static unsigned int gnObjectsFinal = 0;

static struct uloop_timeout commonNotifier = { 0 };

/**********************************************/
/* local function declarations                */
/**********************************************/

static int DSL_MNGR_ObjectMethodHandlerGet(
   const DSL_MNGR_UbusObjectIdx_t nObjectIdx,
   const DSL_MNGR_UbusMethodIdx_t nMethodIdx,
   ubus_handler_t *handler
);

static int DSL_MNGR_ObjectMethodPolicyGet(
   ubus_handler_t handler,
   struct blobmsg_policy **pPolicy,
   int *pPolicyNum
);

static int DSL_MNGR_ObjectMethodQuantitySet(
   const DSL_MNGR_UbusObjectIdx_t nObjectIdx,
   const unsigned int nObjectsQuantity,
   const unsigned int nMethodsQuantity
);

static int DSL_MNGR_ObjectMethodSupportedSet(
   const DSL_MNGR_UbusObjectIdx_t nObjectIdx,
   const DSL_MNGR_UbusMethodIdx_t nMethodIdx
);

static int DSL_MNGR_UbusModuleObjectsGet(
   char ***pObjectsNames,
   unsigned int *pObjectsNamesNum
);

static int DSL_MNGR_UbusModuleObjectsDetailsGet(
   const char *pObjectName,
   char ***pObjectMethods,
   unsigned int *pObjectMethodsNum,
   unsigned int *pObjectNum
);

static int DSL_MNGR_Sync(
   char **pObjectsNames,
   const unsigned int nObjectsNamesNum,
   unsigned int *pObjectsFoundMatchesNum,
   unsigned int *pObjectsMethodsFoundMatchesNum
);

static int DSL_MNGR_MethodCreate(
   struct ubus_method **pMethods,
   const unsigned int nMethodsNum,
   const DSL_MNGR_UbusObjectIdx_t nObjectIdx
);

static int DSL_MNGR_ObjectFill(
   struct ubus_object *pObject,
   const unsigned int nObjectNum,
   const struct ubus_method *pMethods,
   const unsigned int nMethodsNum,
   const char *pName
);

static int DSL_MNGR_ObjectsAdd(
   struct ubus_context *pUbusCtx
);

static int DSL_MNGR_Init(
   struct ubus_context **pUbusCtx,
   const char *pUbusSocket
);

static int DSL_MNGR_Run(
   struct ubus_context *pUbusCtx
);

static int DSL_MNGR_Deinit(
   struct ubus_context *pUbusCtx
);

static int DSL_MNGR_LogLevelRead(
   const char* pPath,
   DSL_MNGR_LogLevel_t *pLogLevel
);

static void DSL_MNGR_NotifyCommonHandler(
   struct uloop_timeout *t
);

static void DSL_MNGR_NotifySend(
   struct ubus_object *pObject,
   const char *pObjectName,
   const char *pMethodName,
   const unsigned int nObjectNum,
   const unsigned int nChannelLineNum
);

static void DSL_MNGR_SubscribeCommonHandler(
   struct ubus_context *ctx,
   struct ubus_object *obj
);

static int DSL_MNGR_ObjectsRelease(void);

static int DSL_MNGR_ArrayRelease(
   char ***pArray,
   const unsigned int nSize
);

#endif /* defined(DSL_MNGR_UBUS_SUPPORT) && (DSL_MNGR_UBUS_SUPPORT == 1) */

/**********************************************/
/* global function definitions                */
/**********************************************/

#if defined(DSL_MNGR_UBUS_SUPPORT) && (DSL_MNGR_UBUS_SUPPORT == 1)

void DSL_MNGR_LogError(
   const char* msg,
   ...)
{
   va_list args;
   va_start(args, msg);

   DSL_MNGR_Log(DSL_MNGR_LOG_LEVEL_ERROR, msg, args);

   va_end(args);
}

void DSL_MNGR_LogWarning(
   const char* msg,
   ...)
{
   va_list args;
   va_start(args, msg);

   DSL_MNGR_Log(DSL_MNGR_LOG_LEVEL_WARNING, msg, args);

   va_end(args);
}

void DSL_MNGR_LogInfo(
   const char* msg,
   ...)
{
   va_list args;
   va_start(args, msg);

   DSL_MNGR_Log(DSL_MNGR_LOG_LEVEL_INFO, msg, args);

   va_end(args);
}

void DSL_MNGR_LogDebug(
   const char* msg,
   ...)
{
   va_list args;
   va_start(args, msg);

   DSL_MNGR_Log(DSL_MNGR_LOG_LEVEL_DEBUG, msg, args);

   va_end(args);
}

#endif /* defined(DSL_MNGR_UBUS_SUPPORT) && (DSL_MNGR_UBUS_SUPPORT == 1) */

int DSL_MNGR_Main(const char *pUbusSocket)
{
   int nErrorCode = 0;

#if defined(DSL_MNGR_UBUS_SUPPORT) && (DSL_MNGR_UBUS_SUPPORT == 1)
   int nRetCode = 0;

   DSL_MNGR_LogInfo(DSL_MNGR_PREFIX
      "%s: IN (socket='%s')" DSL_MNGR_CRLF, __func__,
      pUbusSocket);

   if (DSL_MNGR_LogLevelRead("/root/dslmanager.loglevel", &nLogLevel) == 0)
   {
      DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
         "%s: Logging level=<%u>" DSL_MNGR_CRLF, __func__,
         nLogLevel);
   }

   DSL_MNGR_LogInfo(DSL_MNGR_PREFIX
      "%s: Collecting and synchronizing data..." DSL_MNGR_CRLF, __func__);

   if ((nRetCode = DSL_MNGR_Init(&gpUbusCtx, pUbusSocket)))
   {
      DSL_MNGR_LogError(DSL_MNGR_PREFIX
         "%s: Init stage failed (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
         nRetCode);

      return -1;
   }

   DSL_MNGR_LogInfo(DSL_MNGR_PREFIX
      "%s: Registering data and running main loop..." DSL_MNGR_CRLF, __func__);

   if ((nRetCode = DSL_MNGR_Run(gpUbusCtx)))
   {
      DSL_MNGR_LogError(DSL_MNGR_PREFIX
         "%s: Run stage failed (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
         nRetCode);

      return -1;
   }

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: OUT (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
      nErrorCode);

#else
   fprintf(stdout, DSL_MNGR_PREFIX
      "%s: Ubus framework not available!" DSL_MNGR_CRLF, __func__);

   nErrorCode = -1;
#endif /* defined(DSL_MNGR_UBUS_SUPPORT) && (DSL_MNGR_UBUS_SUPPORT == 1) */

   return nErrorCode;
}

int DSL_MNGR_Stop(void)
{
   int nErrorCode = 0;

#if defined(DSL_MNGR_UBUS_SUPPORT) && (DSL_MNGR_UBUS_SUPPORT == 1)
   int nRetCode = 0;

   DSL_MNGR_LogInfo(DSL_MNGR_PREFIX
      "%s: IN (ubus=<%p>)" DSL_MNGR_CRLF, __func__,
      gpUbusCtx);

   if (gpUbusCtx == NULL)
   {
      DSL_MNGR_LogError(DSL_MNGR_PREFIX
         "%s: Stop stage failed, Manager not started yet!" DSL_MNGR_CRLF, __func__);

      return -1;
   }

   DSL_MNGR_LogInfo(DSL_MNGR_PREFIX
      "%s: Releasing resources and stopping the loop..." DSL_MNGR_CRLF, __func__);

   if ((nRetCode = DSL_MNGR_Deinit(gpUbusCtx)))
   {
      DSL_MNGR_LogError(DSL_MNGR_PREFIX
         "%s: Deinit stage failed (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
         nRetCode);

      return -1;
   }

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: OUT (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
      nErrorCode);
#else
   fprintf(stdout, DSL_MNGR_PREFIX
      "%s: Ubus framework not available!" DSL_MNGR_CRLF, __func__);

   nErrorCode = -1;
#endif /* defined(DSL_MNGR_UBUS_SUPPORT) && (DSL_MNGR_UBUS_SUPPORT == 1) */

   return nErrorCode;
}

/**********************************************/
/* local function definitions                 */
/**********************************************/

#if defined(DSL_MNGR_UBUS_SUPPORT) && (DSL_MNGR_UBUS_SUPPORT == 1)

static int DSL_MNGR_ObjectMethodHandlerGet(
   const DSL_MNGR_UbusObjectIdx_t nObjectIdx,
   const DSL_MNGR_UbusMethodIdx_t nMethodIdx,
   ubus_handler_t *handler)
{
   int nErrorCode = -1;

   *handler = NULL;

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: IN (object=<%u> method=<%u>)" DSL_MNGR_CRLF, __func__,
      nObjectIdx, nMethodIdx);

   for(unsigned int i = 0; i < g_ubusObjectsMethodsNum; ++i)
   {
      if (g_ubusObjectsMethods[i].objectIdx == nObjectIdx &&
         g_ubusObjectsMethods[i].methodIdx == nMethodIdx)
      {
         DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
            "%s: Found handler=<%p>" DSL_MNGR_CRLF, __func__,
            g_ubusObjectsMethods[i].handler);

         *handler = g_ubusObjectsMethods[i].handler;
         nErrorCode = 0;
         break;
      }
   }

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: OUT (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
      nErrorCode);

   return nErrorCode;
}

static int DSL_MNGR_ObjectMethodPolicyGet(
   ubus_handler_t handler,
   struct blobmsg_policy **pPolicy,
   int *pPolicyNum)
{
   int nErrorCode = -1;

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: IN (handler=<%p>)" DSL_MNGR_CRLF, __func__,
      handler);

   for(unsigned int i = 0; i < g_ubusMethodPolicyNum; ++i)
   {
      if (g_ubusMethodPolicy[i].handler == handler)
      {
         DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
            "%s: Found policy=<%p> size=<%u>" DSL_MNGR_CRLF, __func__,
            g_ubusMethodPolicy[i].pPolicy, g_ubusMethodPolicy[i].nPolicy);

         *pPolicy = g_ubusMethodPolicy[i].pPolicy;
         *pPolicyNum = g_ubusMethodPolicy[i].nPolicy;
         nErrorCode = 0;
         break;
      }
   }

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: OUT (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
      nErrorCode);

   return nErrorCode;
}

static int DSL_MNGR_ObjectMethodQuantitySet(
   const DSL_MNGR_UbusObjectIdx_t nObjectIdx,
   const unsigned int nObjectsQuantity,
   const unsigned int nMethodsQuantity)
{
   int nErrorCode = -1;

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: IN (object=<%u> object quantity=<%u> method quantity=<%u>)" DSL_MNGR_CRLF, __func__,
      nObjectIdx, nObjectsQuantity, nMethodsQuantity);

   for (unsigned int i = 0; i < g_ubusObjectsMethodsQuantityNum; ++i)
   {
      if (nObjectIdx == g_ubusObjectsMethodsQuantity[i].objectIdx)
      {
         g_ubusObjectsMethodsQuantity[i].nObjectsNum = nObjectsQuantity;
         g_ubusObjectsMethodsQuantity[i].nMethodsNum = nMethodsQuantity;
         nErrorCode = 0;
         break;
      }
   }

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: OUT (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
      nErrorCode);

   return nErrorCode;
}

static int DSL_MNGR_ObjectMethodSupportedSet(
   const DSL_MNGR_UbusObjectIdx_t nObjectIdx,
   const DSL_MNGR_UbusMethodIdx_t nMethodIdx)
{
   int nErrorCode = -1;

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: IN (object=<%u> method=<%u>)" DSL_MNGR_CRLF, __func__,
      nObjectIdx, nMethodIdx);

   for (unsigned int i = 0; i < g_ubusObjectsMethodsNum; ++i)
   {
      if (nObjectIdx == g_ubusObjectsMethods[i].objectIdx &&
         nMethodIdx == g_ubusObjectsMethods[i].methodIdx)
      {
         g_ubusObjectsMethods[i].bSupported = true;
         nErrorCode = 0;
         break;
      }
   }

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: OUT (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
      nErrorCode);

   return nErrorCode;
}

static int DSL_MNGR_UbusModuleObjectsGet(
   char ***pObjectsNames,
   unsigned int *pObjectsNamesNum)
{
   int nErrorCode = 0;

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: IN" DSL_MNGR_CRLF, __func__);

   nErrorCode = DSL_ST_UbusObjectsGet(pObjectsNames, pObjectsNamesNum);

   if (*pObjectsNames == NULL || pObjectsNamesNum == 0)
   {
      return -1;
   }

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: ubus module objects quantity=<%u>" DSL_MNGR_CRLF, __func__,
      *pObjectsNamesNum);

   for (unsigned int i = 0; i < *pObjectsNamesNum; ++i)
   {
      DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
         "%s: object '%s'" DSL_MNGR_CRLF, __func__,
         (*pObjectsNames)[i]);
   }

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: OUT (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
      nErrorCode);

   return nErrorCode;
}

static int DSL_MNGR_UbusModuleObjectsDetailsGet(
   const char *pObjectName,
   char ***pObjectMethods,
   unsigned int *pObjectMethodsNum,
   unsigned int *pObjectNum)
{
   int nErrorCode = 0;
   int nRetCode = 0;

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: IN (pObjectName='%s')" DSL_MNGR_CRLF, __func__,
      pObjectName);

   nRetCode = DSL_ST_UbusObjectsDetailsGet(pObjectName,
      pObjectMethods, pObjectMethodsNum, pObjectNum);

   if (nRetCode != 0 ||
      *pObjectMethods == NULL ||
      *pObjectMethodsNum == 0 ||
      *pObjectNum == 0)
   {
      return -1;
   }

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: object '%s' quantity=<%u>, methods quantity=<%u>" DSL_MNGR_CRLF, __func__,
      pObjectName, *pObjectNum, *pObjectMethodsNum);

   for (unsigned int i = 0; i < *pObjectMethodsNum; ++i)
   {
      DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
         "%s: method '%s'" DSL_MNGR_CRLF, __func__,
         (*pObjectMethods)[i]);
   }

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: OUT (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
      nErrorCode);

   return nErrorCode;
}

/**
 * Sync Ubus Module and DSL Manager capabilites.
 * Provide to ubus clients only supported objects and methods.
 */
static int DSL_MNGR_Sync(
   char **pObjectsNames,
   const unsigned int nObjectsNamesNum,
   unsigned int *pObjectsFoundMatchesNum,
   unsigned int *pObjectsMethodsFoundMatchesNum)
{
   int nErrorCode = 0;
   int nRetCode = 0;
   char** pObjectMethods = NULL;
   unsigned int nObjectMethodsNum = 0;
   unsigned int nObjectNum = 0;

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: IN (nObjectsNamesNum=<%u>)" DSL_MNGR_CRLF, __func__,
      nObjectsNamesNum);

   if (*pObjectsNames == NULL)
   {
      DSL_MNGR_LogError(DSL_MNGR_PREFIX
         "%s: *pObjectsNames=<%p>!" DSL_MNGR_CRLF, __func__,
         *pObjectsNames);

      return -1;
   }

   *pObjectsFoundMatchesNum = 0;
   *pObjectsMethodsFoundMatchesNum = 0;

   /* iterate over Ubus Module provided objects */
   for (unsigned int i = 0; i < nObjectsNamesNum; ++i)
   {
      bool bObjectMatch = false;
      DSL_MNGR_UbusObjectIdx_t nObjectMatchIdx = DSL_MNGR_OBJECT_FIRST_IDX;
      unsigned int nMethodsFoundMatchesNum = 0;

      /* iterate over DSL Manager supported objects */
      for (DSL_MNGR_UbusObjectIdx_t j = DSL_MNGR_OBJECT_FIRST_IDX; j < DSL_MNGR_OBJECT_LAST_IDX; ++j)
      {
         /* find match */
         if (strncmp(g_ubusObjectsNames[j].name, pObjectsNames[i],
               strlen(pObjectsNames[i])) == 0)
         {
            DSL_MNGR_LogInfo(DSL_MNGR_PREFIX
               "%s: syncing object '%s'" DSL_MNGR_CRLF, __func__,
               pObjectsNames[i]);

            /** get methods for matching object from Ubus Module */
            if ((nRetCode = DSL_MNGR_UbusModuleObjectsDetailsGet(pObjectsNames[i],
               &pObjectMethods, &nObjectMethodsNum, &nObjectNum)))
            {
               DSL_MNGR_LogWarning(DSL_MNGR_PREFIX
                  "%s: DSL_MNGR_UbusModuleObjectsDetailsGet "
                  "failed with retCode=<%d>!"DSL_MNGR_CRLF, __func__,
                  nRetCode);
            }
            else
            {
               nObjectMatchIdx = g_ubusObjectsNames[j].idx;
               ++(*pObjectsFoundMatchesNum);
               bObjectMatch = true;
               break;
            }
         }
      }

      if (bObjectMatch == false)
      {
         continue;
      }

      /* iterate over Ubus Module provided methods */
      for (unsigned int j = 0; j < nObjectMethodsNum; ++j)
      {
         DSL_MNGR_UbusMethodIdx_t nMethodMatchIdx = DSL_MNGR_METHOD_FIRST_IDX;

         /* iterate over DSL Manager supported methods */
         for (DSL_MNGR_UbusMethodIdx_t k = DSL_MNGR_METHOD_FIRST_IDX; k < DSL_MNGR_METHOD_LAST_IDX; ++k)
         {
            if (strncmp(g_ubusMethodNames[k].name, pObjectMethods[j],
                  strlen(pObjectMethods[j])) == 0)
            {
               DSL_MNGR_LogInfo(DSL_MNGR_PREFIX
                  "%s: syncing method '%s'" DSL_MNGR_CRLF, __func__,
                  pObjectMethods[j]);

               nMethodMatchIdx = g_ubusMethodNames[k].idx;

               ++nMethodsFoundMatchesNum;
               ++(*pObjectsMethodsFoundMatchesNum);

               nRetCode = DSL_MNGR_ObjectMethodSupportedSet(nObjectMatchIdx,
                  nMethodMatchIdx);

               break;
            }
         }
      }

      DSL_MNGR_ArrayRelease(&pObjectMethods, nObjectMethodsNum);

      if (nMethodsFoundMatchesNum > 0)
      {
         nRetCode = DSL_MNGR_ObjectMethodQuantitySet(nObjectMatchIdx,
            nObjectNum, nMethodsFoundMatchesNum);
      }
   }

   if (*pObjectsFoundMatchesNum == 0 ||
      *pObjectsMethodsFoundMatchesNum == 0)
   {
      nErrorCode = -1;
   }

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: OUT (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
      nErrorCode);

   return nErrorCode;
}

static int DSL_MNGR_MethodCreate(
   struct ubus_method **pMethods,
   const unsigned int nMethodsNum,
   const DSL_MNGR_UbusObjectIdx_t nObjectIdx)
{
   int nErrorCode = 0;

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: IN (object=<%u> methods quantity=<%u>)" DSL_MNGR_CRLF, __func__,
      nObjectIdx, nMethodsNum);

   /* one methods definitions per all object instances */
   *pMethods = (struct ubus_method *)
      malloc(nMethodsNum * sizeof(struct ubus_method));
   memset(*pMethods, 0,
      nMethodsNum * sizeof(struct ubus_method));

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: Created struct ubus_method array=<%p>" DSL_MNGR_CRLF, __func__,
      *pMethods);

   /* iterate over all supported by DSL Manager methods */
   for (unsigned int k = 0, n = 0; k < g_ubusObjectsMethodsNum && n < nMethodsNum; ++k)
   {
      if (nObjectIdx == g_ubusObjectsMethods[k].objectIdx &&
         g_ubusObjectsMethods[k].bSupported == true)
      {
         DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
            "%s: Created struct ubus_method entry=[%u]<%p>" DSL_MNGR_CRLF, __func__,
            n, pMethods[n]);

         (*pMethods)[n].name =
            g_ubusMethodNames[g_ubusObjectsMethods[k].methodIdx].name;

         DSL_MNGR_ObjectMethodHandlerGet(g_ubusObjectsMethods[k].objectIdx,
            g_ubusObjectsMethods[k].methodIdx, &((*pMethods)[n].handler));

         struct blobmsg_policy *pTmpPolicy = NULL;
         DSL_MNGR_ObjectMethodPolicyGet((*pMethods)[n].handler,
            &pTmpPolicy, &((*pMethods)[n].n_policy));

         (*pMethods)[n].policy = pTmpPolicy;

         DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
            "%s: Method handler=<%p> policy=<%p>" DSL_MNGR_CRLF, __func__,
            (*pMethods)[n].handler, (*pMethods)[n].policy);

         ++n;
      }
   }

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: OUT (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
      nErrorCode);

   return nErrorCode;
}

static int DSL_MNGR_ObjectFill(
   struct ubus_object *pObject,
   const unsigned int nObjectNum,
   const struct ubus_method *pMethods,
   const unsigned int nMethodsNum,
   const char *pName)
{
   int nErrorCode = 0;
   char *pInstanceName = NULL;
   unsigned int nInstanceNameLen = 0;
   /** whole int type (32 bits) size */
   const unsigned int nUbusObjectNumLenMax = 10;

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: IN (instance=<%u> methods quantity=<%u> name='%s')" DSL_MNGR_CRLF, __func__,
      nObjectNum, nMethodsNum, pName);

   if (pObject == NULL)
   {
      DSL_MNGR_LogError(DSL_MNGR_PREFIX
         "%s: pObject=<%p>!" DSL_MNGR_CRLF, __func__,
         pObject);

      return -1;
   }

   nInstanceNameLen = strlen(pName) + strlen(DSL_MNGR_SEPARATOR) + nUbusObjectNumLenMax + 1;
   pInstanceName = malloc(nInstanceNameLen);
   memset(pInstanceName, 0, nInstanceNameLen);
   snprintf(pInstanceName, nInstanceNameLen, "%s%s%u",
      pName, DSL_MNGR_SEPARATOR, nObjectNum);

   DSL_MNGR_LogInfo(DSL_MNGR_PREFIX
      "%s: Instance name '%s'" DSL_MNGR_CRLF, __func__,
      pInstanceName);

   pObject->name = pInstanceName;
   pObject->methods = pMethods;
   pObject->n_methods = nMethodsNum;
   pObject->type->name = pName;
   pObject->type->methods = pMethods;
   pObject->type->n_methods = nMethodsNum;
   pObject->subscribe_cb = DSL_MNGR_SubscribeCommonHandler;

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: OUT (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
      nErrorCode);

   return nErrorCode;
}

static int DSL_MNGR_ObjectsAdd(
   struct ubus_context *pUbusCtx)
{
   int nErrorCode = 0;
   int nRetCode = 0;

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: IN (pUbusCtx=<%p>)" DSL_MNGR_CRLF, __func__,
      pUbusCtx);

   gnObjectsFinal = 0;
   for (unsigned int i = 0; i < g_ubusObjectsMethodsQuantityNum; ++i)
   {
      if (g_ubusObjectsMethodsQuantity[i].nObjectsNum > 0)
      {
         ++gnObjectsFinal;
      }
   }

   if (gnObjectsFinal == 0)
   {
      DSL_MNGR_LogError(DSL_MNGR_PREFIX
         "%s: Missing ubus objects!" DSL_MNGR_CRLF, __func__);

      return -1;
   }
   else
   {
      DSL_MNGR_LogInfo(DSL_MNGR_PREFIX
         "%s: Objects number to be added=<%u>" DSL_MNGR_CRLF, __func__,
         gnObjectsFinal);
   }

   gpUbusObjectsFinal = (DSL_MNGR_UbusObjectFinal_t **)
      malloc(gnObjectsFinal * sizeof(DSL_MNGR_UbusObjectFinal_t *));
   memset(gpUbusObjectsFinal, 0, gnObjectsFinal * sizeof(DSL_MNGR_UbusObjectFinal_t *));

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: Created DSL_MNGR_UbusObjectFinal_t array=<%p>" DSL_MNGR_CRLF, __func__,
      gpUbusObjectsFinal);

   /* iterate over synced data */
   for (unsigned int i = 0, j = 0; i < g_ubusObjectsMethodsQuantityNum && j < gnObjectsFinal; ++i)
   {
      if (g_ubusObjectsMethodsQuantity[i].nObjectsNum > 0)
      {
         DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
            "%s: Adding object=<%u> (instances=<%u> methods=<%u>)" DSL_MNGR_CRLF, __func__,
            g_ubusObjectsMethodsQuantity[i].objectIdx,
            g_ubusObjectsMethodsQuantity[i].nObjectsNum,
            g_ubusObjectsMethodsQuantity[i].nMethodsNum);

         gpUbusObjectsFinal[j] = (DSL_MNGR_UbusObjectFinal_t *)
            malloc(sizeof(DSL_MNGR_UbusObjectFinal_t));
         memset(gpUbusObjectsFinal[j], 0, sizeof(DSL_MNGR_UbusObjectFinal_t));

         DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
            "%s: Created DSL_MNGR_UbusObjectFinal_t entry=[%u]<%p>" DSL_MNGR_CRLF, __func__,
            j, gpUbusObjectsFinal[j]);

         gpUbusObjectsFinal[j]->idx = g_ubusObjectsMethodsQuantity[i].objectIdx;
         gpUbusObjectsFinal[j]->pObjects = (struct ubus_object **)
            malloc(g_ubusObjectsMethodsQuantity[i].nObjectsNum * sizeof(struct ubus_object *));
         memset(gpUbusObjectsFinal[j]->pObjects, 0,
            g_ubusObjectsMethodsQuantity[i].nObjectsNum * sizeof(struct ubus_object *));

         DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
            "%s: Created struct ubus_object array=<%p>" DSL_MNGR_CRLF, __func__,
            gpUbusObjectsFinal[j]->pObjects);

         /* one methods definitinos per all object instances */
         struct ubus_method *pMethods = NULL;
         DSL_MNGR_MethodCreate(&pMethods, g_ubusObjectsMethodsQuantity[i].nMethodsNum,
            gpUbusObjectsFinal[j]->idx);

         /* create object instances */
         for (unsigned int k = 0; k < g_ubusObjectsMethodsQuantity[i].nObjectsNum; ++k)
         {
            gpUbusObjectsFinal[j]->pObjects[k] = (struct ubus_object *)
               malloc(sizeof(struct ubus_object));
            memset(gpUbusObjectsFinal[j]->pObjects[k], 0, sizeof(struct ubus_object));

            DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
               "%s: Created struct ubus_object entry=[%u]<%p>" DSL_MNGR_CRLF, __func__,
               k, gpUbusObjectsFinal[j]->pObjects[k]);

            gpUbusObjectsFinal[j]->pObjects[k]->type = (struct ubus_object_type *)
               malloc(sizeof(struct ubus_object_type));
            memset(gpUbusObjectsFinal[j]->pObjects[k]->type, 0, sizeof(struct ubus_object_type));

            DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
               "%s: Created struct ubus_object_type entry=<%p>" DSL_MNGR_CRLF, __func__,
               gpUbusObjectsFinal[j]->pObjects[k]->type);

            DSL_MNGR_ObjectFill(
               gpUbusObjectsFinal[j]->pObjects[k],
               k, pMethods, g_ubusObjectsMethodsQuantity[i].nMethodsNum,
               g_ubusObjectsNames[g_ubusObjectsMethodsQuantity[i].objectIdx].name);

            for (unsigned int m = 0; m < gpUbusObjectsFinal[j]->pObjects[k]->n_methods; ++m)
            {
               DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
                  "%s: Method name '%s' handler=<%p>" DSL_MNGR_CRLF, __func__,
                  gpUbusObjectsFinal[j]->pObjects[k]->methods[m].name,
                  gpUbusObjectsFinal[j]->pObjects[k]->methods[m].handler);
            }

            if ((nRetCode = ubus_add_object(pUbusCtx, gpUbusObjectsFinal[j]->pObjects[k])))
            {
               DSL_MNGR_LogWarning(DSL_MNGR_PREFIX
                  "%s: Failed to add UBUS object '%s', %s" DSL_MNGR_CRLF, __func__,
                  g_ubusObjectsNames[g_ubusObjectsMethodsQuantity[i].objectIdx].name,
                  ubus_strerror(nRetCode));

               ++nErrorCode;
            }
         }

         /* proceed with next synced object */
         ++j;
      }
   }

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: OUT (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
      nErrorCode);

   return nErrorCode;
}

static int DSL_MNGR_Init(
   struct ubus_context **pUbusCtx,
   const char *pUbusSocket)
{
   int nErrorCode = 0;
   int nRetCode = 0;
   char **pObjectsNames = NULL;
   unsigned int nObjectsNamesNum = 0;
   unsigned int nObjectsFoundMatchesNum = 0;
   unsigned int nObjectsMethodsFoundMatchesNum = 0;

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: IN (*pUbusCtx=<%p>)" DSL_MNGR_CRLF, __func__,
      *pUbusCtx);

   if (*pUbusCtx != NULL)
   {
      DSL_MNGR_LogError(DSL_MNGR_PREFIX
         "%s: Ubus context already exists!" DSL_MNGR_CRLF, __func__);

      return -1;
   }

   if (pUbusSocket == NULL ||
      strlen(pUbusSocket) == 0)
   {
      DSL_MNGR_LogError(DSL_MNGR_PREFIX
         "%s: Missing socket!" DSL_MNGR_CRLF, __func__);

      return -1;
   }

   nRetCode = DSL_MNGR_UbusModuleObjectsGet(&pObjectsNames, &nObjectsNamesNum);
   if (nRetCode)
   {
      DSL_MNGR_LogError(DSL_MNGR_PREFIX
         "%s: DSL_MNGR_UbusModuleObjectsGet retCode=<%d>!" DSL_MNGR_CRLF, __func__,
         nRetCode);

      return -1;
   }

   nRetCode = DSL_MNGR_Sync(pObjectsNames, nObjectsNamesNum,
      &nObjectsFoundMatchesNum, &nObjectsMethodsFoundMatchesNum);
   if (nRetCode)
   {
      DSL_MNGR_LogError(DSL_MNGR_PREFIX
         "%s: DSL_MNGR_Sync retCode=<%d>!" DSL_MNGR_CRLF, __func__,
         nRetCode);

      return -1;
   }
   else if (nObjectsFoundMatchesNum == 0 ||
      nObjectsMethodsFoundMatchesNum == 0)
   {
      DSL_MNGR_LogError(DSL_MNGR_PREFIX
         "%s: None supported ubus objects found!" DSL_MNGR_CRLF, __func__);

      return -1;
   }
   else
   {
      DSL_MNGR_LogInfo(DSL_MNGR_PREFIX
         "%s: Found matching total %u objects and %u methods." DSL_MNGR_CRLF, __func__,
         nObjectsFoundMatchesNum, nObjectsMethodsFoundMatchesNum);
   }

   DSL_MNGR_ArrayRelease(&pObjectsNames, nObjectsNamesNum);

   *pUbusCtx = ubus_connect(pUbusSocket);
   if (*pUbusCtx == NULL)
   {
      DSL_MNGR_LogError(DSL_MNGR_PREFIX
         "%s: ubus_connect failed!" DSL_MNGR_CRLF, __func__);
      return -1;
   }
   else
   {
      DSL_MNGR_LogInfo(DSL_MNGR_PREFIX
         "%s: ubus_connect succeed (*pUbusCtx=<%p>) vVv" DSL_MNGR_CRLF, __func__,
         *pUbusCtx);
   }

   ubus_add_uloop(*pUbusCtx);

   commonNotifier.cb = DSL_MNGR_NotifyCommonHandler;

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: OUT (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
      nErrorCode);

   return nErrorCode;
}

static int DSL_MNGR_Run(struct ubus_context *pUbusCtx)
{
   int nErrorCode = 0;
   int nRetCode = 0;

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: IN (pUbusCtx=<%p>)" DSL_MNGR_CRLF, __func__,
      pUbusCtx);

   if ((nRetCode = DSL_MNGR_ObjectsAdd(pUbusCtx)))
   {
      DSL_MNGR_LogError(DSL_MNGR_PREFIX
         "%s: DSL_MNGR_ObjectsAdd failed with retCode=<%d>!" DSL_MNGR_CRLF, __func__,
         nRetCode);

      return nRetCode;
   }

   nRetCode = uloop_run();

   uloop_done();

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: OUT (uloop=<%d>, errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
      nRetCode, nErrorCode);

   return nErrorCode;
}

static int DSL_MNGR_Deinit(struct ubus_context *pUbusCtx)
{
   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: IN (pUbusCtx=<%p>)" DSL_MNGR_CRLF, __func__,
      pUbusCtx);

   ubus_free(pUbusCtx);
   gpUbusCtx = NULL;

   DSL_MNGR_ObjectsRelease();

   /* indicate to exit main loop */
   uloop_end();

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: OUT (uloop_cancelling()=<%d>)" DSL_MNGR_CRLF, __func__,
      uloop_cancelling());

   return 0;
}

static int DSL_MNGR_LogLevelRead(
   const char* pPath,
   DSL_MNGR_LogLevel_t *pLogLevel)
{
   int nErrorCode = -1;
   FILE* fp = NULL;

   if((fp = fopen(pPath, "r")))
   {
      fscanf(fp, "log_level:%u", pLogLevel);
      fclose(fp);

      nErrorCode = 0;
   }

   if (*pLogLevel >= DSL_MNGR_LOG_LEVEL_LAST)
   {
      *pLogLevel = DSL_MNGR_LOG_LEVEL_LAST - 1;
   }

   return nErrorCode;
}

static void DSL_MNGR_NotifyCommonHandler(struct uloop_timeout *t)
{
   bool hasSubscribers = false;
   const unsigned int nChannelLineMax = 2;

   /* foreach final object... */
   for (unsigned int i = 0; i < gnObjectsFinal; ++i)
   {
      /* ...and its instances... */
      for (unsigned int j = 0; j < g_ubusObjectsMethodsQuantity[gpUbusObjectsFinal[i]->idx].nObjectsNum; ++j)
      {
         /* ...with active subscribers... */
         if (gpUbusObjectsFinal[i]->pObjects[j]->has_subscribers == false)
         {
            continue;
         }
         else
         {
            hasSubscribers = true;
         }

         /**
          * ...send notify per each group of parameters.
          *
          * no distinction between group of parameters
          * cannot subscribe just stats - need to subscribe all - info, stats and others...
          */
         for (unsigned int k = 0; k < gpUbusObjectsFinal[i]->pObjects[j]->n_methods; ++k)
         {
            /* for channel object process all lines */
            if (gpUbusObjectsFinal[i]->pObjects[j]->methods[k].n_policy > 0 &&
               gpUbusObjectsFinal[i]->pObjects[j]->methods[k].policy == DSL_MNGR_ChannelMethodPolicy)
            {
               for (unsigned int nChannelLine = 0; nChannelLine < nChannelLineMax; ++nChannelLine)
               {
                  DSL_MNGR_NotifySend(
                     gpUbusObjectsFinal[i]->pObjects[j],
                     gpUbusObjectsFinal[i]->pObjects[j]->type->name,
                     gpUbusObjectsFinal[i]->pObjects[j]->methods[k].name,
                     j, nChannelLine);
               }
            }
            else
            {
               DSL_MNGR_NotifySend(
                  gpUbusObjectsFinal[i]->pObjects[j],
                  gpUbusObjectsFinal[i]->pObjects[j]->type->name,
                  gpUbusObjectsFinal[i]->pObjects[j]->methods[k].name,
                  j, 0);
            }
         }
      }
   }

   /* again set timer */
   if (hasSubscribers)
   {
      uloop_timeout_set(t, DSL_MNGR_TIMER_NOTIFY_POLL_TIME_MS);
   }

   return;
}

static void DSL_MNGR_NotifySend(
   struct ubus_object *pObject,
   const char *pObjectName,
   const char *pMethodName,
   const unsigned int nObjectNum,
   const unsigned int nChannelLineNum)
{
   int nRetCode = 0;
   struct blob_buf bb = { 0 };
   const bool bSubscriptionMode = true;
   const int nClientAck = -1;

   blob_buf_init(&bb, 0);

   nRetCode = DSL_ST_UbusDataGet(pObjectName, pMethodName, nObjectNum,
      nChannelLineNum, &bb, bSubscriptionMode);
   /* notify only new data */
   if (nRetCode == 0 && blobmsg_data_len(bb.head))
   {
      nRetCode = ubus_notify(gpUbusCtx, pObject, pMethodName, bb.head, nClientAck);

      DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
         "%s: ubus_notify instance:'%s' method:'%s' data len=<%zu>, "
         "(retCode=<%d>)" DSL_MNGR_CRLF, __func__,
         pObjectName, pMethodName, blobmsg_data_len(bb.head), nRetCode);
   }

   blob_buf_free(&bb);

   return;
}

/**
 * Triggers subscription mode.
 * Per object instance - not per group of parameters...
 */
static void DSL_MNGR_SubscribeCommonHandler(struct ubus_context *ctx, struct ubus_object *obj)
{
   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: IN" DSL_MNGR_CRLF, __func__);

   DSL_MNGR_LogDebug("%s: %s - subscribers active: %s" DSL_MNGR_CRLF, __func__,
      obj->name, obj->has_subscribers ? "yes" : "no");

   if (obj->has_subscribers == true)
   {
      uloop_timeout_set(&commonNotifier, DSL_MNGR_TIMER_NOTIFY_POLL_TIME_MS);
   }

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: OUT" DSL_MNGR_CRLF, __func__);
}

static int DSL_MNGR_ObjectsRelease(void)
{
   unsigned int nObjectsFinal = 0;
   bool bMethodsReleased = false;

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: IN" DSL_MNGR_CRLF, __func__);

   if (gnObjectsFinal == 0)
   {
      DSL_MNGR_LogWarning(DSL_MNGR_PREFIX
         "%s: Nothing to release..." DSL_MNGR_CRLF, __func__);

      return -1;
   }

   nObjectsFinal = gnObjectsFinal;
   gnObjectsFinal = 0;

   /* iterate over synced data */
   for (unsigned int i = 0, j = 0; i < g_ubusObjectsMethodsQuantityNum && j < nObjectsFinal; ++i)
   {
      DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
         "%s: i=<%u>, j=<%u> <%p>"DSL_MNGR_CRLF, __func__,
         i, j, gpUbusObjectsFinal[j]);

      bMethodsReleased = false;

      if (g_ubusObjectsMethodsQuantity[i].nObjectsNum > 0)
      {
         /* release object instances */
         for (unsigned int k = 0; k < g_ubusObjectsMethodsQuantity[i].nObjectsNum; ++k)
         {
            DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
               "%s: k=<%u> Releasing UBUS object '%s' <%p>" DSL_MNGR_CRLF, __func__,
               k, g_ubusObjectsNames[g_ubusObjectsMethodsQuantity[i].objectIdx].name,
               gpUbusObjectsFinal[j]->pObjects[k]);

            free(gpUbusObjectsFinal[j]->pObjects[k]->type);
            gpUbusObjectsFinal[j]->pObjects[k]->type = NULL;

            free((char*)gpUbusObjectsFinal[j]->pObjects[k]->name);
            gpUbusObjectsFinal[j]->pObjects[k]->name = NULL;

            /* release methods pointer just once per all instances */
            if (bMethodsReleased == false)
            {
               free((char*)gpUbusObjectsFinal[j]->pObjects[k]->methods);
               bMethodsReleased = true;
            }
            gpUbusObjectsFinal[j]->pObjects[k]->methods = NULL;
            gpUbusObjectsFinal[j]->pObjects[k]->subscribe_cb = NULL;
            free(gpUbusObjectsFinal[j]->pObjects[k]);
            gpUbusObjectsFinal[j]->pObjects[k] = NULL;
         }

         free(gpUbusObjectsFinal[j]->pObjects);
         gpUbusObjectsFinal[j]->pObjects = NULL;
         free(gpUbusObjectsFinal[j]);
         gpUbusObjectsFinal[j] = NULL;

         /* proceed with next synced object */
         ++j;
      }
   }

   free(gpUbusObjectsFinal);
   gpUbusObjectsFinal = NULL;

   DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
      "%s: OUT" DSL_MNGR_CRLF, __func__);

   return 0;
}

static int DSL_MNGR_ArrayRelease(
   char ***pArray,
   const unsigned int nSize)
{
   if (pArray == NULL || nSize == 0)
   {
      return -1;
   }

   for (unsigned int i = 0; i < nSize; ++i)
   {
      free((*pArray)[i]);
      (*pArray)[i] = NULL;
   }

   free((*pArray));
   *pArray = NULL;

   return 0;
}

#endif /* defined(DSL_MNGR_UBUS_SUPPORT) && (DSL_MNGR_UBUS_SUPPORT == 1) */
