/******************************************************************************

         Copyright (c) 2020 Intel Corporation

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

*******************************************************************************/

#include "dsl_mgmt_manager_objects_methods.h"
#include "dsl_mgmt_ubus_interface.h"

#if defined(DSL_MNGR_UBUS_SUPPORT) && (DSL_MNGR_UBUS_SUPPORT == 1)

/**********************************************/
/* local function declarations                */
/**********************************************/

int DSL_MNGR_GetObjectNumber(
   const char *pName,
   unsigned int *pObjectNumber
);

int DSL_MNGR_CommonHandler(
   struct ubus_context *ctx, struct ubus_object *obj, struct ubus_request_data *req,
   const char *method, struct blob_attr *msg
);

int DSL_MNGR_CommonChannelHandler(
   struct ubus_context *ctx, struct ubus_object *obj, struct ubus_request_data *req,
   const char *method, struct blob_attr *msg
);

/**********************************************/
/* global function definitions                */
/**********************************************/

int DSL_MNGR_LineInfoHandle(
   struct ubus_context *ctx, struct ubus_object *obj, struct ubus_request_data *req,
   const char *method, struct blob_attr *msg)
{
   return DSL_MNGR_CommonHandler(ctx, obj, req, method, msg);
}

int DSL_MNGR_LineStatsHandle(
   struct ubus_context *ctx, struct ubus_object *obj, struct ubus_request_data *req,
   const char *method, struct blob_attr *msg)
{
   return DSL_MNGR_CommonHandler(ctx, obj, req, method, msg);
}

int DSL_MNGR_LineStatsIntervalHandle(
   struct ubus_context *ctx, struct ubus_object *obj, struct ubus_request_data *req,
   const char *method, struct blob_attr *msg)
{
   return DSL_MNGR_CommonHandler(ctx, obj, req, method, msg);
}

int DSL_MNGR_ChannelInfoHandle(
   struct ubus_context *ctx, struct ubus_object *obj, struct ubus_request_data *req,
   const char *method, struct blob_attr *msg)
{
   return DSL_MNGR_CommonChannelHandler(ctx, obj, req, method, msg);
}

int DSL_MNGR_ChannelStatsHandle(
   struct ubus_context *ctx, struct ubus_object *obj, struct ubus_request_data *req,
   const char *method, struct blob_attr *msg)
{
   return DSL_MNGR_CommonChannelHandler(ctx, obj, req, method, msg);
}

int DSL_MNGR_ChannelStatsIntervalHandle(
   struct ubus_context *ctx, struct ubus_object *obj, struct ubus_request_data *req,
   const char *method, struct blob_attr *msg)
{
   return DSL_MNGR_CommonChannelHandler(ctx, obj, req, method, msg);
}

int DSL_MNGR_BondingGroupInfoHandle(
   struct ubus_context *ctx, struct ubus_object *obj, struct ubus_request_data *req,
   const char *method, struct blob_attr *msg)
{
   return DSL_MNGR_CommonHandler(ctx, obj, req, method, msg);
}

/**********************************************/
/* local function definitions                 */
/**********************************************/

int DSL_MNGR_GetObjectNumber(
   const char *pName,
   unsigned int *pObjectNumber)
{
   int nErrorCode = -1;
   char *pLastDot = NULL;

   if ((pLastDot = strrchr(pName, '.')) != NULL)
   {
      char *pEnd;
      *pObjectNumber = (unsigned int)strtoul(pLastDot + 1, &pEnd, 10);
      nErrorCode = 0;
   }

   return nErrorCode;
}

int DSL_MNGR_CommonHandler(
   struct ubus_context *ctx, struct ubus_object *obj, struct ubus_request_data *req,
   const char *method, struct blob_attr *msg)
{
   int nErrorCode = 0;
   int nRetCode = 0;
   struct blob_buf bb = { 0 };
   unsigned int num = 0;
   const unsigned int nLineNum = 0;
   const bool bSubscriptionMode = false;

   if ((nRetCode = DSL_MNGR_GetObjectNumber(obj->name, &num)))
   {
      DSL_MNGR_LogError(DSL_MNGR_PREFIX
         "%s: Failed to get object number (retCode=<%d>)" DSL_MNGR_CRLF, __func__,
         nRetCode);

      return -1;
   }

   blob_buf_init(&bb, 0);
   if ((nRetCode = DSL_ST_UbusDataGet(obj->type->name, method, num, nLineNum, &bb, bSubscriptionMode)))
   {
      DSL_MNGR_LogError(DSL_MNGR_PREFIX
         "%s: Failed to get data '%s'(%u)->%s (retCode<%d>)!" DSL_MNGR_CRLF, __func__,
         obj->type->name, num, method, nRetCode);

      nErrorCode = -1;
   }
   else
   {
      ubus_send_reply(ctx, req, bb.head);
   }

   blob_buf_free(&bb);

   return nErrorCode;
}

int DSL_MNGR_CommonChannelHandler(
   struct ubus_context *ctx, struct ubus_object *obj, struct ubus_request_data *req,
   const char *method, struct blob_attr *msg)
{
   int nErrorCode = 0;
   int nRetCode = 0;
   struct blob_buf bb = { 0 };
   unsigned int num = 0;
   unsigned int nLine = 0;
   struct blob_attr *args[__DSL_MNGR_CHANNEL_METHOD_ARG_MAX] = { 0 };
   const bool bSubscriptionMode = false;

   if ((nRetCode = DSL_MNGR_GetObjectNumber(obj->name, &num)))
   {
      DSL_MNGR_LogError(DSL_MNGR_PREFIX
         "%s: Failed to get object number (retCode=<%d>)" DSL_MNGR_CRLF, __func__,
         nRetCode);

      return -1;
   }

   blobmsg_parse(DSL_MNGR_ChannelMethodPolicy, __DSL_MNGR_CHANNEL_METHOD_ARG_MAX,
      args, blob_data(msg), blob_len(msg));

   if (args[DSL_MNGR_CHANNEL_METHOD_ARG_LINE] == NULL)
   {
      DSL_MNGR_LogWarning(DSL_MNGR_PREFIX
         "%s: No line number provided! Assuming line 0..." DSL_MNGR_CRLF, __func__);
   }
   else
   {
      nLine = blobmsg_get_u32(args[DSL_MNGR_CHANNEL_METHOD_ARG_LINE]);

      DSL_MNGR_LogDebug(DSL_MNGR_PREFIX
         "%s: Line number=<%u>" DSL_MNGR_CRLF, __func__,
         nLine);
   }

   blob_buf_init(&bb, 0);
   if ((nRetCode = DSL_ST_UbusDataGet(obj->type->name, method, num, nLine, &bb, bSubscriptionMode)))
   {
      DSL_MNGR_LogError(DSL_MNGR_PREFIX
         "%s: Failed to get data '%s'(%u)->%s (retCode<%d>)!" DSL_MNGR_CRLF, __func__,
         obj->type->name, num, method, nRetCode);

      nErrorCode = -1;
   }
   else
   {
      ubus_send_reply(ctx, req, bb.head);
   }

   blob_buf_free(&bb);

   return nErrorCode;
}

#endif /* defined(DSL_MNGR_UBUS_SUPPORT) && (DSL_MNGR_UBUS_SUPPORT == 1) */
