/******************************************************************************

         Copyright (c) 2020 Intel Corporation

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

*******************************************************************************/

#include <stdio.h>
#include "dsl_mgmt_manager_testclient.h"
#include "dsl_mgmt_manager_testclient_interface.h"

#if defined(DSL_MNGR_UBUS_SUPPORT) && (DSL_MNGR_UBUS_SUPPORT == 1)

#include <stdlib.h>
#include <string.h>
#include <libubox/uloop.h>
#include <libubox/blobmsg.h>
#include <libubox/blobmsg_json.h>
#include <libubus.h>

/**********************************************/
/* defines                                    */
/**********************************************/

#define DSL_MNGR_TIMER_ITER_MAX 500
#define DSL_MNGR_TIMER_POLL_TIME_MS 200

/**********************************************/
/* global variable declarations               */
/**********************************************/

static struct ubus_context *gpTestClientUbusCtx = NULL;
static const char *gpUbusObjectToWatch = NULL;

/**********************************************/
/* local function declarations                */
/**********************************************/

static void DSL_MNGR_TestClientSubscriptionRemove(
   struct ubus_context *ctx,
   struct ubus_subscriber *s,
   unsigned int id
);

static int DSL_MNGR_TestClientNotify(
   struct ubus_context *ctx,
   struct ubus_object *obj,
   struct ubus_request_data *req,
   const char *method,
   struct blob_attr *msg
);

void DSL_MNGR_TestClientNotifierTimerHandle(
   struct uloop_timeout *t
);

int DSL_MNGR_TestClientInit(
   struct ubus_context **pUbusCtx,
   const char *pUbusSocket
);

int DSL_MNGR_TestClientRun(
   struct ubus_context *pUbusCtx
);

int DSL_MNGR_TestClientDeinit(
   struct ubus_context *pUbusCtx
);

#endif /* defined(DSL_MNGR_UBUS_SUPPORT) && (DSL_MNGR_UBUS_SUPPORT == 1) */

/**********************************************/
/* global function definitions                */
/**********************************************/

int DSL_MNGR_TestClient(
   const char *pUbusSocket,
   const char *pUbusObject)
{
   int nErrorCode = 0;

#if defined(DSL_MNGR_UBUS_SUPPORT) && (DSL_MNGR_UBUS_SUPPORT == 1)
   int nRetCode = 0;

   fprintf(stdout, DSL_MNGR_PREFIX
      "%s: IN (socket='%s', watch='%s')" DSL_MNGR_CRLF, __func__,
      pUbusSocket, pUbusObject);

   gpUbusObjectToWatch = pUbusObject;

   fprintf(stdout, DSL_MNGR_PREFIX
      "%s: Initializing..." DSL_MNGR_CRLF, __func__);

   if ((nRetCode = DSL_MNGR_TestClientInit(&gpTestClientUbusCtx, pUbusSocket)))
   {
      fprintf(stdout, DSL_MNGR_PREFIX
         "%s: Init stage failed (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
         nRetCode);

      return -1;
   }

   fprintf(stdout, DSL_MNGR_PREFIX
      "%s: Running main loop..." DSL_MNGR_CRLF, __func__);

   if ((nRetCode = DSL_MNGR_TestClientRun(gpTestClientUbusCtx)))
   {
      fprintf(stdout, DSL_MNGR_PREFIX
         "%s: Run stage failed (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
         nRetCode);

      return -1;
   }

   fprintf(stdout, DSL_MNGR_PREFIX
      "%s: Stopping the loop..." DSL_MNGR_CRLF, __func__);

   if ((nRetCode = DSL_MNGR_TestClientDeinit(gpTestClientUbusCtx)))
   {
      fprintf(stdout, DSL_MNGR_PREFIX
         "%s: Deinit stage failed (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
         nRetCode);

      return -1;
   }

   fprintf(stdout, DSL_MNGR_PREFIX
      "%s: OUT (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
      nErrorCode);

#else
   fprintf(stdout, DSL_MNGR_PREFIX
      "%s: Ubus framework not available!" DSL_MNGR_CRLF, __func__);

   nErrorCode = -1;
#endif /* defined(DSL_MNGR_UBUS_SUPPORT) && (DSL_MNGR_UBUS_SUPPORT == 1) */

   return nErrorCode;
}

/**********************************************/
/* local function definitions                 */
/**********************************************/

#if defined(DSL_MNGR_UBUS_SUPPORT) && (DSL_MNGR_UBUS_SUPPORT == 1)

static void DSL_MNGR_TestClientSubscriptionRemove(
   struct ubus_context *ctx,
   struct ubus_subscriber *s,
   unsigned int id)
{
   fprintf(stdout, DSL_MNGR_PREFIX
      "%s: Object id=<%u> does not exist!" DSL_MNGR_CRLF, __func__,
      id);

   return;
}

static int DSL_MNGR_TestClientNotify(
   struct ubus_context *ctx,
   struct ubus_object *obj,
   struct ubus_request_data *req,
   const char *method,
   struct blob_attr *msg)
{
   int nErrorCode = 0;
   char *str;

   str = blobmsg_format_json(msg, true);

   fprintf(stdout, DSL_MNGR_PREFIX
      "%s: Received notification '%s': %s" DSL_MNGR_CRLF, __func__,
      method, str);

   free(str);

   return nErrorCode;
}

void DSL_MNGR_TestClientNotifierTimerHandle(
   struct uloop_timeout *t)
{
   int nRetCode = 0;
   static unsigned int nLoop = 0;
   const unsigned int nLoopMax = DSL_MNGR_TIMER_ITER_MAX;
   static unsigned int nObjectId = 0;
   static struct ubus_subscriber s;

   if (gpUbusObjectToWatch == NULL ||
      strlen(gpUbusObjectToWatch) == 0)
   {
      fprintf(stdout, DSL_MNGR_PREFIX
         "%s: No object given to watch!" DSL_MNGR_CRLF, __func__);

      uloop_cancelled = true;

      return;
   }

   /**
    * on first time or until bonding_group will not be registered:
    * - if not yet - then register subscriber!!
    * - find wanted object
    * - set callbacks and subscribe changes
    */
   if (nLoop == 0 || nObjectId == 0)
   {
      if (s.obj.methods == NULL)
      {
         if ((nRetCode = ubus_register_subscriber(gpTestClientUbusCtx, &s)))
         {
            fprintf(stdout, DSL_MNGR_PREFIX
               "%s: Failed to register subscriber ('%s')!" DSL_MNGR_CRLF, __func__,
               ubus_strerror(nRetCode));
         }
      }
      else
      {
         if ((nRetCode = ubus_lookup_id(gpTestClientUbusCtx, gpUbusObjectToWatch, &nObjectId)))
         {
            fprintf(stdout, DSL_MNGR_PREFIX
               "%s: Failed to get '%s' id, ('%s')!" DSL_MNGR_CRLF, __func__,
               gpUbusObjectToWatch, ubus_strerror(nRetCode));
         }
         else
         {
            s.remove_cb = DSL_MNGR_TestClientSubscriptionRemove;
            s.cb = DSL_MNGR_TestClientNotify;

            if ((nRetCode = ubus_subscribe(gpTestClientUbusCtx, &s, nObjectId)))
            {
               fprintf(stdout, DSL_MNGR_PREFIX
                  "%s: Failed to subscribe object id=<%u> ('%s')!" DSL_MNGR_CRLF, __func__,
                  nObjectId, ubus_strerror(nRetCode));

               uloop_cancelled = true;
            }
            else
            {
               fprintf(stdout, DSL_MNGR_PREFIX
                  "%s: Watching object id=<%u>" DSL_MNGR_CRLF, __func__,
                  nObjectId);
            }
         }
      }
   }
   /* on last iteration unregister, unsubscribe and exit main loop */
   else if (nLoop == nLoopMax)
   {
      if (s.obj.methods != NULL)
      {
         if ((nRetCode = ubus_unsubscribe(gpTestClientUbusCtx, &s, nObjectId)))
         {
            fprintf(stdout, DSL_MNGR_PREFIX
               "%s: Failed to unsubscribe object id=<%u> ('%s')" DSL_MNGR_CRLF, __func__,
               nObjectId, ubus_strerror(nRetCode));
         }
         else
         {
            fprintf(stdout, DSL_MNGR_PREFIX
               "%s: Unwatching object id=<%u>" DSL_MNGR_CRLF, __func__,
               nObjectId);
         }

         if ((nRetCode = ubus_unregister_subscriber(gpTestClientUbusCtx, &s)))
         {
            fprintf(stdout, DSL_MNGR_PREFIX
               "%s: Failed to unregister subscriber ('%s')!" DSL_MNGR_CRLF, __func__,
               ubus_strerror(nRetCode));
         }
         else
         {
            fprintf(stdout, DSL_MNGR_PREFIX
               "%s: Unregistering subscriber." DSL_MNGR_CRLF, __func__);
         }
      }

      uloop_cancelled = true;
   }

   if (nLoop < nLoopMax)
   {
      ++nLoop;
      uloop_timeout_set(t, DSL_MNGR_TIMER_POLL_TIME_MS);
   }

   return;
}

int DSL_MNGR_TestClientInit(
   struct ubus_context **pUbusCtx,
   const char *pUbusSocket)
{
   int nErrorCode = 0;

   fprintf(stdout, DSL_MNGR_PREFIX
      "%s: IN (*pUbusCtx=<%p>)" DSL_MNGR_CRLF, __func__,
      *pUbusCtx);

   if (pUbusSocket == NULL ||
      strlen(pUbusSocket) == 0)
   {
      fprintf(stdout, DSL_MNGR_PREFIX
         "%s: Missing socket!" DSL_MNGR_CRLF, __func__);

      return -1;
   }

   uloop_init();

   *pUbusCtx = ubus_connect(pUbusSocket);
   if (*pUbusCtx == NULL)
   {
      fprintf(stdout, DSL_MNGR_PREFIX
         "%s: ubus_connect failed!" DSL_MNGR_CRLF, __func__);
      return -1;
   }
   else
   {
      fprintf(stdout, DSL_MNGR_PREFIX
         "%s: ubus_connect succeed (*pUbusCtx=<%p>)" DSL_MNGR_CRLF, __func__,
         *pUbusCtx);
   }

   ubus_add_uloop(*pUbusCtx);

   fprintf(stdout, DSL_MNGR_PREFIX
      "%s: OUT (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
      nErrorCode);

   return nErrorCode;
}

int DSL_MNGR_TestClientRun(struct ubus_context *pUbusCtx)
{
   int nErrorCode = 0;
   struct uloop_timeout notifier = { 0 };

   fprintf(stdout, DSL_MNGR_PREFIX
      "%s: IN (pUbusCtx=<%p>)" DSL_MNGR_CRLF, __func__,
      pUbusCtx);

   notifier.cb = DSL_MNGR_TestClientNotifierTimerHandle;
   uloop_timeout_set(&notifier, DSL_MNGR_TIMER_POLL_TIME_MS);

   uloop_run();

   fprintf(stdout, DSL_MNGR_PREFIX
      "%s: OUT (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
      nErrorCode);

   return nErrorCode;
}

int DSL_MNGR_TestClientDeinit(struct ubus_context *pUbusCtx)
{
   int nErrorCode = 0;

   fprintf(stdout, DSL_MNGR_PREFIX
      "%s: IN (pUbusCtx=<%p>)" DSL_MNGR_CRLF, __func__,
      pUbusCtx);

   ubus_free(pUbusCtx);
   uloop_done();

   fprintf(stdout, DSL_MNGR_PREFIX
      "%s: OUT (errorCode=<%d>)" DSL_MNGR_CRLF, __func__,
      nErrorCode);

   return nErrorCode;
}

#endif /* defined(DSL_MNGR_UBUS_SUPPORT) && (DSL_MNGR_UBUS_SUPPORT == 1) */
