/******************************************************************************

         Copyright (c) 2020 Intel Corporation

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

*******************************************************************************/

#ifndef _DSL_MGMT_STATUS_DEFINITIONS_H
#define _DSL_MGMT_STATUS_DEFINITIONS_H

#if defined(DSL_VRX_DEVICE_VRX518) || defined(DSL_GFAST_DEVICE_VRX618)

#include "dsl_mgmt_status_parser.h"
#include "dsl_mgmt_interface.h"

#if defined(DSL_VRX_DEVICE_VRX518)
#define DSL_ST_PLATFORM_VRX518
#elif defined(DSL_GFAST_DEVICE_VRX618)
#define DSL_ST_PLATFORM_VRX618
#endif

/*
    Definitions of status params sections (needed for example to indicate
    entire polling group - all params that have to be updated during
    one polling iteration)
*/
typedef enum{
    DSL_ST_SECTION_UNKNOWN,
    DSL_ST_SECTION_INIT,
    DSL_ST_SECTION_STATUS,
    DSL_ST_SECTION_STATS,
    DSL_ST_SECTION_CONFIG
} DSL_ST_StatusNodeSection_t;

/*
    Definitions of status params subsections (needed to create final list
    of parameters according to configured number of lines, channels and
    bonding groups in the system)
*/
typedef enum
{
   DSL_ST_SUBSECTION_COMMON,   /* neither line nor channel nr included */
   DSL_ST_SUBSECTION_LINE,     /* indicates that line nr must be added */
   DSL_ST_SUBSECTION_CHANNEL,  /* indicates that line and channel nr must be added */
#if defined(DSL_ST_PLATFORM_VRX518)
   DSL_ST_SUBSECTION_BND_GROUP /* indicates that bonding_group nr must be added */
#endif /* defined(DSL_ST_PLATFORM_VRX518) */
} DSL_ST_StatusNodeSubsection_t;

/*
   Definitions of status file parameters' types
*/
typedef enum
{
   DSL_ST_NODE_TYPE_OBJ, /* object param (no update) */
   DSL_ST_NODE_TYPE_ARRAY, /* array param */
   DSL_ST_NODE_TYPE_STRING, /* string param */
   DSL_ST_NODE_TYPE_INT, /* integer param */
   DSL_ST_NODE_TYPE_DOUBLE, /* double param */
   DSL_ST_NODE_TYPE_BOOL /* boolean param */
} DSL_ST_StatusNodeType_t;

#define STATUS_PARAM_PATH_LENGTH 256
#define STATUS_PARAM_NAME_LENGTH 64

/*
   Definitions for status file parameters' update types,
   to be used in different update functions in order to
   determine which parameters need to be updated
*/
typedef enum
{
   DSL_UPDATE_TYPE_NONE, /* only for nodes with JSON object type */
   DSL_UPDATE_TYPE_INIT, /* parameters to be updated during initialization */
   DSL_UPDATE_TYPE_POLL, /* parameters to be updated on polling timer */
   DSL_UPDATE_TYPE_EVENT /* parameters to be updated by events only */
} DSL_ST_StatusNodeUpdateType_t;

/*
   Enumeration of ubus methods supported by Ubus Module.

   Used to group parameters according to ubus client request.
*/
typedef enum
{
   DSL_ST_UBUS_GROUP_INFO = 0,
   DSL_ST_UBUS_GROUP_STATS,
   DSL_ST_UBUS_GROUP_STATS_INTERVAL,
   DSL_ST_UBUS_GROUP_NONE
} DSL_ST_StatusNodeUbusGroup_t;

/*
   JSON Status File node definition
*/
typedef struct
{
   DSL_ST_StatusNodeIdx_t eNodeIdx;
   DSL_ST_StatusNodeSection_t eSection;
   DSL_ST_StatusNodeSubsection_t eSubsection;
   DSL_ST_StatusNodeUbusGroup_t eUbusGroup;
   DSL_char_t jsonBasePath[STATUS_PARAM_PATH_LENGTH];
   DSL_char_t jsonParamName[STATUS_PARAM_NAME_LENGTH];
   DSL_ST_StatusNodeType_t eNodeType;
   DSL_ST_StatusNodeUpdateType_t eUpdateType;
} DSL_ST_StatusNode_t;

/*
   Definitions of some JSON common paths to be used in the template
   nodes (statusNodesArray[]) and to generate JSON paths
*/
/* JSON sections' base paths: */
#define DSL_ST_JSON_COMMON_PATH "dsl"
#define DSL_ST_JSON_STATUS_PATH "dsl.status"
#if defined(DSL_ST_PLATFORM_VRX618) && (INCLUDE_DSL_API_GFAST_SUPPORT == 1)
   #define DSL_ST_JSON_STATUS_FAST_PATH "dsl.status.fast"
   #define DSL_ST_JSON_STATS_FAST_PATH "dsl.stats.fast"
#endif /* defined(DSL_ST_PLATFORM_VRX618) && (INCLUDE_DSL_API_GFAST_SUPPORT == 1) */
#define DSL_ST_JSON_STATS_PATH "dsl.stats"

/* JSON sections' nodes names: */
#define DSL_ST_JSON_LINE_NODE "line"
#define DSL_ST_JSON_CHANNEL_NODE "channel"
#if defined(DSL_ST_PLATFORM_VRX518)
   #define DSL_ST_JSON_BND_GROUP_NODE "bonding_group"
#if defined(INCLUDE_DSL_FILTER_DETECTION)
   #define DSL_ST_JSON_MFD_BASIC_DATA_NODE "mfd_basic_data"
#endif /* defined(INCLUDE_DSL_FILTER_DETECTION) */
#endif /* defined(DSL_ST_PLATFORM_VRX518) */
#define DSL_ST_JSON_CONFIGURATION_NODE "configuration"

/* JSON params' names: */
#define DSL_ST_JSON_NO_PARAM ""

/*
   This array defines template structure and nodes for the intended
   status params - metadata that will be used to prepar a list of all
   actual status params (with additional line and channel numbers) needed
   to generate the final status output (see: pAllStatusParams,
   pJsonStatusParams) */
static const DSL_ST_StatusNode_t statusNodesArray[] =
{
   { DSL_ROOT,
      DSL_ST_SECTION_INIT, DSL_ST_SUBSECTION_COMMON, DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_COMMON_PATH, DSL_ST_JSON_NO_PARAM,
      DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
   /* common section */
   { DSL_LINE_ENTRIES,
      DSL_ST_SECTION_INIT, DSL_ST_SUBSECTION_COMMON,  DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_COMMON_PATH, DSL_CSP_STAT_LINE_NUMBER_OF_ENTRIES,
      DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_INIT },
   { DSL_CHANNEL_ENTRIES,
      DSL_ST_SECTION_INIT, DSL_ST_SUBSECTION_COMMON,  DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_COMMON_PATH, DSL_CSP_STAT_CHANNEL_NUMBER_OF_ENTRIES,
      DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_INIT },
   { DSL_BND_GROUP_ENTRIES,
      DSL_ST_SECTION_INIT, DSL_ST_SUBSECTION_COMMON,  DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_COMMON_PATH, DSL_CSP_STAT_BONDING_GROUP_NUMBER_OF_ENTRIES,
      DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_INIT },
   { DSL_API_VERSION,
      DSL_ST_SECTION_INIT, DSL_ST_SUBSECTION_COMMON,  DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_COMMON_PATH, DSL_CSP_STAT_API_VERSION,
      DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_INIT },
#if defined(DSL_ST_PLATFORM_VRX518)
   { DSL_DRV_VERSION,
      DSL_ST_SECTION_INIT, DSL_ST_SUBSECTION_COMMON,  DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_COMMON_PATH, DSL_CSP_STAT_LL_MEI_DRIVER_VERSION,
      DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_INIT },
#elif defined(DSL_ST_PLATFORM_VRX618)
   { DSL_DRV_VERSION,
      DSL_ST_SECTION_INIT, DSL_ST_SUBSECTION_COMMON,  DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_COMMON_PATH, DSL_CSP_STAT_LL_PMI_DRIVER_VERSION,
      DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_INIT },
#endif /* defined(DSL_ST_PLATFORM_VRX618) */
   { DSL_HW_VERSION,
      DSL_ST_SECTION_INIT, DSL_ST_SUBSECTION_COMMON,  DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_COMMON_PATH, DSL_CSP_STAT_HARDWARE_VERSION,
      DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_INIT },
   { DSL_CHIPSET_TYPE,
      DSL_ST_SECTION_INIT, DSL_ST_SUBSECTION_COMMON,  DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_COMMON_PATH, DSL_CSP_STAT_CHIPSET_TYPE,
      DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_INIT },
   /* <dsl.status> section */
   { DSL_STATUS,
      DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_COMMON, DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_STATUS_PATH, DSL_ST_JSON_NO_PARAM,
      DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
   { DSL_STATUS_LINE,
      DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_COMMON, DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_STATUS_PATH, DSL_ST_JSON_LINE_NODE,
      DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      /* <dsl.status.line.[id]> section */
      { DSL_STATUS_LINE_ID,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_NONE,
         DSL_ST_JSON_STATUS_PATH, DSL_ST_JSON_NO_PARAM,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATUS_LINE_STATUS,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_LINE_STATUS,
         DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_UPSTREAM,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_UPSTREAM,
         DSL_ST_NODE_TYPE_BOOL, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_FW_VERSION,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_FIRMWARE_VERSION,
         DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_LINK_STATUS,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_LINK_STATUS,
         DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_STD_SUPPORTED,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_STANDARDS_SUPPORTED,
         DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_XTSE,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_XTSE,
         DSL_ST_NODE_TYPE_ARRAY, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_STD_USED,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_STANDARD_USED,
         DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_XTSE_USED,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_XTSE_USED,
         DSL_ST_NODE_TYPE_ARRAY, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_ENCODING,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_LINE_ENCODING,
         DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_ALLOWED_PROFILES,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_ALLOWED_PROFILES,
         DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_CURRENT_PROFILE,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_CURRENT_PROFILE,
         DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_PWR_MGMT_STATE,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_POWER_MANAGEMENT_STATE,
         DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_SUCC_FAIL_CAUSE,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_SUCCESS_FAILURE_CAUSE,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_LAST_STATE_TRANSM,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_LAST_STATE_TRANSMITTED,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATUS_LINE_LAST_STATE_TRANSM_US,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_LAST_STATE_TRANSMITTED PATH_SEPARATOR US,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_LAST_STATE_TRANSM_DS,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_LAST_STATE_TRANSMITTED PATH_SEPARATOR DS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_US0_MASK,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_US0_MASK,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_TRELLIS,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_TRELLIS,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATUS_LINE_TRELLIS_US,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_TRELLIS PATH_SEPARATOR US,
         DSL_ST_NODE_TYPE_BOOL, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_TRELLIS_DS,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_TRELLIS PATH_SEPARATOR DS,
         DSL_ST_NODE_TYPE_BOOL, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_ACT_SNR_MODE,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_ACT_SNR_MODE,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATUS_LINE_ACT_SNR_MODE_US,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_ACT_SNR_MODE PATH_SEPARATOR US,
         DSL_ST_NODE_TYPE_BOOL, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_ACT_SNR_MODE_DS,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_ACT_SNR_MODE PATH_SEPARATOR DS,
         DSL_ST_NODE_TYPE_BOOL, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_LINE_NR,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_LINE_NUMBER,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_MAX_BIT_RATE,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_MAX_BIT_RATE,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATUS_LINE_MAX_BIT_RATE_US,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_MAX_BIT_RATE PATH_SEPARATOR US,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_MAX_BIT_RATE_DS,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_MAX_BIT_RATE PATH_SEPARATOR DS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_NOISE_MARGIN,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_NOISE_MARGIN,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATUS_LINE_NOISE_MARGIN_US,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_NOISE_MARGIN PATH_SEPARATOR US,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_NOISE_MARGIN_DS,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_NOISE_MARGIN PATH_SEPARATOR DS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_SNR_MPB,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_SNR_MPB,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATUS_LINE_SNR_MPB_US,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_SNR_MPB PATH_SEPARATOR US,
         DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_SNR_MPB_DS,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_SNR_MPB PATH_SEPARATOR DS,
         DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_POWER,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_POWER,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATUS_LINE_POWER_US,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_POWER PATH_SEPARATOR US,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_POWER_DS,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_POWER PATH_SEPARATOR DS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_XTUR_VENDOR,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_XTUR_VENDOR,
         DSL_ST_NODE_TYPE_ARRAY, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_XTUR_COUNTRY,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_XTUR_COUNTRY,
         DSL_ST_NODE_TYPE_ARRAY, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_XTUR_ANSI_STD,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_XTUR_ANSI_STD,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_XTUR_ANSI_REV,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_XTUR_ANSI_REV,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_XTUC_VENDOR,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_XTUC_VENDOR,
         DSL_ST_NODE_TYPE_ARRAY, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_XTUC_COUNTRY,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_XTUC_COUNTRY,
         DSL_ST_NODE_TYPE_ARRAY, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_XTUC_ANSI_STD,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_XTUC_ANSI_STD,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_XTUC_ANSI_REV,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_XTUC_ANSI_REV,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
#if defined(DSL_ST_PLATFORM_VRX518)
      { DSL_STATUS_LINE_UPBOKLER,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_UPBOKLER,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
#endif /* defined(DSL_ST_PLATFORM_VRX518) */
      { DSL_STATUS_LINE_UPBOKLER_PB,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_UPBOKLER_PB,
         DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_RXTHRSH_DS,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_RXTHRSH_DS,
         DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_ACT_RA_MODE,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_ACT_RA_MODE,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATUS_LINE_ACT_RA_MODE_US,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_ACT_RA_MODE PATH_SEPARATOR US,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_ACT_RA_MODE_DS,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_ACT_RA_MODE PATH_SEPARATOR DS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_SNR_MROC,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_SNR_MROC,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATUS_LINE_SNR_MROC_US,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_SNR_MROC PATH_SEPARATOR US,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_SNR_MROC_DS,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_SNR_MROC PATH_SEPARATOR DS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_ATTENUATION,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_ATTENUATION,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATUS_LINE_ATTENUATION_US,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_ATTENUATION PATH_SEPARATOR US,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_LINE_ATTENUATION_DS,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_ATTENUATION PATH_SEPARATOR DS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
#if defined(DSL_ST_PLATFORM_VRX518) && defined(INCLUDE_DSL_FILTER_DETECTION)
      { DSL_STATUS_LINE_MFD_BASIC_DATA,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_ST_JSON_MFD_BASIC_DATA_NODE,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATUS_LINE_MFD_BASIC_DATA_FILTER_DETECTION,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_ST_JSON_MFD_BASIC_DATA_NODE PATH_SEPARATOR DSL_CSP_STAT_FILTER_DETECTION,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_INIT },
      { DSL_STATUS_LINE_MFD_BASIC_DATA_METRIC3,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_ST_JSON_MFD_BASIC_DATA_NODE PATH_SEPARATOR DSL_CSP_STAT_METRIC3,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_INIT },
#endif /* defined(DSL_ST_PLATFORM_VRX518) && defined(INCLUDE_DSL_FILTER_DETECTION) */
      { DSL_STATUS_LINE_CHANNEL,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_NONE,
         DSL_ST_JSON_STATUS_PATH, DSL_ST_JSON_CHANNEL_NODE,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
         /* <dsl.status.line.[id].channel.[id]> section */
         { DSL_STATUS_LINE_CHANNEL_ID,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_NONE,
            DSL_ST_JSON_STATUS_PATH, DSL_ST_JSON_NO_PARAM,
            DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
         { DSL_STATUS_LINE_CHANNEL_STATUS,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_CHANNEL_STATUS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATUS_LINE_CHANNEL_LINKENC_SUPP,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_LINK_ENCAPSULATION_SUPPORTED,
            DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_POLL },
         { DSL_STATUS_LINE_CHANNEL_LINKENC_USED,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_LINK_ENCAPSULATION_USED,
            DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_POLL },
         { DSL_STATUS_LINE_CHANNEL_LPATH,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_LPATH,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATUS_LINE_CHANNEL_INTLVDEPTH,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_INTLVDEPTH,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATUS_LINE_CHANNEL_INTLVBLOCK,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_INTLVBLOCK,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATUS_LINE_CHANNEL_ACT_INTERL_DELAY,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_ACTUAL_INTERLEAVING_DELAY,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATUS_LINE_CHANNEL_ACTINP,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_ACTINP,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATUS_LINE_CHANNEL_INPREPORT,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_INPREPORT,
            DSL_ST_NODE_TYPE_BOOL, DSL_UPDATE_TYPE_POLL },
         { DSL_STATUS_LINE_CHANNEL_NFEC,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_NFEC,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATUS_LINE_CHANNEL_RFEC,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_RFEC,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATUS_LINE_CHANNEL_LSYMB,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_LSYMB,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATUS_LINE_CHANNEL_DATARATE,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_CURR_RATE,
            DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
         { DSL_STATUS_LINE_CHANNEL_DATARATE_US,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_CURR_RATE PATH_SEPARATOR US,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATUS_LINE_CHANNEL_DATARATE_DS,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_CURR_RATE PATH_SEPARATOR DS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATUS_LINE_CHANNEL_ACTNDR,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_ACTNDR,
            DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
         { DSL_STATUS_LINE_CHANNEL_ACTNDR_US,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_ACTNDR PATH_SEPARATOR US,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATUS_LINE_CHANNEL_ACTNDR_DS,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_ACTNDR PATH_SEPARATOR DS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATUS_LINE_CHANNEL_ACTINPREIN,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_ACTINPREIN,
            DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
         { DSL_STATUS_LINE_CHANNEL_ACTINPREIN_US,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_ACTINPREIN PATH_SEPARATOR US,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATUS_LINE_CHANNEL_ACTINPREIN_DS,
            DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_INFO,
            DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_ACTINPREIN PATH_SEPARATOR DS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
#if defined(DSL_ST_PLATFORM_VRX518)
   { DSL_STATUS_BND_GROUP,
      DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_COMMON, DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_STATUS_PATH, DSL_ST_JSON_BND_GROUP_NODE,
      DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      /* <dsl.status.bonding_group.[id]> section */
      { DSL_STATUS_BND_GROUP_ID,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_BND_GROUP, DSL_ST_UBUS_GROUP_NONE,
         DSL_ST_JSON_STATUS_PATH, DSL_ST_JSON_NO_PARAM,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATUS_BND_GROUP_BOND_SCHEMES_SUPPORTED,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_BND_GROUP, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_BOND_SCHEMES_SUPPORTED,
         DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_BND_GROUP_BOND_SCHEME,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_BND_GROUP, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_BOND_SCHEME,
         DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_BND_GROUP_DATARATE,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_BND_GROUP, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_BOND_GROUP_RATE,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATUS_BND_GROUP_DATARATE_US,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_BND_GROUP, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_BOND_GROUP_RATE PATH_SEPARATOR US,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_BND_GROUP_DATARATE_DS,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_BND_GROUP, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_PATH, DSL_CSP_STAT_BOND_GROUP_RATE PATH_SEPARATOR DS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
#endif /* defined(DSL_ST_PLATFORM_VRX518) */
#if defined(DSL_ST_PLATFORM_VRX618) && (INCLUDE_DSL_API_GFAST_SUPPORT == 1)
   /* <dsl.status.fast> section */
   { DSL_STATUS_FAST,
      DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_COMMON, DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_STATUS_FAST_PATH, DSL_ST_JSON_NO_PARAM,
      DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
   { DSL_STATUS_FAST_LINE,
      DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_COMMON, DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_STATUS_FAST_PATH, DSL_ST_JSON_LINE_NODE,
      DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      /* <dsl.status.fast.line.[id]> section */
      { DSL_STATUS_FAST_LINE_ID,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_NONE,
         DSL_ST_JSON_STATUS_FAST_PATH, DSL_ST_JSON_NO_PARAM,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATUS_FAST_LINE_ALLOWED_PROFILES,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_FAST_PATH, DSL_CSP_STAT_FAST_ALLOWED_PROFILES,
         DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_FAST_LINE_CURRENT_PROFILE,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_FAST_PATH, DSL_CSP_STAT_FAST_CURRENT_PROFILE,
         DSL_ST_NODE_TYPE_STRING, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_FAST_LINE_SNRMRMC,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_FAST_PATH, DSL_CSP_STAT_FAST_SNRMRMC,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATUS_FAST_LINE_SNRMRMC_US,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_FAST_PATH, DSL_CSP_STAT_FAST_SNRMRMC PATH_SEPARATOR US,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_FAST_LINE_SNRMRMC_DS,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_FAST_PATH, DSL_CSP_STAT_FAST_SNRMRMC PATH_SEPARATOR DS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_FAST_LINE_BITSRMCPS,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_FAST_PATH, DSL_CSP_STAT_FAST_BITSRMCPS,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATUS_FAST_LINE_BITSRMCPS_US,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_FAST_PATH, DSL_CSP_STAT_FAST_BITSRMCPS PATH_SEPARATOR US,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_FAST_LINE_BITSRMCPS_DS,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_FAST_PATH, DSL_CSP_STAT_FAST_BITSRMCPS PATH_SEPARATOR DS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATUS_FAST_LINE_MINEFTR,
         DSL_ST_SECTION_STATUS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_INFO,
         DSL_ST_JSON_STATUS_FAST_PATH, DSL_CSP_STAT_FAST_MINEFTR,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
#endif /* defined(DSL_ST_PLATFORM_VRX618) && (INCLUDE_DSL_API_GFAST_SUPPORT == 1) */
   { DSL_STATS,
      DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_COMMON, DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_STATS_PATH, DSL_ST_JSON_NO_PARAM,
      DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
   { DSL_STATS_LINE,
      DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_COMMON, DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_STATS_PATH, DSL_ST_JSON_LINE_NODE,
      DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      /* <dsl.stats.line.[id]> section */
      { DSL_STATS_LINE_ID,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_NONE,
         DSL_ST_JSON_STATS_PATH, DSL_ST_JSON_NO_PARAM,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATS_LINE_TOTAL_START,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_TOTAL_START,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATS_LINE_SHOWTIME_START,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_SHOWTIME_START,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATS_LINE_LAST_SHOWTIME_START,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_LAST_SHOWTIME_START,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATS_LINE_CURRENT_DAY_START,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_CURRENT_DAY_START,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATS_LINE_QUARTER_HOUR_START,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_QUARTER_HOUR_START,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATS_LINE_TOTAL,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_TOTAL,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATS_LINE_TOTAL_ERR_SECS,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_TOTAL PATH_SEPARATOR DSL_CSP_STAT_ERRORED_SECS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATS_LINE_TOTAL_SEV_ERR_SECS,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_TOTAL PATH_SEPARATOR DSL_CSP_STAT_SEVERELY_ERRORED_SECS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATS_LINE_SHOWTIME,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_SHOWTIME,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATS_LINE_SHOWTIME_ERR_SECS,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_ERRORED_SECS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATS_LINE_SHOWTIME_SEV_ERR_SECS,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_SEVERELY_ERRORED_SECS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATS_LINE_LAST_SHOWTIME,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_LAST_SHOWTIME,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATS_LINE_LAST_SHOWTIME_ERR_SECS,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_LAST_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_ERRORED_SECS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATS_LINE_LAST_SHOWTIME_SEV_ERR_SECS,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_LAST_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_SEVERELY_ERRORED_SECS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATS_LINE_CURRENT_DAY,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_CURRENT_DAY,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATS_LINE_CURRENT_DAY_ERR_SECS,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_CURRENT_DAY PATH_SEPARATOR DSL_CSP_STAT_ERRORED_SECS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATS_LINE_CURRENT_DAY_SEV_ERR_SECS,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_CURRENT_DAY PATH_SEPARATOR DSL_CSP_STAT_SEVERELY_ERRORED_SECS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATS_LINE_QUARTER_HOUR,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_QUARTER_HOUR,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
      { DSL_STATS_LINE_QUARTER_HOUR_ERR_SECS,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_QUARTER_HOUR PATH_SEPARATOR DSL_CSP_STAT_ERRORED_SECS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATS_LINE_QUARTER_HOUR_SEV_ERR_SECS,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
         DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_QUARTER_HOUR PATH_SEPARATOR DSL_CSP_STAT_SEVERELY_ERRORED_SECS,
         DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
      { DSL_STATS_LINE_CHANNEL,
         DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_NONE,
         DSL_ST_JSON_STATS_PATH, DSL_ST_JSON_CHANNEL_NODE,
         DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
         /* dsl.stats.line.[id].channel.[id]> section */
         { DSL_STATS_LINE_CHANNEL_ID,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_NONE,
            DSL_ST_JSON_STATS_PATH, DSL_ST_JSON_NO_PARAM,
            DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
         { DSL_STATS_LINE_CHANNEL_TOTAL_START,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_TOTAL_START,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_SHOWTIME_START,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_SHOWTIME_START,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_LAST_SHOWTIME_START,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_LAST_SHOWTIME_START,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_CURRENT_DAY_START,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_CURRENT_DAY_START,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_QUARTER_HOUR_START,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_QUARTER_HOUR_START,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_TOTAL,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_TOTAL,
            DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
         { DSL_STATS_LINE_CHANNEL_TOTAL_XTUR_FEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_TOTAL PATH_SEPARATOR DSL_CSP_STAT_XTUR_FEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_TOTAL_XTUR_CRC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_TOTAL PATH_SEPARATOR DSL_CSP_STAT_XTUR_CRC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_TOTAL_XTUC_FEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_TOTAL PATH_SEPARATOR DSL_CSP_STAT_XTUC_FEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_TOTAL_XTUC_CRC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_TOTAL PATH_SEPARATOR DSL_CSP_STAT_XTUC_CRC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_TOTAL_XTUR_HEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_TOTAL PATH_SEPARATOR DSL_CSP_STAT_XTUR_HEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_TOTAL_XTUC_HEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_TOTAL PATH_SEPARATOR DSL_CSP_STAT_XTUC_HEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_SHOWTIME,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_SHOWTIME,
            DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
         { DSL_STATS_LINE_CHANNEL_SHOWTIME_XTUR_FEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_XTUR_FEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_SHOWTIME_XTUR_CRC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_XTUR_CRC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_SHOWTIME_XTUC_FEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_XTUC_FEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_SHOWTIME_XTUC_CRC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_XTUC_CRC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_SHOWTIME_XTUR_HEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_XTUR_HEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_SHOWTIME_XTUC_HEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_XTUC_HEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_LAST_SHOWTIME,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_LAST_SHOWTIME,
            DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
         { DSL_STATS_LINE_CHANNEL_LAST_SHOWTIME_XTUR_FEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_LAST_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_XTUR_FEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_LAST_SHOWTIME_XTUR_CRC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_LAST_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_XTUR_CRC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_LAST_SHOWTIME_XTUC_FEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_LAST_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_XTUC_FEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_LAST_SHOWTIME_XTUC_CRC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_LAST_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_XTUC_CRC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_LAST_SHOWTIME_XTUR_HEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_LAST_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_XTUR_HEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_LAST_SHOWTIME_XTUC_HEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_LAST_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_XTUC_HEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_CURRENT_DAY,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_CURRENT_DAY,
            DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
         { DSL_STATS_LINE_CHANNEL_CURRENT_DAY_XTUR_FEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_CURRENT_DAY PATH_SEPARATOR DSL_CSP_STAT_XTUR_FEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_CURRENT_DAY_XTUR_CRC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_CURRENT_DAY PATH_SEPARATOR DSL_CSP_STAT_XTUR_CRC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_CURRENT_DAY_XTUC_FEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_CURRENT_DAY PATH_SEPARATOR DSL_CSP_STAT_XTUC_FEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_CURRENT_DAY_XTUC_CRC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_CURRENT_DAY PATH_SEPARATOR DSL_CSP_STAT_XTUC_CRC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_CURRENT_DAY_XTUR_HEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_CURRENT_DAY PATH_SEPARATOR DSL_CSP_STAT_XTUR_HEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_CURRENT_DAY_XTUC_HEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_CURRENT_DAY PATH_SEPARATOR DSL_CSP_STAT_XTUC_HEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_QUARTER_HOUR,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_QUARTER_HOUR,
            DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
         { DSL_STATS_LINE_CHANNEL_QUARTER_HOUR_XTUR_FEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_QUARTER_HOUR PATH_SEPARATOR DSL_CSP_STAT_XTUR_FEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_QUARTER_HOUR_XTUR_CRC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_QUARTER_HOUR PATH_SEPARATOR DSL_CSP_STAT_XTUR_CRC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_QUARTER_HOUR_XTUC_FEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_QUARTER_HOUR PATH_SEPARATOR DSL_CSP_STAT_XTUC_FEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_QUARTER_HOUR_XTUC_CRC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_QUARTER_HOUR PATH_SEPARATOR DSL_CSP_STAT_XTUC_CRC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_QUARTER_HOUR_XTUR_HEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_QUARTER_HOUR PATH_SEPARATOR DSL_CSP_STAT_XTUR_HEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_LINE_CHANNEL_QUARTER_HOUR_XTUC_HEC_ERRORS,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_CHANNEL, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_PATH, DSL_CSP_STAT_QUARTER_HOUR PATH_SEPARATOR DSL_CSP_STAT_XTUC_HEC_ERRORS,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
#if defined(DSL_ST_PLATFORM_VRX618) && (INCLUDE_DSL_API_GFAST_SUPPORT == 1)
   { DSL_STATS_FAST,
      DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_COMMON, DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_STATS_FAST_PATH, DSL_ST_JSON_NO_PARAM,
      DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
   { DSL_STATS_FAST_LINE,
      DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_COMMON, DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_STATS_FAST_PATH, DSL_ST_JSON_LINE_NODE,
      DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
         /* <dsl.stats.fast.line.[id]> section */
         { DSL_STATS_FAST_LINE_ID,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_NONE,
            DSL_ST_JSON_STATS_FAST_PATH, DSL_ST_JSON_NO_PARAM,
            DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
         { DSL_STATS_FAST_LINE_TOTAL,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_FAST_PATH, DSL_CSP_STAT_TOTAL,
            DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
         { DSL_STATS_FAST_LINE_TOTAL_SUCCESS_BSW,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_FAST_PATH, DSL_CSP_STAT_TOTAL PATH_SEPARATOR DSL_CSP_STAT_FAST_SUCCESS_BSW,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_FAST_LINE_TOTAL_SUCCESS_SRA,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_FAST_PATH, DSL_CSP_STAT_TOTAL PATH_SEPARATOR DSL_CSP_STAT_FAST_SUCCESS_SRA,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_FAST_LINE_TOTAL_SUCCESS_FRA,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_FAST_PATH, DSL_CSP_STAT_TOTAL PATH_SEPARATOR DSL_CSP_STAT_FAST_SUCCESS_FRA,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_FAST_LINE_TOTAL_SUCCESS_RPA,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_FAST_PATH, DSL_CSP_STAT_TOTAL PATH_SEPARATOR DSL_CSP_STAT_FAST_SUCCESS_RPA,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_FAST_LINE_TOTAL_SUCCESS_TIGA,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_FAST_PATH, DSL_CSP_STAT_TOTAL PATH_SEPARATOR DSL_CSP_STAT_FAST_SUCCESS_TIGA,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_FAST_LINE_SHOWTIME,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_FAST_PATH, DSL_CSP_STAT_SHOWTIME,
            DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
         { DSL_STATS_FAST_LINE_SHOWTIME_SUCCESS_BSW,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_FAST_PATH, DSL_CSP_STAT_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_FAST_SUCCESS_BSW,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_FAST_LINE_SHOWTIME_SUCCESS_SRA,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_FAST_PATH, DSL_CSP_STAT_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_FAST_SUCCESS_SRA,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_FAST_LINE_SHOWTIME_SUCCESS_FRA,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_FAST_PATH, DSL_CSP_STAT_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_FAST_SUCCESS_FRA,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_FAST_LINE_SHOWTIME_SUCCESS_RPA,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_FAST_PATH, DSL_CSP_STAT_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_FAST_SUCCESS_RPA,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_FAST_LINE_SHOWTIME_SUCCESS_TIGA,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_FAST_PATH, DSL_CSP_STAT_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_FAST_SUCCESS_TIGA,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_FAST_LINE_LAST_SHOWTIME,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_FAST_PATH, DSL_CSP_STAT_LAST_SHOWTIME,
            DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
         { DSL_STATS_FAST_LINE_LAST_SHOWTIME_SUCCESS_BSW,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_FAST_PATH, DSL_CSP_STAT_LAST_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_FAST_SUCCESS_BSW,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_FAST_LINE_LAST_SHOWTIME_SUCCESS_SRA,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_FAST_PATH, DSL_CSP_STAT_LAST_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_FAST_SUCCESS_SRA,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_FAST_LINE_LAST_SHOWTIME_SUCCESS_FRA,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_FAST_PATH, DSL_CSP_STAT_LAST_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_FAST_SUCCESS_FRA,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_FAST_LINE_LAST_SHOWTIME_SUCCESS_RPA,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_FAST_PATH, DSL_CSP_STAT_LAST_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_FAST_SUCCESS_RPA,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
         { DSL_STATS_FAST_LINE_LAST_SHOWTIME_SUCCESS_TIGA,
            DSL_ST_SECTION_STATS, DSL_ST_SUBSECTION_LINE, DSL_ST_UBUS_GROUP_STATS_INTERVAL,
            DSL_ST_JSON_STATS_FAST_PATH, DSL_CSP_STAT_LAST_SHOWTIME PATH_SEPARATOR DSL_CSP_STAT_FAST_SUCCESS_TIGA,
            DSL_ST_NODE_TYPE_INT, DSL_UPDATE_TYPE_POLL },
#endif /* defined(DSL_ST_PLATFORM_VRX618) && (INCLUDE_DSL_API_GFAST_SUPPORT == 1) */
   /* <dsl.configuration> section */
   { DSL_CONFIGURATION,
      DSL_ST_SECTION_CONFIG, DSL_ST_SUBSECTION_COMMON, DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_COMMON_PATH, DSL_ST_JSON_CONFIGURATION_NODE,
      DSL_ST_NODE_TYPE_OBJ, DSL_UPDATE_TYPE_NONE },
   { DSL_CONFIGURATION_STATUS,
      DSL_ST_SECTION_CONFIG, DSL_ST_SUBSECTION_COMMON, DSL_ST_UBUS_GROUP_NONE,
      DSL_ST_JSON_COMMON_PATH, DSL_ST_JSON_CONFIGURATION_NODE PATH_SEPARATOR DSL_CSP_STAT_CONFIG_STATUS,
      DSL_ST_NODE_TYPE_ARRAY, DSL_UPDATE_TYPE_POLL }
};
/*
   Number of all nodes in the statusNodesArray[]
*/
static const DSL_uint32_t nNodesNum =
   sizeof(statusNodesArray) / sizeof(statusNodesArray[0]);

/*
   Union of all supported status params' types
   to provide the updated params' values to
   the common, framework-specific param-update
   function in an unified way (regardless of
   the supported frameworks)
*/
typedef union
{
   DSL_uint32_t nValue;        /* integer type */
   const DSL_char_t *sValue;   /* string type */
   DSL_boolean_t bValue;       /* boolean type */
   struct {
      union {
         /* handle to an array of uint8 or uint32 */
         const DSL_uint8_t *pU8Array;
         const DSL_uint32_t *pU32Array;
         /* handle to an array of strings */
         DSL_char_t **ppStrArray;
      } uArray;
      /* size of an array */
      DSL_uint32_t nSize;
      /* input type to indicate correct handle to use
         (currently needed only to choose correct type
         of an array of integers; array of strings is
         single - doesn't need input type to distinguish)*/
      enum { U8, U32 } eInputType;
      /* output type to indicate the selected format
         of the values in the output array */
      enum { HEX, STR } eOutputType;
   } aValue;                  /* array type */
} DSL_ST_ParamValue_t;

/*
   Unified local definitions for platform
   specific types, constants and functions
*/
#if defined(DSL_ST_PLATFORM_VRX518)
/* consts */
#define DSL_ST_MAX_DSL_ENTITIES DSL_CPE_MAX_DSL_ENTITIES
#define DSL_ST_DSL_ENTITIES DSL_CPE_DSL_ENTITIES
#define DSL_ST_CHANNELS_PER_LINE DSL_CPE_CHANNELS_PER_LINE
#define DSL_ST_BND_GROUP_ENTRIES DSL_CPE_BND_GROUP_ENTRIES
/* types */
#define DSL_ST_File_t DSL_CPE_File_t
/* functions */
#define DSL_ST_FOpen DSL_CPE_FOpen
#define DSL_ST_FWrite DSL_CPE_FWrite
#define DSL_ST_FClose DSL_CPE_FClose
#define DSL_ST_strncpy_s cpe_control_strncpy_s
#define DSL_ST_snprintf_s cpe_control_snprintf_s
#define DSL_ST_strnlen_s cpe_control_strnlen_s
#define DSL_ST_strtok_s cpe_control_strtok_s
#define DSL_ST_Malloc DSL_CPE_Malloc
#define DSL_ST_Free DSL_CPE_Free

#elif defined(DSL_ST_PLATFORM_VRX618)
/* consts */
#define DSL_ST_MAX_DSL_ENTITIES DSL_MAX_DEVICE_NUMBER
#define DSL_ST_DSL_ENTITIES DSL_MAX_DEVICE_NUMBER
#define DSL_ST_CHANNELS_PER_LINE DSL_CHANNELS_PER_LINE
#define DSL_ST_BND_GROUP_ENTRIES DSL_BND_MAX_GRP_NUMBER
/* types */
#define DSL_ST_File_t DSL_File_t
/* functions */
#define DSL_ST_FOpen DSL_FOpen
#define DSL_ST_FWrite DSL_FWrite
#define DSL_ST_FClose DSL_FClose
#define DSL_ST_strncpy_s DSL_StrnCpy_s
#define DSL_ST_snprintf_s DSL_SNprintf_s
#define DSL_ST_strnlen_s DSL_StrnLen_s
#define DSL_ST_strtok_s DSL_StrTok_s
#define DSL_ST_Malloc DSL_Malloc
#define DSL_ST_Free DSL_Free

#else
#error DSL Status Parser - platform not supported!
#endif

/*
   Additional definitions for a DEBUG mode
   (memory allocation info in the runtime etc.)
*/
#if defined(DSL_ST_DEBUG_MODE) && (DSL_ST_DEBUG_MODE == 1)
DSL_ulong_t g_memory_usage;
#define DSL_ST_DEBUG_MEMINFO_ADD(size) g_memory_usage += (size)
#if defined(DSL_ST_PLATFORM_VRX518)
#define DSL_ST_DEBUG_MEMINFO_PRINT \
   do { \
   DSL_CCA_DEBUG(DSL_CCA_DBG_MSG, (DSL_CPE_PREFIX \
      "DSL_ST: (MEMINFO) Allocated memory: %lu bytes" \
      DSL_CPE_CRLF, g_memory_usage)); \
   } while(0)
#elif defined(DSL_ST_PLATFORM_VRX618)
#define DSL_ST_DEBUG_MEMINFO_PRINT \
   do { \
      DSL_FPrintf(DSL_STDOUT, \
         "DSL_ST: (MEMINFO) Allocated memory: %lu bytes" DSL_CRLF, \
         g_memory_usage); \
   } while(0)
#endif /* VRX518/VRX618 */
#else
#define DSL_ST_DEBUG_MEMINFO_ADD(size) do{}while(0)
#define DSL_ST_DEBUG_MEMINFO_PRINT do{}while(0)
#endif /* defined(DSL_ST_DEBUG_MODE) && (DSL_ST_DEBUG_MODE == 1) */

#endif /* defined(DSL_VRX_DEVICE_VRX518) || defined(DSL_GFAST_DEVICE_VRX618) */

#endif /* _DSL_MGMT_STATUS_DEFINITIONS_H */
