/******************************************************************************

         Copyright (c) 2020 Intel Corporation

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

*******************************************************************************/

#if defined(DSL_VRX_DEVICE_VRX518) || defined(DSL_GFAST_DEVICE_VRX618)
#if defined(INCLUDE_DSL_JSON_PARSING) && (INCLUDE_DSL_JSON_PARSING == 1)

#include "dsl_mgmt_status_json.h"
#include "json.h"

#if defined(WIN32) /* SIMULATION */
#define STATUS_JSON_FILE_PATH_TMP "..\\dsl_daemon\\dsl_daemon.status_tmp.json"
#define STATUS_JSON_FILE_PATH "..\\dsl_daemon\\dsl_daemon.status.json"
#else /* LINUX */
#define STATUS_JSON_FILE_PATH_TMP "/tmp/status/dsl.tmp"
#define STATUS_JSON_FILE_PATH "/tmp/status/dsl"
#endif /* defined(WIN32) */

/*
   JSON Status File node definition
*/
typedef struct DSL_ST_JsonStatusParam_t
{
   const DSL_ST_StatusNode_t *pStatusNode;
   DSL_uint16_t nLine;
   DSL_uint8_t nChannel;
   DSL_uint8_t nBndGroup;
   DSL_char_t jsonPath[STATUS_PARAM_PATH_LENGTH];
   struct DSL_ST_JsonStatusParam_t* pNext;
} DSL_ST_JsonStatusParamsList_t;
static DSL_uint32_t nJsonParamsNum = 0;

/*
   Main array to keep JSON Status File entire structure definition (metadata),
   to be used for creation of physical Status File as well as for further
   operations. Filled with data on the basis of statusNodesArray[]
*/
static DSL_ST_JsonStatusParamsList_t *pJsonStatusParams = DSL_NULL;

/*
   main buffer to keep the status file in a json format
   (this buffer will be kept for whole parser lifetime
   in order to be updated and written down to the
   destination file only in case of need - with this
   approach no status file read operation is needed)
*/
static json_object *pJsonStatusRootObj = DSL_NULL;

/*
   This separator has to be defined as an array, because
   it is used differently by some JSON functions
*/
static const char JSON_NODES_SEPARATOR[] = ".";

/*
   Macro to check 2 related pointers and free them accordingly
   if necessary
*/
#define DSL_ST_CHECK_POINTERS(pA, pB) \
do { \
   if (pA == DSL_NULL || pB == DSL_NULL) \
   { \
      if (pA == DSL_NULL && pB == DSL_NULL) \
      { \
         return DSL_NULL; \
      } \
      else if (pA == DSL_NULL) \
      { \
         DSL_ST_Free(pB); \
         pB = DSL_NULL; \
      } \
      else if (pB == DSL_NULL) \
      { \
         DSL_ST_Free(pA); \
         pA = DSL_NULL; \
      } \
      return DSL_NULL; \
   } \
} while(0)

/**********************************************/
/* start local function declarations          */
/**********************************************/

static DSL_void_t DSL_ST_JsonStatusFileCreate();

static DSL_Error_t DSL_ST_JsonStatusFileWrite();

static DSL_void_t DSL_ST_JsonParamsListCreate();

static json_object* DSL_ST_JsonNodeParentGet(
   json_object *pRoot,
   const DSL_char_t *pSectionPath
);

static const DSL_char_t* DSL_ST_JsonNodePathGet(
   const DSL_ST_StatusNodeIdx_t eNodeIdx,
   const DSL_uint16_t nLine,
   const DSL_uint8_t nChannel
);

static const DSL_char_t* DSL_ST_JsonNodeNameGet(
   const DSL_char_t* jsonPath
);

/**********************************************/
/* start global function definitions          */
/**********************************************/

DSL_void_t DSL_ST_JsonStatusInitialize()
{
   DSL_ST_JsonParamsListCreate();
   DSL_ST_JsonStatusFileCreate();
}

DSL_Error_t DSL_ST_JsonStatusUpdate()
{
   return DSL_ST_JsonStatusFileWrite();
}

/*
   This function updates selected JSON parameter
   with provided value
*/
DSL_void_t DSL_ST_JsonStatusParamUpdate(
   const DSL_ST_StatusNodeIdx_t eNodeIdx,
   const DSL_uint16_t nLine,
   const DSL_uint8_t nChannel,
   const DSL_ST_StatusNodeType_t eParamType,
   const DSL_ST_ParamValue_t *uValue
)
{
   const DSL_char_t *jsonPath = DSL_NULL;
   json_object *pParentObj = DSL_NULL, *pNewObj = DSL_NULL;

   if (pJsonStatusRootObj == DSL_NULL)
   {
#if defined(DSL_ST_PLATFORM_VRX518)
      /* TODO: unify printing for VRX518 and VRX618 */
      DSL_CCA_DEBUG(DSL_CCA_DBG_ERR, (DSL_CPE_PREFIX
         "ERROR - DSL_ST_JsonStatusParamUpdate: root object is NULL!"
         DSL_CPE_CRLF));
#elif defined(DSL_ST_PLATFORM_VRX618)
      DSL_FPrintf(DSL_STDOUT,
         "ERROR - DSL_ST_JsonStatusParamUpdate: root object is NULL!"
         DSL_CRLF);
#endif /* defined(DSL_ST_PLATFORM_VRX518) */
      return;
   }

   jsonPath = DSL_ST_JsonNodePathGet(eNodeIdx, nLine, nChannel);
   if (jsonPath == DSL_NULL)
   {
#if defined(DSL_ST_PLATFORM_VRX518)
      /* TODO: unify printing for VRX518 and VRX618 */
      DSL_CCA_DEBUG(DSL_CCA_DBG_ERR, (DSL_CPE_PREFIX
         "ERROR - DSL_ST_JsonStatusParamUpdate: node index (%d) undefined!"
         DSL_CPE_CRLF, (DSL_int_t)eNodeIdx));
#elif defined(DSL_ST_PLATFORM_VRX618)
      DSL_FPrintf(DSL_STDOUT,
         "ERROR - DSL_ST_JsonStatusParamUpdate: node index (%d) undefined!"
         DSL_CRLF, (DSL_int_t)eNodeIdx);
#endif /* defined(DSL_ST_PLATFORM_VRX518) */
      return;
   }

   switch (eParamType)
   {
   case DSL_ST_NODE_TYPE_ARRAY:
      if (uValue->aValue.eOutputType == HEX)
      {
         /* biggest possible value would be "0xFF" */
         char sHex[MAX_JSON_HEX_STR_LEN] = { 0 };
         /* create json array object */
         pNewObj = json_object_new_array();

         /* update json object */
         for (DSL_uint32_t i = 0; i < uValue->aValue.nSize; ++i)
         {
            if (uValue->aValue.eInputType == U8)
            {
               DSL_ST_snprintf_s(sHex, sizeof(sHex),
                  "0x%02X", uValue->aValue.uArray.pU8Array[i]);
            }
            else if (uValue->aValue.eInputType == U32)
            {
               DSL_ST_snprintf_s(sHex, sizeof(sHex),
                  "0x%02X", uValue->aValue.uArray.pU32Array[i]);
            }
            else
            {
               break;
            }
            sHex[MAX_JSON_HEX_STR_LEN - 1] = '\0';

            json_object_array_add(pNewObj,
               json_object_new_string(sHex));
         }
      }
      else if (uValue->aValue.eOutputType == STR)
      {
         char sStr[MAX_JSON_ARR_STR_LEN] = { 0 };
         /* create json array object */
         pNewObj = json_object_new_array();

         /* update json object */
         for (DSL_uint32_t i = 0; i < uValue->aValue.nSize; ++i)
         {
            memset(sStr, 0, MAX_JSON_ARR_STR_LEN);
            DSL_ST_strncpy_s(sStr, MAX_JSON_ARR_STR_LEN,
               uValue->aValue.uArray.ppStrArray[i],
               DSL_ST_strnlen_s(uValue->aValue.uArray.ppStrArray[i],
                  MAX_JSON_ARR_STR_LEN - 1));

            json_object_array_add(pNewObj,
               json_object_new_string(sStr));
         }
      }
      break;
   case DSL_ST_NODE_TYPE_STRING:
      pNewObj = json_object_new_string(uValue->sValue);
      break;
   case DSL_ST_NODE_TYPE_INT:
      pNewObj = json_object_new_int(uValue->nValue);
      break;
   case DSL_ST_NODE_TYPE_DOUBLE:
      pNewObj = json_object_new_double(uValue->nValue);
      break;
   case DSL_ST_NODE_TYPE_BOOL:
      pNewObj = json_object_new_boolean(uValue->bValue);
      break;
   default:
      break;
   }

   if(pNewObj != DSL_NULL)
   {
      pParentObj = DSL_ST_JsonNodeParentGet(pJsonStatusRootObj, jsonPath);
      json_object_object_add(pParentObj,
         DSL_ST_JsonNodeNameGet(jsonPath), pNewObj);
   }
}


/**********************************************/
/* start local function definitions          */
/**********************************************/

/**
   This function creates JSON status file and stores it in the
   common JSON object for further use
*/
static DSL_void_t DSL_ST_JsonStatusFileCreate()
{
   json_object *pParentObj = DSL_NULL, *pNewObj = DSL_NULL;
   DSL_ST_JsonStatusParamsList_t *pStatusParamsListItem = DSL_NULL;

   if (pJsonStatusParams != DSL_NULL)
   {
      pStatusParamsListItem = pJsonStatusParams;
   }
   else
   {
      #if defined(DSL_ST_PLATFORM_VRX518)
      /* TODO: unify printing for VRX518 and VRX618 */
      DSL_CCA_DEBUG(DSL_CCA_DBG_ERR, (DSL_CPE_PREFIX
         "ERROR - DSL_ST_JsonStatusFileCreate: status params "
         "list not initialized!" DSL_CPE_CRLF));
#endif /* defined(DSL_ST_PLATFORM_VRX518) */
      return;
   }

   pJsonStatusRootObj = json_object_new_object();
   if (pJsonStatusRootObj == DSL_NULL)
   {
#if defined(DSL_ST_PLATFORM_VRX518)
      /* TODO: unify printing for VRX518 and VRX618 */
      DSL_CCA_DEBUG(DSL_CCA_DBG_ERR, (DSL_CPE_PREFIX
         "ERROR - DSL_ST_JsonStatusFileCreate: root object is NULL!"
         DSL_CPE_CRLF));
#endif /* defined(DSL_ST_PLATFORM_VRX518) */
      return;
   }

   while (pStatusParamsListItem != DSL_NULL)
   {
      pParentObj = DSL_ST_JsonNodeParentGet(
         pJsonStatusRootObj, pStatusParamsListItem->jsonPath);

      /* to prevent json_object_object_add function from asserting */
      if (pParentObj != DSL_NULL &&
         json_object_get_type(pParentObj) == json_type_object)
      {
         switch (pStatusParamsListItem->pStatusNode->eNodeType)
         {
         case DSL_ST_NODE_TYPE_OBJ:
            pNewObj = json_object_new_object();
            break;
         case DSL_ST_NODE_TYPE_ARRAY:
            pNewObj = json_object_new_array();
            break;
         case DSL_ST_NODE_TYPE_STRING:
            pNewObj = json_object_new_string("");
            break;
         case DSL_ST_NODE_TYPE_INT:
            pNewObj = json_object_new_int(0);
            break;
         default:
            break;
         }

         if (pNewObj)
         {
            json_object_object_add(pParentObj,
               DSL_ST_JsonNodeNameGet(pStatusParamsListItem->jsonPath), pNewObj);

            pNewObj = DSL_NULL;
         }
      }

      /* go to the next param in the list */
      pStatusParamsListItem = pStatusParamsListItem->pNext;
   }
}

static DSL_Error_t DSL_ST_JsonStatusFileWrite()
{
   const DSL_char_t * pBufferOut = DSL_NULL;
   DSL_ST_File_t *pFile = DSL_NULL;
   DSL_size_t nBufSize = 0;
   /* The size of pJsonStatusRootObj is growing in runtime
      due to increasing parameters' values. Upper limit
      is needed for safe_C function and is taken from debug
      calculations, assuming that the AVERAGE LENGTH of each
      parameter's name and its value will never exceeds
      50 characters (needs to be updated if necessary) */
   const DSL_size_t nMaxBufSize = (DSL_size_t)(nJsonParamsNum) * 50;

   if (pJsonStatusRootObj == DSL_NULL)
   {
#if defined(DSL_ST_PLATFORM_VRX518)
      /* TODO: unify printing for VRX518 and VRX618 */
      DSL_CCA_DEBUG(DSL_CCA_DBG_ERR, (DSL_CPE_PREFIX
         "ERROR - DSL_ST_JsonStatusFileWrite: root object is NULL!"
         DSL_CPE_CRLF));
#endif /* defined(DSL_ST_PLATFORM_VRX518) */
      return DSL_ERROR;
   }

   /* Write updated data to file */
   pBufferOut = json_object_to_json_string_ext(pJsonStatusRootObj,
      JSON_C_TO_STRING_PRETTY);
   pFile = DSL_ST_FOpen(STATUS_JSON_FILE_PATH_TMP, "w");
   if (pFile == DSL_NULL)
   {
#if defined(DSL_ST_PLATFORM_VRX518)
      /* TODO: unify printing for VRX518 and VRX618 */
      DSL_CCA_DEBUG(DSL_CCA_DBG_ERR, (DSL_CPE_PREFIX
         "ERROR - DSL_ST_JsonStatusFileWrite: failed to open status file!"
         DSL_CPE_CRLF));
#endif /* defined(DSL_ST_PLATFORM_VRX518) */
      return DSL_ERROR;
   }

   nBufSize = DSL_ST_strnlen_s(pBufferOut, nMaxBufSize);
   if (nBufSize == nMaxBufSize)
   {
#if defined(DSL_ST_PLATFORM_VRX518)
      DSL_CCA_DEBUG(DSL_CCA_DBG_WRN, (DSL_CPE_PREFIX
         "DSL: WARNING - status parser max buffer size might be exceeded!"
         DSL_CPE_CRLF));
#elif defined(DSL_ST_PLATFORM_VRX618)
      DSL_FPrintf(DSL_STDOUT,
         "DSL: WARNING - status parser max buffer size might be exceeded!"
         DSL_CRLF);
#endif /* VRX518/VRX618 */
   }

   DSL_ST_FWrite(
      pBufferOut,
      sizeof(DSL_char_t),
      nBufSize,
      pFile);
   DSL_ST_FClose(pFile);

   /* execute atomic rename operation to store the status file in the final
      destination and keep it locked as short as possible */
   rename(STATUS_JSON_FILE_PATH_TMP, STATUS_JSON_FILE_PATH);

   return DSL_SUCCESS;
}

/*
   This function returns common section JSON path
*/
static DSL_char_t* DSL_ST_JsonStatusCommonPathCreate(
   const DSL_uint32_t nNode
)
{
   DSL_char_t* sJsonPath =
      (DSL_char_t*)DSL_ST_Malloc(STATUS_PARAM_PATH_LENGTH);

   if (sJsonPath == DSL_NULL)
   {
#if defined(DSL_ST_PLATFORM_VRX518)
      DSL_CCA_DEBUG(DSL_CCA_DBG_WRN, (DSL_CPE_PREFIX
         "DSL: WARNING - Memory allocation for Common Path failed!"
         DSL_CPE_CRLF));
#elif defined(DSL_ST_PLATFORM_VRX618)
      DSL_FPrintf(DSL_STDOUT,
         "DSL: WARNING - Memory allocation for Common Path failed!"
         DSL_CRLF);
#endif /* VRX518/VRX618 */
      return DSL_NULL;
   }
   memset(sJsonPath, 0, STATUS_PARAM_PATH_LENGTH);

   if (strcmp(statusNodesArray[nNode].jsonParamName, DSL_ST_JSON_NO_PARAM) == 0)
   {
      /* no param name defined, take only base path */
      DSL_ST_snprintf_s(sJsonPath, STATUS_PARAM_PATH_LENGTH, "%s",
         statusNodesArray[nNode].jsonBasePath);
   }
   else
   {
      /* concatenate base path with the param name */
      DSL_ST_snprintf_s(sJsonPath, STATUS_PARAM_PATH_LENGTH, "%s.%s",
         statusNodesArray[nNode].jsonBasePath,
         statusNodesArray[nNode].jsonParamName);
   }

   return sJsonPath;
}

/*
   This function returns line section JSON path
   (with line number)
*/
static DSL_char_t* DSL_ST_JsonStatusLinePathCreate(
   const DSL_uint32_t nNode,
   const DSL_uint16_t nLine
)
{
   DSL_char_t *sJsonPath, *sJsonTmpPath;

   sJsonPath = (DSL_char_t*)DSL_ST_Malloc(STATUS_PARAM_PATH_LENGTH);
   sJsonTmpPath = (DSL_char_t*)DSL_ST_Malloc(STATUS_PARAM_PATH_LENGTH);
   DSL_ST_CHECK_POINTERS(sJsonPath, sJsonTmpPath);

   memset(sJsonPath, 0, STATUS_PARAM_PATH_LENGTH);
   memset(sJsonTmpPath, 0, STATUS_PARAM_PATH_LENGTH);

   /* add "line" keyword to the path */
   DSL_ST_snprintf_s(sJsonPath, STATUS_PARAM_PATH_LENGTH, "%s.%s",
      statusNodesArray[nNode].jsonBasePath, DSL_ST_JSON_LINE_NODE);
   if (strcmp(statusNodesArray[nNode].jsonParamName, DSL_ST_JSON_NO_PARAM) == 0)
   {
      /* no param name defined, add only line number */
      DSL_ST_snprintf_s(sJsonTmpPath, STATUS_PARAM_PATH_LENGTH, "%s.%d",
         sJsonPath, nLine);
   }
   else
   {
      DSL_ST_snprintf_s(sJsonTmpPath, STATUS_PARAM_PATH_LENGTH, "%s.%d.%s",
         sJsonPath, nLine, statusNodesArray[nNode].jsonParamName);
   }

   /* set the final line path */
   DSL_ST_strncpy_s(sJsonPath, STATUS_PARAM_PATH_LENGTH,
      sJsonTmpPath, STATUS_PARAM_PATH_LENGTH);
   DSL_ST_Free(sJsonTmpPath);

   return sJsonPath;
}

/*
   This function returns channel section JSON path
   (with line and channel number)
*/
static DSL_char_t* DSL_ST_JsonStatusChannelPathCreate(
   const DSL_uint32_t nNode,
   const DSL_uint16_t nLine,
   const DSL_uint8_t nChannel
)
{
   DSL_char_t *sJsonPath, *sJsonTmpPath;

   sJsonPath = (DSL_char_t*)DSL_ST_Malloc(STATUS_PARAM_PATH_LENGTH);
   sJsonTmpPath = (DSL_char_t*)DSL_ST_Malloc(STATUS_PARAM_PATH_LENGTH);
   DSL_ST_CHECK_POINTERS(sJsonPath, sJsonTmpPath);

   memset(sJsonPath, 0, STATUS_PARAM_PATH_LENGTH);
   memset(sJsonTmpPath, 0, STATUS_PARAM_PATH_LENGTH);

   /* add "line" (with line number) and "channel" keyword to the path */
   DSL_ST_snprintf_s(sJsonPath, STATUS_PARAM_PATH_LENGTH, "%s.%s.%d.%s",
      statusNodesArray[nNode].jsonBasePath, DSL_ST_JSON_LINE_NODE,
      nLine, DSL_ST_JSON_CHANNEL_NODE);
   if (strcmp(statusNodesArray[nNode].jsonParamName, DSL_ST_JSON_NO_PARAM) == 0)
   {
      /* no param name defined, add only channel number */
      DSL_ST_snprintf_s(sJsonTmpPath, STATUS_PARAM_PATH_LENGTH, "%s.%d",
         sJsonPath, nChannel);
   }
   else
   {
      DSL_ST_snprintf_s(sJsonTmpPath, STATUS_PARAM_PATH_LENGTH, "%s.%d.%s",
         sJsonPath, nChannel , statusNodesArray[nNode].jsonParamName);
   }

   /* set the final channel path */
   DSL_ST_strncpy_s(sJsonPath, STATUS_PARAM_PATH_LENGTH,
      sJsonTmpPath, STATUS_PARAM_PATH_LENGTH);
   DSL_ST_Free(sJsonTmpPath);

   return sJsonPath;
}

#if defined(DSL_ST_PLATFORM_VRX518)
/*
   This function returns bonding group section JSON path
   (with bonding group number)
*/
static DSL_char_t* DSL_ST_JsonStatusBndGroupPathCreate(
   const DSL_uint32_t nNode,
   const DSL_uint8_t nBndGroup
)
{
   DSL_char_t *sJsonPath, *sJsonTmpPath;

   sJsonPath = (DSL_char_t*)DSL_ST_Malloc(STATUS_PARAM_PATH_LENGTH);
   sJsonTmpPath = (DSL_char_t*)DSL_ST_Malloc(STATUS_PARAM_PATH_LENGTH);
   DSL_ST_CHECK_POINTERS(sJsonPath, sJsonTmpPath);

   memset(sJsonPath, 0, STATUS_PARAM_PATH_LENGTH);
   memset(sJsonTmpPath, 0, STATUS_PARAM_PATH_LENGTH);

   /* add "bonding_group" keyword to the path */
   DSL_ST_snprintf_s(sJsonPath, STATUS_PARAM_PATH_LENGTH, "%s.%s",
      statusNodesArray[nNode].jsonBasePath, DSL_ST_JSON_BND_GROUP_NODE);
   if (strcmp(statusNodesArray[nNode].jsonParamName, DSL_ST_JSON_NO_PARAM) == 0)
   {
      /* no param name defined, add only bonding_group number */
      DSL_ST_snprintf_s(sJsonTmpPath, STATUS_PARAM_PATH_LENGTH, "%s.%d",
         sJsonPath, nBndGroup);
   }
   else
   {
      DSL_ST_snprintf_s(sJsonTmpPath, STATUS_PARAM_PATH_LENGTH, "%s.%d.%s",
         sJsonPath, nBndGroup, statusNodesArray[nNode].jsonParamName);
   }

   /* set the final bonding_group path */
   DSL_ST_strncpy_s(sJsonPath, STATUS_PARAM_PATH_LENGTH,
      sJsonTmpPath, STATUS_PARAM_PATH_LENGTH);
   DSL_ST_Free(sJsonTmpPath);

   return sJsonPath;
}
#endif /* defined(DSL_ST_PLATFORM_VRX518) */

/*
   This function adds item to the global pJsonStatusParams list
*/
static DSL_void_t DSL_ST_JsonParamsListItemAdd(
   const DSL_ST_StatusNode_t* pNode,
   const DSL_uint16_t nLine,
   const DSL_uint8_t nChannel,
   const DSL_uint8_t nBndGroup,
   const DSL_char_t *pJsonPath
)
{
   DSL_ST_JsonStatusParamsList_t *pJsonParam =
      (DSL_ST_JsonStatusParamsList_t*)
      DSL_ST_Malloc(sizeof(DSL_ST_JsonStatusParamsList_t));

   if (pJsonParam != DSL_NULL)
   {
      /* create new status parameter */
      pJsonParam->pStatusNode = pNode;
      pJsonParam->nLine = nLine;
      pJsonParam->nChannel = nChannel;
      pJsonParam->nBndGroup = nBndGroup;
      DSL_ST_strncpy_s(pJsonParam->jsonPath,
         STATUS_PARAM_PATH_LENGTH, pJsonPath, STATUS_PARAM_PATH_LENGTH);
      pJsonParam->pNext = DSL_NULL;

      /* update pJsonStatusParams list with the new parameter */
      if (pJsonStatusParams == DSL_NULL)
      {
         pJsonStatusParams = pJsonParam;
      }
      else
      {
         DSL_ST_JsonStatusParamsList_t* pHead = pJsonStatusParams;
         while (pHead->pNext != DSL_NULL)
         {
            pHead = pHead->pNext;
         }

         pHead->pNext = pJsonParam;
      }

      /* update global variable keeping number of parameters */
      nJsonParamsNum++;
   }
}

/*
   This function creates list of all JSON status parameters
   with line and channel numbers and the related JSON path,
   on the basis of "nodes" defined in the statusNodesArray array
   and number of lines and channels in the current configuration
*/
static DSL_void_t DSL_ST_JsonParamsListCreate()
{
   DSL_uint16_t nLine = 0;
   DSL_uint8_t nChannel = 0, nBndGroup = 0;
   DSL_char_t* sJsonPath = DSL_NULL;

   /* iterate through all nodes in the statusNodesArray array
      and create appropriate JSON status parameter */
   for (DSL_uint32_t nNode = 0; nNode < nNodesNum; ++nNode)
   {
      if (statusNodesArray[nNode].eSubsection  == DSL_ST_SUBSECTION_COMMON)
      {
#if defined(DSL_ST_PLATFORM_VRX518)
         if (statusNodesArray[nNode].eNodeIdx == DSL_STATUS_BND_GROUP &&
            !INCLUDE_DSL_BONDING)
         {
            /* skip this node in single link mode */
            continue;
         }
#endif /* defined(DSL_ST_PLATFORM_VRX518) */
         /* create a common JSON path (without a line and channel number)
            and add it to the list */
         sJsonPath = DSL_ST_JsonStatusCommonPathCreate(nNode);
         if (sJsonPath != DSL_NULL)
         {
            DSL_ST_JsonParamsListItemAdd(
               &statusNodesArray[nNode],
               nLine, nChannel, nBndGroup, sJsonPath);
            DSL_ST_Free(sJsonPath);
            sJsonPath = DSL_NULL;
         }
      }
#if defined(DSL_ST_PLATFORM_VRX518)
      else if (statusNodesArray[nNode].eSubsection  == DSL_ST_SUBSECTION_BND_GROUP &&
         INCLUDE_DSL_BONDING)
      {
         /* for CPE nBndGroup is always 0 */
         sJsonPath = DSL_ST_JsonStatusBndGroupPathCreate(nNode, nBndGroup);
         if (sJsonPath != DSL_NULL)
         {
            DSL_ST_JsonParamsListItemAdd(
               &statusNodesArray[nNode],
               nLine, nChannel, nBndGroup, sJsonPath);
            DSL_ST_Free(sJsonPath);
            sJsonPath = DSL_NULL;
         }
      }
#endif /* defined(DSL_ST_PLATFORM_VRX518) */
      else if (statusNodesArray[nNode].eSubsection  == DSL_ST_SUBSECTION_LINE
         || statusNodesArray[nNode].eSubsection  == DSL_ST_SUBSECTION_CHANNEL)
      {
         for (nLine = 0; nLine < DSL_ST_DSL_ENTITIES; ++nLine)
         {
            if (statusNodesArray[nNode].eSubsection  == DSL_ST_SUBSECTION_LINE)
            {
               /* create a JSON path with a line number
                  and add it to the list */
               sJsonPath = DSL_ST_JsonStatusLinePathCreate(nNode, nLine);
               if (sJsonPath != DSL_NULL)
               {
                  DSL_ST_JsonParamsListItemAdd(
                     &statusNodesArray[nNode],
                     nLine, nChannel, nBndGroup, sJsonPath);
                  DSL_ST_Free(sJsonPath);
                  sJsonPath = DSL_NULL;
               }
            }
            else if (statusNodesArray[nNode].eSubsection  == DSL_ST_SUBSECTION_CHANNEL)
            {
               for (nChannel = 0;
                  nChannel < DSL_ST_CHANNELS_PER_LINE; ++nChannel)
               {
                  /* create a JSON path with a line and channel number
                     and add it to the list */
                  sJsonPath = DSL_ST_JsonStatusChannelPathCreate(
                     nNode, nLine, nChannel);
                  if (sJsonPath != DSL_NULL)
                  {
                     DSL_ST_JsonParamsListItemAdd(
                        &statusNodesArray[nNode],
                        nLine, nChannel, nBndGroup, sJsonPath);
                     DSL_ST_Free(sJsonPath);
                     sJsonPath = DSL_NULL;
                  }
               }
               /* reset nChannel value for line-only (no channel) params,
                  in order to not to store incorrect (non-zero) channel
                  value in them */
               nChannel = 0;
            }
         }
         /* reset nLine value for common (no line and no channel) params,
            in order to not to store incorrect (non-zero) line value in them */
         nLine = 0;
      }
   }
}

/**
   This function returns parent node for the given json node
*/
static json_object* DSL_ST_JsonNodeParentGet(
   json_object *pRoot,
   const DSL_char_t *pSectionPath)
{
   typedef struct
   {
      DSL_char_t *pName;
      json_object *pJsonObj;
   } DSL_ST_JsonNode_t;
   DSL_char_t inputPath[STATUS_PARAM_PATH_LENGTH];
   DSL_ST_JsonNode_t visitedNode;
   DSL_char_t *pToken = DSL_NULL;
   DSL_size_t nStrMax = STATUS_PARAM_PATH_LENGTH;
   DSL_char_t *pNextToken = DSL_NULL;

   memset(inputPath, 0, sizeof(inputPath));
   DSL_ST_strncpy_s(inputPath, sizeof(inputPath),
      pSectionPath, sizeof(inputPath));
   nStrMax = DSL_ST_strnlen_s(inputPath, STATUS_PARAM_PATH_LENGTH - 1) + 1;

   memset(&visitedNode, 0, sizeof(visitedNode));
   visitedNode.pJsonObj = pRoot;

   pToken = DSL_ST_strtok_s(
      inputPath, &nStrMax, JSON_NODES_SEPARATOR, &pNextToken);
   while (pToken)
   {
      visitedNode.pName = pToken;
      json_object_object_foreach(visitedNode.pJsonObj, key, val)
      {
         if (strcmp(key, visitedNode.pName) == 0)
         {
            if (json_object_get_type(val) == json_type_object)
            {
               visitedNode.pJsonObj = val;
            }
            break;
         }
      }
      pToken = DSL_ST_strtok_s(
         NULL, &nStrMax, JSON_NODES_SEPARATOR, &pNextToken);
   }

   return visitedNode.pJsonObj;
}

/**
   This function returns full json path for the given node index
*/
static const DSL_char_t* DSL_ST_JsonNodePathGet(
   const DSL_ST_StatusNodeIdx_t eNodeIdx,
   const DSL_uint16_t nLine,
   const DSL_uint8_t nChannel)
{
   const DSL_char_t *sOutputPath = DSL_NULL;
   DSL_ST_JsonStatusParamsList_t *pStatusParamsListItem = pJsonStatusParams;

   while (pStatusParamsListItem != DSL_NULL)
   {
      if (pStatusParamsListItem->pStatusNode->eNodeIdx == eNodeIdx &&
          (pStatusParamsListItem->pStatusNode->eSubsection
            == DSL_ST_SUBSECTION_COMMON ||
#if defined(DSL_ST_PLATFORM_VRX518)
           pStatusParamsListItem->pStatusNode->eSubsection
            == DSL_ST_SUBSECTION_BND_GROUP ||
#endif /* defined(DSL_ST_PLATFORM_VRX518 */
           (pStatusParamsListItem->nLine == nLine &&
            pStatusParamsListItem->nChannel == nChannel)))
      {
         sOutputPath = pStatusParamsListItem->jsonPath;
         break;
      }

      /* go to the next param in the list */
      pStatusParamsListItem = pStatusParamsListItem->pNext;
   }

   return sOutputPath;
}

/**
   This function returns key (last node) name for the given json path
*/
static const DSL_char_t* DSL_ST_JsonNodeNameGet(
   const DSL_char_t* jsonPath)
{
   const char *jsonLastNode =
      strrchr(jsonPath, (int)JSON_NODES_SEPARATOR[0]);
   return jsonLastNode ? jsonLastNode + 1 : jsonPath;
}

#endif /* defined(INCLUDE_DSL_JSON_PARSING) && (INCLUDE_DSL_JSON_PARSING == 1) */
#endif /* defined(DSL_VRX_DEVICE_VRX518) || defined(DSL_GFAST_DEVICE_VRX618) */