/******************************************************************************

         Copyright (c) 2020, MaxLinear, Inc.
         Copyright (c) 2020 Intel Corporation

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

*******************************************************************************/

#include "dsl_mgmt_ubus_interface.h"

#if defined(DSL_VRX_DEVICE_VRX518)

#include "dsl_mgmt_status_ubus.h"

#if defined(INCLUDE_DSL_STATUS_UBUS_SUPPORT) && (INCLUDE_DSL_STATUS_UBUS_SUPPORT == 1)

#include "json.h"
#include <libubox/blobmsg_json.h>

/**********************************************/
/* defines                                    */
/**********************************************/

#undef DSL_CCA_DBG_BLOCK
#define DSL_CCA_DBG_BLOCK DSL_CCA_DBG_APP

#define DSL_UBUS_SEPARATOR "."

#define DSL_UBUS_OBJECT_DSL "dsl"
#define DSL_UBUS_OBJECT_LINE DSL_UBUS_OBJECT_DSL DSL_UBUS_SEPARATOR "line"
#define DSL_UBUS_OBJECT_CHANNEL DSL_UBUS_OBJECT_DSL DSL_UBUS_SEPARATOR "channel"
#define DSL_UBUS_OBJECT_BONDING_GROUP DSL_UBUS_OBJECT_DSL DSL_UBUS_SEPARATOR "bonding_group"

#define DSL_UBUS_METHOD_INFO "info"
#define DSL_UBUS_METHOD_STATS "stats"
#define DSL_UBUS_METHOD_STATS_INTERVAL "stats_interval"

/* Max string length to be written to an output array */
#define DSL_UBUS_JSON_ARR_STR_LEN_MAX 512
#define DSL_UBUS_JSON_HEX_STR_LEN_MAX 5
#define DSL_UBUS_OBJECT_NAME_LEN_MAX 256
#define DSL_UBUS_METHOD_NAME_LEN_MAX 256
#define DSL_UBUS_PARAM_PATH_LENGTH 256

/**********************************************/
/* typedefs                                   */
/**********************************************/

typedef enum {
   DSL_UBUS_OBJECT_FIRST_IDX = 0,
   DSL_UBUS_OBJECT_LINE_IDX = 0,
   DSL_UBUS_OBJECT_CHANNEL_IDX,
   DSL_UBUS_OBJECT_BONDING_GROUP_IDX,
   DSL_UBUS_OBJECT_LAST_IDX
} DSL_ST_UbusObjectIdx_t;

typedef enum {
   DSL_UBUS_METHOD_FIRST_IDX = 0,
   DSL_UBUS_METHOD_INFO_IDX = 0,
   DSL_UBUS_METHOD_STATS_IDX,
   DSL_UBUS_METHOD_STATS_INTERVAL_IDX,
   DSL_UBUS_METHOD_LAST_IDX
} DSL_ST_UbusMethodIdx_t;

typedef struct {
   DSL_ST_UbusObjectIdx_t idx;
   DSL_char_t name[DSL_UBUS_OBJECT_NAME_LEN_MAX];
} DSL_ST_UbusObjectName_t;

typedef struct {
   DSL_ST_UbusMethodIdx_t idx;
   DSL_char_t name[DSL_UBUS_METHOD_NAME_LEN_MAX];
} DSL_ST_UbusMethodName_t;

/* Per ubus method */
typedef struct DSL_ST_UbusParam_t
{
   DSL_char_t path[DSL_UBUS_PARAM_PATH_LENGTH];
   const DSL_ST_StatusNode_t *pStatusNode;
   struct DSL_ST_UbusParam_t *pNext;
} DSL_ST_UbusParamsList_t;

typedef struct
{
   DSL_CPE_Lock_t sManagerLock;
   DSL_boolean_t bHotFlag;
   struct json_object *pJsonObject;
} DSL_ST_UbusManagerData_t;

typedef struct DSL_ST_UbusData_t
{
   DSL_uint16_t nLine;
   DSL_uint8_t nChannel;
   DSL_uint8_t nBndGroup;
   DSL_boolean_t bUpdated;
   DSL_ST_UbusManagerData_t *pManagerData;
   struct json_object *pJsonRootObject;
   struct DSL_ST_UbusData_t *pNext;
} DSL_ST_UbusDataList_t;

typedef struct {
   DSL_ST_UbusObjectIdx_t objectIdx;
   DSL_ST_UbusMethodIdx_t methodIdx;
   DSL_ST_UbusParamsList_t *pParams;
   DSL_ST_UbusDataList_t *pData;
} DSL_ST_UbusObject_t;

typedef struct
{
   DSL_char_t *pName;
   struct json_object *pJsonObj;
} DSL_ST_UbusJsonNode_t;

/**********************************************/
/* global variable declarations               */
/**********************************************/

static const char JSON_NODES_SEPARATOR[] = ".";

/** Dictionary of ubus objects supported by UBUS Module */
static const DSL_ST_UbusObjectName_t g_objectsNames[] = {
   { DSL_UBUS_OBJECT_LINE_IDX, DSL_UBUS_OBJECT_LINE },
   { DSL_UBUS_OBJECT_CHANNEL_IDX, DSL_UBUS_OBJECT_CHANNEL },
   { DSL_UBUS_OBJECT_BONDING_GROUP_IDX, DSL_UBUS_OBJECT_BONDING_GROUP }
};
static const DSL_uint32_t g_objectsNamesNum =
   sizeof(g_objectsNames)/sizeof(g_objectsNames[0]);

/** Dictionary of ubus methods supported by UBUS Module */
static const DSL_ST_UbusMethodName_t g_methodsNames[] = {
   { DSL_UBUS_METHOD_INFO_IDX, DSL_UBUS_METHOD_INFO },
   { DSL_UBUS_METHOD_STATS_IDX , DSL_UBUS_METHOD_STATS },
   { DSL_UBUS_METHOD_STATS_INTERVAL_IDX, DSL_UBUS_METHOD_STATS_INTERVAL }
};
static const DSL_uint32_t g_methodsNamesNum =
   sizeof(g_methodsNames)/sizeof(g_methodsNames[0]);

/**
 * All object and method pairs with corresponding params list
 * supported by UBUS Module.
*/
static DSL_ST_UbusObject_t g_objectMethodPairs[] = {
   { DSL_UBUS_OBJECT_LINE_IDX, DSL_UBUS_METHOD_INFO_IDX, DSL_NULL, DSL_NULL },
   { DSL_UBUS_OBJECT_LINE_IDX, DSL_UBUS_METHOD_STATS_IDX, DSL_NULL, DSL_NULL },
   { DSL_UBUS_OBJECT_LINE_IDX, DSL_UBUS_METHOD_STATS_INTERVAL_IDX, DSL_NULL, DSL_NULL },
   { DSL_UBUS_OBJECT_CHANNEL_IDX, DSL_UBUS_METHOD_INFO_IDX, DSL_NULL, DSL_NULL },
   { DSL_UBUS_OBJECT_CHANNEL_IDX, DSL_UBUS_METHOD_STATS_IDX, DSL_NULL, DSL_NULL },
   { DSL_UBUS_OBJECT_CHANNEL_IDX, DSL_UBUS_METHOD_STATS_INTERVAL_IDX, DSL_NULL, DSL_NULL },
   { DSL_UBUS_OBJECT_BONDING_GROUP_IDX, DSL_UBUS_METHOD_INFO_IDX, DSL_NULL, DSL_NULL }
};
static const DSL_uint32_t g_objectMethodPairsNum =
   sizeof(g_objectMethodPairs)/sizeof(g_objectMethodPairs[0]);

/**********************************************/
/* local function declarations                */
/**********************************************/

/**
 * Simply copy parameter path
 * from Core's nodes array
 * to internal data structure.
 */
static int DSL_ST_UbusParamPathCreate(
   const DSL_uint32_t nNode,
   DSL_char_t *pPath
);

/**
 * Create list with status parameters
 * corelated to ubus object and method pair.
 * This is generic list - contains no object number.
 */
static DSL_Error_t DSL_ST_UbusParamsListCreate(
   const DSL_ST_UbusObjectIdx_t objectIdx,
   const DSL_ST_UbusMethodIdx_t methodIdx,
   DSL_ST_UbusParamsList_t **pParams
);

/**
 * Get JSON parent object of given JSON node path.
 */
static DSL_Error_t DSL_ST_UbusJsonNodeParentGet(
   struct json_object *pRoot,
   const DSL_char_t *pNodePath,
   struct json_object **pParent
);

/**
   Return name of leaf node of the given JSON path.
*/
static const DSL_char_t* DSL_ST_UbusJsonNodeNameGet(
   const DSL_char_t *pJsonPath
);

/**
 * Create JSON data structure
 * based on related parameters list.
 */
static DSL_Error_t DSL_ST_UbusParamsDataCreate(
   const DSL_uint16_t nLine,
   const DSL_uint8_t nChannel,
   const DSL_uint8_t nBndGroup,
   const DSL_ST_UbusParamsList_t *pParams,
   DSL_ST_UbusDataList_t **pData
);

/**
 * Get parameter list and data details
 * based on given criteria.
 */
static DSL_Error_t DSL_ST_UbusParamDetailsGet(
   const DSL_ST_StatusNodeIdx_t eNodeIdx,
   const DSL_uint16_t nLine,
   const DSL_uint8_t nChannel,
   const DSL_uint8_t nBndGroup,
   DSL_ST_UbusParamsList_t **pParams,
   DSL_ST_UbusDataList_t **pData
);

/**
 * Get data details
 * based on given criteria.
 */
static DSL_Error_t DSL_ST_UbusDataDetailsGet(
   const char *pObjectName,
   const char *pMethodName,
   const unsigned int nObjectNum,
   const unsigned int nChannelLineNum,
   DSL_ST_UbusDataList_t **pData
);

/**
 * Provide information if JSON objects are differenet or equal.
 *
 * \return
 *    DSL_TRUE    - objects differ
 *    DSL_FALSE   - otherwise
 */
static DSL_boolean_t DSL_ST_UbusJsonNodeCompare(
   struct json_object *pJsonObjA,
   struct json_object *pJsonObjB
);

/**********************************************/
/* global function definitions                */
/**********************************************/

DSL_Error_t DSL_ST_UbusInit()
{
   DSL_Error_t nErrorCode = DSL_ERROR;
   DSL_Error_t nRetVal = DSL_ERROR;
   DSL_uint16_t nLine = 0;
   DSL_uint8_t nChannel = 0, nBndGroup = 0;
   DSL_ST_UbusDataList_t *pData = DSL_NULL;

   DSL_CCA_DEBUG(DSL_CCA_DBG_MSG, (DSL_CPE_PREFIX
      "%s: IN" DSL_CPE_CRLF, __func__));

   /* For each pair of object and method... */
   for (DSL_uint32_t pair = 0; pair < g_objectMethodPairsNum; ++pair)
   {
      nLine = 0, nChannel = 0, nBndGroup = 0;

      /* ...create common (one per object) list of parameters... */
      nRetVal = DSL_ST_UbusParamsListCreate(g_objectMethodPairs[pair].objectIdx,
         g_objectMethodPairs[pair].methodIdx,
         &g_objectMethodPairs[pair].pParams);

      if (nRetVal == DSL_ERROR)
      {
         continue;
      }

      /* ...and use it to create an internal data struct per each Ubus instance... */
      if (g_objectMethodPairs[pair].objectIdx == DSL_UBUS_OBJECT_LINE_IDX)
      {
         for (nLine = 0; nLine < DSL_ST_DSL_ENTITIES; ++nLine)
         {
            nRetVal = DSL_ST_UbusParamsDataCreate(nLine, nChannel, nBndGroup,
               g_objectMethodPairs[pair].pParams, &pData);

            if (g_objectMethodPairs[pair].pData == DSL_NULL)
            {
               g_objectMethodPairs[pair].pData = pData;
            }

            pData = pData->pNext;
         }
      }
      else if (g_objectMethodPairs[pair].objectIdx == DSL_UBUS_OBJECT_CHANNEL_IDX)
      {
         for (nChannel = 0; nChannel < DSL_ST_CHANNELS_PER_LINE; ++nChannel)
         {
            for (nLine = 0; nLine < DSL_ST_DSL_ENTITIES; ++nLine)
            {
               nRetVal = DSL_ST_UbusParamsDataCreate(nLine, nChannel, nBndGroup,
                  g_objectMethodPairs[pair].pParams, &pData);

               if (g_objectMethodPairs[pair].pData == DSL_NULL)
               {
                  g_objectMethodPairs[pair].pData = pData;
               }

               pData = pData->pNext;
            }
         }
      }
      else if (g_objectMethodPairs[pair].objectIdx == DSL_UBUS_OBJECT_BONDING_GROUP_IDX &&
         INCLUDE_DSL_BONDING)
      {
         for (nBndGroup = 0; nBndGroup < 1; ++nBndGroup)
         {
            nRetVal = DSL_ST_UbusParamsDataCreate(nLine, nChannel, nBndGroup,
               g_objectMethodPairs[pair].pParams, &pData);

            if (g_objectMethodPairs[pair].pData == DSL_NULL)
            {
               g_objectMethodPairs[pair].pData = pData;
            }

            pData = pData->pNext;
         }
      }

      if (nRetVal == DSL_SUCCESS)
      {
         nErrorCode = DSL_SUCCESS;
      }
   }

   DSL_CCA_DEBUG(DSL_CCA_DBG_MSG, (DSL_CPE_PREFIX
      "%s: OUT (nErrorCode=<%d>)" DSL_CPE_CRLF, __func__,
      nErrorCode));

   return nErrorCode;
}

DSL_Error_t DSL_ST_UbusUpdate(
   const DSL_ST_StatusNodeIdx_t eNodeIdx,
   const DSL_uint16_t nLine,
   const DSL_uint8_t nChannel,
   const DSL_ST_StatusNodeType_t eParamType,
   const DSL_ST_ParamValue_t *uValue)
{
   DSL_Error_t nErrorCode = DSL_ERROR;
   const DSL_uint8_t nBndGroup = 0;
   DSL_ST_UbusParamsList_t *pParams = DSL_NULL;
   DSL_ST_UbusDataList_t *pData = DSL_NULL;
   struct json_object *pJsonParentObj = DSL_NULL;
   struct json_object *pJsonNewObj = DSL_NULL;

   if (DSL_ST_UbusParamDetailsGet(eNodeIdx, nLine, nChannel, nBndGroup,
         &pParams, &pData) != DSL_SUCCESS)
   {
      DSL_CCA_DEBUG(DSL_CCA_DBG_ERR, (DSL_CPE_PREFIX
         "%s: failed to get param details!" DSL_CPE_CRLF, __func__));
      return nErrorCode;
   }

   if ((pParams == DSL_NULL || pParams->path == DSL_NULL) ||
       (pData == DSL_NULL || pData->pJsonRootObject == DSL_NULL))
   {
      DSL_CCA_DEBUG(DSL_CCA_DBG_ERR, (DSL_CPE_PREFIX
         "%s: output data is NULL!" DSL_CPE_CRLF, __func__));
      return nErrorCode;
   }

   switch (eParamType)
   {
   case DSL_ST_NODE_TYPE_ARRAY:
      if (uValue->aValue.eOutputType == HEX)
      {
         /* biggest possible value would be "0xFF" */
         DSL_char_t sHex[DSL_UBUS_JSON_HEX_STR_LEN_MAX] = { 0 };
         pJsonNewObj = json_object_new_array();

         /* update json object */
         for (DSL_uint32_t i = 0; i < uValue->aValue.nSize; ++i)
         {
            if (uValue->aValue.eInputType == U8)
            {
               DSL_ST_snprintf_s(sHex, sizeof(sHex),
                  "0x%02X", uValue->aValue.uArray.pU8Array[i]);
            }
            else if (uValue->aValue.eInputType == U32)
            {
               DSL_ST_snprintf_s(sHex, sizeof(sHex),
                  "0x%02X", uValue->aValue.uArray.pU32Array[i]);
            }
            else
            {
               break;
            }
            sHex[DSL_UBUS_JSON_HEX_STR_LEN_MAX - 1] = '\0';

            json_object_array_add(pJsonNewObj,
               json_object_new_string(sHex));
         }
      }
      else if (uValue->aValue.eOutputType == STR)
      {
         DSL_char_t sStr[DSL_UBUS_JSON_ARR_STR_LEN_MAX] = { 0 };
         pJsonNewObj = json_object_new_array();

         /* update json object */
         for (DSL_uint32_t i = 0; i < uValue->aValue.nSize; ++i)
         {
            memset(sStr, 0, DSL_UBUS_JSON_ARR_STR_LEN_MAX);
            DSL_ST_strncpy_s(sStr, DSL_UBUS_JSON_ARR_STR_LEN_MAX,
               uValue->aValue.uArray.ppStrArray[i],
               DSL_ST_strnlen_s(uValue->aValue.uArray.ppStrArray[i],
                  DSL_UBUS_JSON_ARR_STR_LEN_MAX - 1));

            json_object_array_add(pJsonNewObj,
               json_object_new_string(sStr));
         }
      }
      break;
   case DSL_ST_NODE_TYPE_STRING:
      pJsonNewObj = json_object_new_string(uValue->sValue);
      break;
   case DSL_ST_NODE_TYPE_INT:
      pJsonNewObj = json_object_new_int(uValue->nValue);
      break;
   case DSL_ST_NODE_TYPE_DOUBLE:
      pJsonNewObj = json_object_new_double(uValue->nValue);
      break;
   case DSL_ST_NODE_TYPE_BOOL:
      pJsonNewObj = json_object_new_boolean(uValue->bValue);
      break;
   default:
      break;
   }

   if (pJsonNewObj != DSL_NULL)
   {
      DSL_ST_UbusJsonNodeParentGet(pData->pJsonRootObject, pParams->path, &pJsonParentObj);
      json_object_object_add(pJsonParentObj,
         DSL_ST_UbusJsonNodeNameGet(pParams->path), pJsonNewObj);

      pData->bUpdated = DSL_TRUE;
      nErrorCode = DSL_SUCCESS;
   }

   return nErrorCode;
}

DSL_Error_t DSL_ST_UbusSync(DSL_void_t)
{
   DSL_ST_UbusDataList_t *pData = DSL_NULL;

   /* For each pair of object and method... */
   for (DSL_uint32_t pair = 0; pair < g_objectMethodPairsNum; ++pair)
   {
      pData = g_objectMethodPairs[pair].pData;

      /* ...check if internal data were updated... */
      while(pData != DSL_NULL && pData->bUpdated == DSL_TRUE)
      {
         /* ...and if this update caused a real change... */
         if(DSL_ST_UbusJsonNodeCompare(pData->pJsonRootObject,
            pData->pManagerData->pJsonObject))
         {
            DSL_CPE_LockGet(&pData->pManagerData->sManagerLock);
            /* ...free previous object... */
            json_object_put(pData->pManagerData->pJsonObject);
            /* ...create a new copy,
               to be used when DSL Manager will request a data. */
            pData->pManagerData->pJsonObject =
               json_tokener_parse(json_object_get_string(pData->pJsonRootObject));
            pData->pManagerData->bHotFlag = DSL_TRUE;
            DSL_CPE_LockSet(&pData->pManagerData->sManagerLock);
         }

         pData->bUpdated = DSL_FALSE;
         pData = pData->pNext;
      }
   }

   return DSL_SUCCESS;
}

DSL_Error_t DSL_ST_UbusDeinit(DSL_void_t)
{
   DSL_ST_UbusDataList_t *pData = DSL_NULL;
   DSL_ST_UbusDataList_t *pNextData = DSL_NULL;
   DSL_ST_UbusParamsList_t *pParams = DSL_NULL;
   DSL_ST_UbusParamsList_t *pNextParams = DSL_NULL;
   DSL_ST_UbusManagerData_t *pManagerData = DSL_NULL;

   DSL_CCA_DEBUG(DSL_CCA_DBG_MSG, (DSL_CPE_PREFIX
      "%s: IN" DSL_CPE_CRLF, __func__));

   for (DSL_uint32_t pair = 0; pair < g_objectMethodPairsNum; ++pair)
   {
      pParams = g_objectMethodPairs[pair].pParams;
      g_objectMethodPairs[pair].pParams = DSL_NULL;

      while(pParams != DSL_NULL)
      {
         pNextParams = pParams->pNext;

         DSL_ST_Free(pParams);
         pParams = pNextParams;
      }

      pData = g_objectMethodPairs[pair].pData;
      g_objectMethodPairs[pair].pData = DSL_NULL;

      while(pData != DSL_NULL)
      {
         pNextData = pData->pNext;

         pManagerData = pData->pManagerData;
         pData->pManagerData = DSL_NULL;

         DSL_CPE_LockDelete(&pManagerData->sManagerLock);
         json_object_put(pManagerData->pJsonObject);
         DSL_ST_Free(pManagerData);

         json_object_put(pData->pJsonRootObject);
         DSL_ST_Free(pData);

         pData = pNextData;
      }
   }

   DSL_CCA_DEBUG(DSL_CCA_DBG_MSG, (DSL_CPE_PREFIX
      "%s: OUT" DSL_CPE_CRLF, __func__));

   return DSL_SUCCESS;
}

#endif /* defined(INCLUDE_DSL_STATUS_UBUS_SUPPORT) && (INCLUDE_DSL_STATUS_UBUS_SUPPORT == 1) */

int DSL_ST_UbusObjectsGet(
   char ***pObjectsNames,
   unsigned int *pObjectsNamesNum)
{
   int nErrorCode = -1;

#if defined(INCLUDE_DSL_STATUS_UBUS_SUPPORT) && (INCLUDE_DSL_STATUS_UBUS_SUPPORT == 1)
   DSL_size_t nSize = 0;
   DSL_size_t nObjectNamesNum =
      (INCLUDE_DSL_BONDING) ? g_objectsNamesNum : g_objectsNamesNum - 1;

   DSL_CCA_DEBUG(DSL_CCA_DBG_MSG, (DSL_CPE_PREFIX
      "%s: IN (objects=<%p>)" DSL_CPE_CRLF, __func__,
      pObjectsNames));

   if (pObjectsNames == NULL || pObjectsNamesNum == NULL)
   {
      return nErrorCode;
   }

   *pObjectsNames = (char**)DSL_ST_Malloc(nObjectNamesNum * sizeof(char*));
   if (*pObjectsNames == NULL)
   {
      return nErrorCode;
   }

   *pObjectsNamesNum = 0;
   /* Copy each supported object name to pObjectsNames. */
   for (unsigned int objectIdx = 0; objectIdx < g_objectsNamesNum; ++objectIdx)
   {
      if (objectIdx == DSL_UBUS_OBJECT_BONDING_GROUP_IDX && !INCLUDE_DSL_BONDING)
      {
         continue;
      }

      nSize = DSL_ST_strnlen_s(g_objectsNames[objectIdx].name,
         DSL_UBUS_OBJECT_NAME_LEN_MAX);
      (*pObjectsNames)[*pObjectsNamesNum] = (char*)DSL_ST_Malloc((nSize + 1) * sizeof(char));

      if ((*pObjectsNames)[*pObjectsNamesNum] != NULL)
      {
         memset((*pObjectsNames)[*pObjectsNamesNum], 0, nSize + 1);
         DSL_ST_strncpy_s((*pObjectsNames)[*pObjectsNamesNum], nSize + 1,
            g_objectsNames[objectIdx].name, nSize);

         /* Increment number of supported objects. */
         ++(*pObjectsNamesNum);
      }
      else
      {
         DSL_CCA_DEBUG(DSL_CCA_DBG_ERR, (DSL_CPE_PREFIX
            "%s: Memory allocation failure!" DSL_CPE_CRLF, __func__));
      }

      nErrorCode = (*pObjectsNamesNum > 0) ? 0 : -1;
   }

   DSL_CCA_DEBUG(DSL_CCA_DBG_MSG, (DSL_CPE_PREFIX
      "%s: OUT (nErrorCode=<%d>, objects: %u)" DSL_CPE_CRLF, __func__,
      nErrorCode, *pObjectsNamesNum));
#endif /* defined(INCLUDE_DSL_STATUS_UBUS_SUPPORT) && (INCLUDE_DSL_STATUS_UBUS_SUPPORT == 1) */

   return nErrorCode;
}

int DSL_ST_UbusObjectsDetailsGet(
   const char *pObjectName,
   char ***pObjectMethods,
   unsigned int *pObjectMethodsNum,
   unsigned int *pObjectNum)
{
   int nErrorCode = -1;

#if defined(INCLUDE_DSL_STATUS_UBUS_SUPPORT) && (INCLUDE_DSL_STATUS_UBUS_SUPPORT == 1)
   unsigned int nMethodsNum = 0;

   DSL_CCA_DEBUG(DSL_CCA_DBG_MSG, (DSL_CPE_PREFIX
      "%s: IN (object='%s', methods=<%p>)" DSL_CPE_CRLF, __func__,
      pObjectName, pObjectMethods));

   if (pObjectName == NULL || pObjectMethods == NULL ||
      pObjectMethodsNum == NULL || pObjectNum == NULL)
   {
      return nErrorCode;
   }

   /* For each supported object name. */
   for (unsigned int objectIdx = 0; objectIdx < g_objectsNamesNum; ++objectIdx)
   {
      /* Match given name to internal object index. */
      if (strncmp(g_objectsNames[objectIdx].name, pObjectName,
         DSL_ST_strnlen_s(g_objectsNames[objectIdx].name, DSL_UBUS_OBJECT_NAME_LEN_MAX)) == 0)
      {
         /* Count number of supported methods for this object idx. */
         for (DSL_uint32_t pair = 0; pair < g_objectMethodPairsNum; ++pair)
         {
            if (g_objectMethodPairs[pair].objectIdx == objectIdx)
            {
               ++nMethodsNum;
            }
         }

         /* Set number of supported object instances. */
         if (objectIdx == DSL_UBUS_OBJECT_LINE_IDX)
         {
            *pObjectNum = DSL_ST_DSL_ENTITIES;
         }
         else if (objectIdx == DSL_UBUS_OBJECT_CHANNEL_IDX)
         {
            *pObjectNum = DSL_ST_CHANNELS_PER_LINE;
         }
         else if (objectIdx == DSL_UBUS_OBJECT_BONDING_GROUP_IDX &&
            INCLUDE_DSL_BONDING)
         {
            *pObjectNum = 1;
         }
      }
   }

   *pObjectMethodsNum = 0;
   if (nMethodsNum > 0 && *pObjectNum > 0)
   {
      DSL_size_t nSize = 0;
      *pObjectMethods = (char**)DSL_ST_Malloc(nMethodsNum * sizeof(char*));

      /* Copy each supported method name to pObjectMethods. */
      for (unsigned int methodIdx = 0;
         methodIdx < nMethodsNum && methodIdx < g_methodsNamesNum;
         ++methodIdx)
      {
         nSize = DSL_ST_strnlen_s(g_methodsNames[methodIdx].name,
            DSL_UBUS_METHOD_NAME_LEN_MAX);
         (*pObjectMethods)[*pObjectMethodsNum] = (char*)DSL_ST_Malloc((nSize + 1) * sizeof(char));

         if ((*pObjectMethods)[*pObjectMethodsNum] != NULL)
         {
            memset((*pObjectMethods)[*pObjectMethodsNum], 0, nSize + 1);
            DSL_ST_strncpy_s((*pObjectMethods)[*pObjectMethodsNum], nSize + 1,
               g_methodsNames[methodIdx].name, nSize);

            /* Increment number of supported methods. */
            ++(*pObjectMethodsNum);
         }
         else
         {
            DSL_CCA_DEBUG(DSL_CCA_DBG_ERR, (DSL_CPE_PREFIX
               "%s: Memory allocation failure!" DSL_CPE_CRLF, __func__));
         }
      }

      nErrorCode = (*pObjectMethodsNum > 0) ? 0 : -1;
   }

   DSL_CCA_DEBUG(DSL_CCA_DBG_MSG, (DSL_CPE_PREFIX
      "%s: OUT (nErrorCode=<%d>, methods: %u)" DSL_CPE_CRLF, __func__,
      nErrorCode, *pObjectMethodsNum));
#endif /* defined(INCLUDE_DSL_STATUS_UBUS_SUPPORT) && (INCLUDE_DSL_STATUS_UBUS_SUPPORT == 1) */

   return nErrorCode;
}

int DSL_ST_UbusDataGet(
   const char *pObjectName,
   const char *pMethodName,
   const unsigned int nObjectNum,
   const unsigned int nChannelLineNum,
   struct blob_buf *pData,
   const bool bSubscriptionMode)
{
   int nErrorCode = -1;

#if defined(INCLUDE_DSL_STATUS_UBUS_SUPPORT) && (INCLUDE_DSL_STATUS_UBUS_SUPPORT == 1)
   DSL_ST_UbusDataList_t *pUbusData = DSL_NULL;

   /* Handle only supported Ubus instance. */
   if (pData != DSL_NULL &&
      DSL_ST_UbusDataDetailsGet(pObjectName, pMethodName, nObjectNum,
      nChannelLineNum, &pUbusData) == DSL_SUCCESS)
   {
      /* In command mode provide data always. */
      if (bSubscriptionMode == false)
      {
         DSL_CPE_LockGet(&pUbusData->pManagerData->sManagerLock);
         blobmsg_add_object(pData, pUbusData->pManagerData->pJsonObject);
         DSL_CPE_LockSet(&pUbusData->pManagerData->sManagerLock);

         nErrorCode = 0;
      }
      /* In subscription mode provide only updated data. */
      else if (bSubscriptionMode == true &&
         pUbusData->pManagerData->bHotFlag == DSL_TRUE)
      {
         DSL_CPE_LockGet(&pUbusData->pManagerData->sManagerLock);
         blobmsg_add_object(pData, pUbusData->pManagerData->pJsonObject);
         pUbusData->pManagerData->bHotFlag = DSL_FALSE;
         DSL_CPE_LockSet(&pUbusData->pManagerData->sManagerLock);

         nErrorCode = 0;
      }
   }

#endif /* defined(INCLUDE_DSL_STATUS_UBUS_SUPPORT) && (INCLUDE_DSL_STATUS_UBUS_SUPPORT == 1) */

   return nErrorCode;
}

/**********************************************/
/* local function definitions                 */
/**********************************************/

#if defined(INCLUDE_DSL_STATUS_UBUS_SUPPORT) && (INCLUDE_DSL_STATUS_UBUS_SUPPORT == 1)

static int DSL_ST_UbusParamPathCreate(
   const DSL_uint32_t nNode,
   DSL_char_t* pPath)
{
   return DSL_ST_snprintf_s(pPath, DSL_UBUS_PARAM_PATH_LENGTH, "%s",
      statusNodesArray[nNode].jsonParamName);
}

static DSL_Error_t DSL_ST_UbusParamsListCreate(
   const DSL_ST_UbusObjectIdx_t objectIdx,
   const DSL_ST_UbusMethodIdx_t methodIdx,
   DSL_ST_UbusParamsList_t** pParams)
{
   DSL_Error_t nErrorCode = DSL_ERROR;
   DSL_ST_UbusParamsList_t *pHead = DSL_NULL;

   DSL_CCA_DEBUG(DSL_CCA_DBG_MSG, (DSL_CPE_PREFIX
      "%s: IN (object='%s' method='%s')" DSL_CPE_CRLF, __func__,
      g_objectsNames[objectIdx].name, g_methodsNames[methodIdx].name));

   if (pParams == DSL_NULL)
   {
      return nErrorCode;
   }

   for (DSL_uint32_t nNode = 0; nNode < nNodesNum; ++nNode)
   {
      if ((objectIdx == DSL_UBUS_OBJECT_LINE_IDX &&
            statusNodesArray[nNode].eSubsection == DSL_ST_SUBSECTION_LINE) ||
         (objectIdx == DSL_UBUS_OBJECT_CHANNEL_IDX &&
            statusNodesArray[nNode].eSubsection == DSL_ST_SUBSECTION_CHANNEL) ||
         (objectIdx == DSL_UBUS_OBJECT_BONDING_GROUP_IDX &&
            INCLUDE_DSL_BONDING &&
            statusNodesArray[nNode].eSubsection == DSL_ST_SUBSECTION_BND_GROUP))
      {
         if ((methodIdx == DSL_UBUS_METHOD_INFO_IDX &&
               DSL_ST_UBUS_GROUP_INFO == statusNodesArray[nNode].eUbusGroup) ||
            (methodIdx == DSL_UBUS_METHOD_STATS_IDX &&
               DSL_ST_UBUS_GROUP_STATS == statusNodesArray[nNode].eUbusGroup) ||
            (methodIdx == DSL_UBUS_METHOD_STATS_INTERVAL_IDX &&
               DSL_ST_UBUS_GROUP_STATS_INTERVAL == statusNodesArray[nNode].eUbusGroup))
         {
            DSL_ST_UbusParamsList_t *pParam =
               (DSL_ST_UbusParamsList_t *)DSL_ST_Malloc(sizeof(DSL_ST_UbusParamsList_t));

            if (pParam != DSL_NULL)
            {
               DSL_ST_UbusParamPathCreate(nNode, pParam->path);
               pParam->pStatusNode = &statusNodesArray[nNode];
               pParam->pNext = DSL_NULL;

               if (*pParams == DSL_NULL)
               {
                  *pParams = pParam;
               }
               else
               {
                  pHead = *pParams;
                  while (pHead->pNext != DSL_NULL)
                  {
                     pHead = pHead->pNext;
                  }

                  pHead->pNext = pParam;
               }

               nErrorCode = DSL_SUCCESS;
            }
         }
      }
   }

   pHead = *pParams;
   while (pHead != DSL_NULL)
   {
      DSL_CCA_DEBUG(DSL_CCA_DBG_MSG, (DSL_CPE_PREFIX
         "%s: node=<%d> path='%s'" DSL_CPE_CRLF, __func__,
         pHead->pStatusNode->eNodeIdx, pHead->path));

      pHead = pHead->pNext;
   }

   DSL_CCA_DEBUG(DSL_CCA_DBG_MSG, (DSL_CPE_PREFIX
      "%s: OUT (nErrorCode=<%d>)" DSL_CPE_CRLF, __func__,
      nErrorCode));

   return nErrorCode;
}

static DSL_Error_t DSL_ST_UbusJsonNodeParentGet(
   struct json_object *pRoot,
   const DSL_char_t *pNodePath,
   struct json_object **pParent)
{
   DSL_char_t inputPath[DSL_UBUS_PARAM_PATH_LENGTH] = { 0 };
   DSL_ST_UbusJsonNode_t visitedNode = { DSL_NULL };
   DSL_char_t *pToken = DSL_NULL;
   DSL_uint_t nStrMax = 0;
   DSL_char_t *pNextToken = DSL_NULL;

   DSL_ST_strncpy_s(inputPath, sizeof(inputPath),
      pNodePath, sizeof(inputPath));
   nStrMax = DSL_ST_strnlen_s(inputPath, DSL_UBUS_PARAM_PATH_LENGTH - 1) + 1;

   visitedNode.pJsonObj = pRoot;

   pToken = DSL_ST_strtok_s(
      inputPath, &nStrMax, JSON_NODES_SEPARATOR, &pNextToken);
   while (pToken)
   {
      visitedNode.pName = pToken;
      json_object_object_foreach(visitedNode.pJsonObj, key, val)
      {
         if (strcmp(key, visitedNode.pName) == 0)
         {
            if (json_object_get_type(val) == json_type_object)
            {
               visitedNode.pJsonObj = val;
            }
            break;
         }
      }
      pToken = DSL_ST_strtok_s(
         NULL, &nStrMax, JSON_NODES_SEPARATOR, &pNextToken);
   }

   *pParent = visitedNode.pJsonObj;

   return (*pParent != DSL_NULL) ? DSL_SUCCESS : DSL_ERROR;
}

static const DSL_char_t* DSL_ST_UbusJsonNodeNameGet(
   const DSL_char_t *pJsonPath)
{
   const DSL_char_t *pLast = strrchr(pJsonPath, (int)JSON_NODES_SEPARATOR[0]);
   return (pLast ? pLast + 1 : pJsonPath);
}

static DSL_Error_t DSL_ST_UbusParamsDataCreate(
   const DSL_uint16_t nLine,
   const DSL_uint8_t nChannel,
   const DSL_uint8_t nBndGroup,
   const DSL_ST_UbusParamsList_t *pParams,
   DSL_ST_UbusDataList_t **pData)
{
   DSL_Error_t nErrorCode = DSL_ERROR;
   struct json_object *pJsonParentObj = DSL_NULL;
   struct json_object *pJsonNewObj = DSL_NULL;
   DSL_ST_UbusDataList_t *pNewData = DSL_NULL;
   const DSL_ST_UbusParamsList_t *pParam = pParams;

   DSL_CCA_DEBUG(DSL_CCA_DBG_MSG, (DSL_CPE_PREFIX
      "%s: IN (line=<%d> channel=<%d> bndgroup=<%d> "
      "params=<%p> data=<%p>)" DSL_CPE_CRLF, __func__,
      nLine, nChannel, nBndGroup,
      pParams, pData));

   pNewData =
      (DSL_ST_UbusDataList_t *)DSL_ST_Malloc(sizeof(DSL_ST_UbusDataList_t));
   if (pNewData == DSL_NULL)
   {
      return nErrorCode;
   }

   pNewData->nLine = nLine;
   pNewData->nChannel = nChannel;
   pNewData->nBndGroup = nBndGroup;
   pNewData->pNext = DSL_NULL;
   pNewData->bUpdated = DSL_FALSE;

   pNewData->pJsonRootObject = json_object_new_object();
   if (pNewData->pJsonRootObject == DSL_NULL)
   {
      DSL_ST_Free(pNewData);
      pNewData = DSL_NULL;

      return nErrorCode;
   }

   pNewData->pManagerData =
      (DSL_ST_UbusManagerData_t *)DSL_ST_Malloc(sizeof(DSL_ST_UbusManagerData_t));
   if (pNewData->pManagerData == DSL_NULL)
   {
      DSL_ST_Free(pNewData);
      pNewData = DSL_NULL;

      return nErrorCode;
   }

   pNewData->pManagerData->pJsonObject = json_object_new_object();
   if (pNewData->pManagerData->pJsonObject == DSL_NULL)
   {
      DSL_ST_Free(pNewData->pManagerData);
      pNewData->pManagerData = DSL_NULL;

      DSL_ST_Free(pNewData);
      pNewData = DSL_NULL;

      return nErrorCode;
   }

   DSL_CPE_LockCreate(&pNewData->pManagerData->sManagerLock);
   pNewData->pManagerData->bHotFlag = DSL_FALSE;

   while (pParam != DSL_NULL)
   {
      if (DSL_ST_UbusJsonNodeParentGet(pNewData->pJsonRootObject, pParam->path,
         &pJsonParentObj) == DSL_SUCCESS)
      {
         if (json_object_get_type(pJsonParentObj) == json_type_object)
         {
            switch (pParam->pStatusNode->eNodeType)
            {
            case DSL_ST_NODE_TYPE_OBJ:
               pJsonNewObj = json_object_new_object();
               break;
            case DSL_ST_NODE_TYPE_ARRAY:
               pJsonNewObj = json_object_new_array();
               break;
            case DSL_ST_NODE_TYPE_STRING:
               pJsonNewObj = json_object_new_string("");
               break;
            case DSL_ST_NODE_TYPE_INT:
               pJsonNewObj = json_object_new_int(0);
               break;
            case DSL_ST_NODE_TYPE_DOUBLE:
               pJsonNewObj = json_object_new_double(0.0);
               break;
            case DSL_ST_NODE_TYPE_BOOL:
               pJsonNewObj = json_object_new_boolean(0);
               break;
            default:
               break;
            }

            if (pJsonNewObj)
            {
               json_object_object_add(pJsonParentObj,
                  DSL_ST_UbusJsonNodeNameGet(pParam->path), pJsonNewObj);

               pJsonNewObj = DSL_NULL;
               nErrorCode = DSL_SUCCESS;
            }
         }
      }

      pParam = pParam->pNext;
   }

   *pData = pNewData;

   DSL_CCA_DEBUG(DSL_CCA_DBG_MSG, (DSL_CPE_PREFIX
      "%s: OUT (nErrorCode=<%d>)" DSL_CPE_CRLF, __func__,
      nErrorCode));

   return nErrorCode;
}

static DSL_Error_t DSL_ST_UbusParamDetailsGet(
   const DSL_ST_StatusNodeIdx_t eNodeIdx,
   const DSL_uint16_t nLine,
   const DSL_uint8_t nChannel,
   const DSL_uint8_t nBndGroup,
   DSL_ST_UbusParamsList_t **pParams,
   DSL_ST_UbusDataList_t **pData)
{
   DSL_boolean_t bFound = DSL_FALSE;
   DSL_ST_UbusObjectIdx_t eObjectIdx = DSL_UBUS_OBJECT_LAST_IDX;
   DSL_ST_UbusMethodIdx_t eMethodIdx = DSL_UBUS_METHOD_LAST_IDX;
   DSL_ST_UbusParamsList_t *pParamsListHead = DSL_NULL;
   DSL_ST_UbusDataList_t *pDataListHead = DSL_NULL;

   /* find requested ubus object index */
   switch(statusNodesArray[eNodeIdx].eSubsection)
   {
   case DSL_ST_SUBSECTION_LINE:
      eObjectIdx = DSL_UBUS_OBJECT_LINE_IDX;
      break;
   case DSL_ST_SUBSECTION_CHANNEL:
      eObjectIdx = DSL_UBUS_OBJECT_CHANNEL_IDX;
      break;
   case DSL_ST_SUBSECTION_BND_GROUP:
      eObjectIdx = DSL_UBUS_OBJECT_BONDING_GROUP_IDX;
      break;
   default:
      break;
   }

   /* find requested ubus object method */
   switch (statusNodesArray[eNodeIdx].eUbusGroup)
   {
   case DSL_ST_UBUS_GROUP_INFO:
      eMethodIdx = DSL_UBUS_METHOD_INFO_IDX;
      break;
   case DSL_ST_UBUS_GROUP_STATS:
      eMethodIdx = DSL_UBUS_METHOD_STATS_IDX;
      break;
   case DSL_ST_UBUS_GROUP_STATS_INTERVAL:
      eMethodIdx = DSL_UBUS_METHOD_STATS_INTERVAL_IDX;
      break;
   default:
      break;
   }

   if (eObjectIdx == DSL_UBUS_OBJECT_LAST_IDX ||
       eMethodIdx == DSL_UBUS_METHOD_LAST_IDX)
   {
      DSL_CCA_DEBUG(DSL_CCA_DBG_WRN, (DSL_CPE_PREFIX
         "%s: Requested ubus object not found!" DSL_CPE_CRLF, __func__));
      return DSL_ERROR;
   }

   for (DSL_uint32_t pair = 0; pair < g_objectMethodPairsNum && !bFound; ++pair)
   {
      if (g_objectMethodPairs[pair].objectIdx == eObjectIdx &&
          g_objectMethodPairs[pair].methodIdx == eMethodIdx)
      {
         /* find matching ubus data */
         pDataListHead = g_objectMethodPairs[pair].pData;
         while (pDataListHead != DSL_NULL)
         {
            switch (eObjectIdx)
            {
            case DSL_UBUS_OBJECT_LINE_IDX:
               if (pDataListHead->nLine == nLine)
               {
                  bFound = DSL_TRUE;
               }
               break;
            case DSL_UBUS_OBJECT_CHANNEL_IDX:
               if (pDataListHead->nLine == nLine &&
                   pDataListHead->nChannel == nChannel)
               {
                  bFound = DSL_TRUE;
               }
               break;
            case DSL_UBUS_OBJECT_BONDING_GROUP_IDX:
               if (pDataListHead->nBndGroup == nBndGroup)
               {
                  bFound = DSL_TRUE;
               }
               break;
            default:
               break;
            }

            if (bFound == DSL_TRUE)
            {
               *pData = pDataListHead;
               break;
            }
            pDataListHead = pDataListHead->pNext;
         }

         /* find matching ubus param */
         if (bFound == DSL_TRUE)
         {
            pParamsListHead = g_objectMethodPairs[pair].pParams;
            while(pParamsListHead != DSL_NULL)
            {
               if (pParamsListHead->pStatusNode->eNodeIdx == eNodeIdx)
               {
                  *pParams = pParamsListHead;
                  break;
               }
               pParamsListHead = pParamsListHead->pNext;
            }
            /* matching ubus param and data found, exit for loop */
            break;
         }
      }
   }

   return bFound ? DSL_SUCCESS : DSL_ERROR;
}

static DSL_Error_t DSL_ST_UbusDataDetailsGet(
   const char *pObjectName,
   const char *pMethodName,
   const unsigned int nObjectNum,
   const unsigned int nChannelLineNum,
   DSL_ST_UbusDataList_t **pData)
{
   DSL_boolean_t bFound = DSL_FALSE;
   DSL_ST_UbusObjectIdx_t objectIdx = DSL_UBUS_OBJECT_LAST_IDX;
   DSL_ST_UbusMethodIdx_t methodIdx = DSL_UBUS_METHOD_LAST_IDX;
   DSL_ST_UbusDataList_t *pTmpData = DSL_NULL;

   for (unsigned int i = 0; i < g_objectsNamesNum; ++i)
   {
      if (strncmp(g_objectsNames[i].name, pObjectName,
         DSL_ST_strnlen_s(g_objectsNames[i].name, DSL_UBUS_OBJECT_NAME_LEN_MAX)) == 0)
      {
         for (unsigned int j = 0; j < g_methodsNamesNum; ++j)
         {
            if (strncmp(g_methodsNames[j].name, pMethodName,
               DSL_ST_strnlen_s(g_methodsNames[j].name, DSL_UBUS_METHOD_NAME_LEN_MAX)) == 0)
            {
               objectIdx = i;
               methodIdx = j;
            }
         }
      }
   }

   for (DSL_uint32_t pair = 0; pair < g_objectMethodPairsNum && !bFound; ++pair)
   {
      pTmpData = DSL_NULL;
      bFound = DSL_FALSE;

      if (g_objectMethodPairs[pair].objectIdx == objectIdx &&
         g_objectMethodPairs[pair].methodIdx == methodIdx)
      {
         pTmpData = g_objectMethodPairs[pair].pData;

         while(pTmpData != DSL_NULL)
         {
            if ((objectIdx == DSL_UBUS_OBJECT_LINE_IDX &&
                     pTmpData->nLine == nObjectNum) ||
               (objectIdx == DSL_UBUS_OBJECT_CHANNEL_IDX &&
                     pTmpData->nChannel == nObjectNum && pTmpData->nLine == nChannelLineNum)
               || (objectIdx == DSL_UBUS_OBJECT_BONDING_GROUP_IDX &&
                     INCLUDE_DSL_BONDING &&
                     pTmpData->nBndGroup == nObjectNum))
            {
               bFound = DSL_TRUE;
               break;
            }

            pTmpData = pTmpData->pNext;
         }
      }

      if (bFound == DSL_TRUE)
      {
         *pData = pTmpData;
      }
   }

   return bFound ? DSL_SUCCESS : DSL_ERROR;
}

static DSL_boolean_t DSL_ST_UbusJsonNodeCompare(
   struct json_object *pJsonObjA,
   struct json_object *pJsonObjB)
{
   DSL_boolean_t bDifferent = DSL_FALSE;

   if (pJsonObjA == pJsonObjB)
   {
      ;
   }
   else if (pJsonObjA == DSL_NULL || pJsonObjB == DSL_NULL)
   {
      bDifferent = DSL_TRUE;
   }
   else if (json_object_get_type(pJsonObjA) != json_object_get_type(pJsonObjB))
   {
      bDifferent = DSL_TRUE;
   }
   else
   {
      switch(json_object_get_type(pJsonObjA))
      {
      case json_type_boolean:
         if (json_object_get_boolean(pJsonObjA) != json_object_get_boolean(pJsonObjB))
         {
            bDifferent = DSL_TRUE;
         }
         break;
      case json_type_double:
         if (json_object_get_double(pJsonObjA) != json_object_get_double(pJsonObjB))
         {
            bDifferent = DSL_TRUE;
         }
         break;
      case json_type_int:
         if (json_object_get_int(pJsonObjA) != json_object_get_int(pJsonObjB))
         {
            bDifferent = DSL_TRUE;
         }
         break;
      case json_type_string:
         ;
         const DSL_char_t *pStrA = json_object_get_string(pJsonObjA);
         const DSL_char_t *pStrB = json_object_get_string(pJsonObjB);
         DSL_size_t nLenA = DSL_ST_strnlen_s(pStrA, DSL_UBUS_JSON_ARR_STR_LEN_MAX);
         DSL_size_t nLenB = DSL_ST_strnlen_s(pStrB, DSL_UBUS_JSON_ARR_STR_LEN_MAX);

         if (nLenA != nLenB)
         {
            bDifferent = DSL_TRUE;
         }
         else if (strncmp(pStrA, pStrB, nLenA) != 0)
         {
            bDifferent = DSL_TRUE;
         }
         break;
      case json_type_array:
         ;
         struct array_list *pJsonArrayA = json_object_get_array(pJsonObjA);
         struct array_list *pJsonArrayB = json_object_get_array(pJsonObjB);

         if (pJsonArrayA->length != pJsonArrayB->length)
         {
            bDifferent = DSL_TRUE;
         }
         else if (pJsonArrayA->size != pJsonArrayB->size)
         {
            bDifferent = DSL_TRUE;
         }
         else
         {
            for (DSL_int32_t idx = 0;
               idx < pJsonArrayA->length && bDifferent == DSL_FALSE;
               ++idx)
            {
               struct json_object *pJsonObjC = json_object_array_get_idx(pJsonObjA, idx);
               struct json_object *pJsonObjD = json_object_array_get_idx(pJsonObjB, idx);

               bDifferent = DSL_ST_UbusJsonNodeCompare(pJsonObjC, pJsonObjD);
            }
         }
         break;
      case json_type_object:
         ;
         struct lh_table *plhA = json_object_get_object(pJsonObjA);
         struct lh_table *plhB = json_object_get_object(pJsonObjB);

         if (plhA->count != plhB->count)
         {
            bDifferent = DSL_TRUE;
         }
         else
         {
            struct json_object_iter iter = { 0 };
            struct json_object *sub = DSL_NULL;

            json_object_object_foreachC(pJsonObjA, iter)
            {
               if (!lh_table_lookup_ex(plhB, iter.key, (void **)&sub))
               {
                  bDifferent = DSL_TRUE;
               }
               else
               {
                  bDifferent = DSL_ST_UbusJsonNodeCompare(iter.val, sub);
               }

               if (bDifferent == DSL_TRUE)
               {
                  break;
               }
            }
         }
         break;
      default:
         break;
      }
   }

   return bDifferent;
}

#endif /* defined(INCLUDE_DSL_STATUS_UBUS_SUPPORT) && (INCLUDE_DSL_STATUS_UBUS_SUPPORT == 1) */
#endif /* defined(DSL_VRX_DEVICE_VRX518) */